@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -419,7 +419,7 @@ boxed_new( const BoxedDef *def )
 {
 	NABoxed *boxed;
 
-	boxed = g_object_new( NA_BOXED_TYPE, NULL );
+	boxed = g_object_new( NA_TYPE_BOXED, NULL );
 	boxed->private->def = def;
 
 	return( boxed );
@@ -1203,6 +1203,7 @@ string_list_from_void( NABoxed *boxed, const void *value )
 			boxed->private->u.string_list = g_slist_prepend( boxed->private->u.string_list, g_strdup(( const gchar * ) it->data ));
 		}
 	}
+	boxed->private->u.string_list = g_slist_reverse( boxed->private->u.string_list );
 }
 
 static gconstpointer
