@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -49,7 +49,9 @@
 #define SIZE_MIN		  1
 #define SIZE_MAX	1048576		/* 1 MB */
 
+#ifdef NA_ENABLE_DEPRECATED
 static GSList  *text_to_string_list( const gchar *text, const gchar *separator, const gchar *default_value );
+#endif
 static gboolean info_dir_is_writable( GFile *file, const gchar *path );
 static gboolean file_is_loadable( GFile *file );
 
@@ -70,6 +72,7 @@ na_core_utils_boolean_from_string( const gchar *string )
 	return( g_ascii_strcasecmp( string, "true" ) == 0 || atoi( string ) != 0 );
 }
 
+#ifdef NA_ENABLE_DEPRECATED
 /**
  * na_core_utils_str_add_prefix:
  * @prefix: the prefix to be prepended.
@@ -80,6 +83,7 @@ na_core_utils_boolean_from_string( const gchar *string )
  * Returns: a new string which should be g_free() by the caller.
  *
  * Since: 2.30
+ * Deprecated: 3.2
  */
 gchar *
 na_core_utils_str_add_prefix( const gchar *prefix, const gchar *str )
@@ -99,6 +103,32 @@ na_core_utils_str_add_prefix( const gchar *prefix, const gchar *str )
 	return( g_string_free( result, FALSE ));
 }
 
+/*
+ * split a text buffer in lines
+ */
+static GSList *
+text_to_string_list( const gchar *text, const gchar *separator, const gchar *default_value )
+{
+	GSList *strlist = NULL;
+	gchar **tokens;
+	gchar *tmp;
+	gchar *source = g_strdup( text );
+
+	tmp = g_strstrip( source );
+	if( !strlen( tmp ) && default_value ){
+		strlist = g_slist_append( strlist, g_strdup( default_value ));
+
+	} else {
+		tokens = g_strsplit( source, separator, -1 );
+		strlist = na_core_utils_slist_from_array(( const gchar ** ) tokens );
+		g_strfreev( tokens );
+	}
+
+	g_free( source );
+	return( strlist );
+}
+#endif /* NA_ENABLE_DEPRECATED */
+
 /**
  * na_core_utils_str_collate:
  * @str1: an UTF-8 encoded string.
@@ -265,11 +295,11 @@ na_core_utils_str_split_first_word( const gchar *string, gchar **first, gchar **
  * @key: the key string to be substituted.
  * @subst: the string which will replace @key.
  *
- * Returns: a copy of @pattern where the first occurrence of @key has been
+ * Returns:
+ * a copy of @pattern where the first occurrence of @key has been
  * substituted with @subst, as a newly allocated string which should be
- * g_free() by the caller.
- *
- * Returns: a copy of @pattern if @key is not found in @pattern.
+ * g_free() by the caller,
+ * or a copy of @pattern if @key is not found in @pattern.
  */
 gchar *
 na_core_utils_str_subst( const gchar *pattern, const gchar *key, const gchar *subst )
@@ -745,31 +775,6 @@ na_core_utils_gstring_joinv( const gchar *start, const gchar *separator, gchar *
 	return( g_string_free( tmp_string, FALSE ));
 }
 
-/*
- * split a text buffer in lines
- */
-static GSList *
-text_to_string_list( const gchar *text, const gchar *separator, const gchar *default_value )
-{
-	GSList *strlist = NULL;
-	gchar **tokens;
-	gchar *tmp;
-	gchar *source = g_strdup( text );
-
-	tmp = g_strstrip( source );
-	if( !strlen( tmp ) && default_value ){
-		strlist = g_slist_append( strlist, g_strdup( default_value ));
-
-	} else {
-		tokens = g_strsplit( source, separator, -1 );
-		strlist = na_core_utils_slist_from_array(( const gchar ** ) tokens );
-		g_strfreev( tokens );
-	}
-
-	g_free( source );
-	return( strlist );
-}
-
 /***
  * na_core_utils_selcount_get_ope:
  * @selcount: the selection count condition string.
@@ -1184,7 +1189,7 @@ na_core_utils_file_load_from_uri( const gchar *uri, gsize *length )
  * <programlisting>
  *   nautilus-actions-new (Nautilus-Actions) v 2.29.1
  *   Copyright (C) 2005-2007 Frederic Ruaudel
- *   Copyright (C) 2009, 2010, 2011 Pierre Wieser
+ *   Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser
  *   Nautilus-Actions is free software, licensed under GPLv2 or later.
  * </programlisting>
  *
