@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -175,7 +175,7 @@ register_type( void )
 
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( NA_BOXED_TYPE, "NADataBoxed", &info, 0 );
+	type = g_type_register_static( NA_TYPE_BOXED, "NADataBoxed", &info, 0 );
 
 	return( type );
 }
@@ -200,14 +200,10 @@ class_init( NADataBoxedClass *klass )
 static void
 instance_init( GTypeInstance *instance, gpointer klass )
 {
-	static const gchar *thisfn = "na_data_boxed_instance_init";
 	NADataBoxed *self;
 
 	g_return_if_fail( NA_IS_DATA_BOXED( instance ));
 
-	g_debug( "%s: instance=%p (%s), klass=%p",
-			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
-
 	self = NA_DATA_BOXED( instance );
 
 	self->private = g_new0( NADataBoxedPrivate, 1 );
@@ -240,16 +236,10 @@ instance_dispose( GObject *object )
 static void
 instance_finalize( GObject *object )
 {
-	static const gchar *thisfn = "na_data_boxed_instance_finalize";
 	NADataBoxed *self;
 
 	g_return_if_fail( NA_IS_DATA_BOXED( object ));
 
-	g_debug( "%s: object=%p (%s), name=%s",
-			thisfn,
-			( void * ) object, G_OBJECT_TYPE_NAME( object ),
-			NA_DATA_BOXED( object )->private->data_def->name );
-
 	self = NA_DATA_BOXED( object );
 
 	g_free( self->private );
@@ -291,7 +281,7 @@ na_data_boxed_new( const NADataDef *def )
 
 	g_return_val_if_fail( def != NULL, NULL );
 
-	boxed = g_object_new( NA_DATA_BOXED_TYPE, NULL );
+	boxed = g_object_new( NA_TYPE_DATA_BOXED, NULL );
 	na_boxed_set_type( NA_BOXED( boxed ), def->type );
 	boxed->private->data_def = def;
 	boxed->private->boxed_def = get_data_boxed_def( def->type );
@@ -378,7 +368,7 @@ na_data_boxed_get_param_spec( const NADataDef *def )
 	return( spec );
 }
 
-#ifndef NA_DISABLE_DEPRECATED
+#ifdef NA_ENABLE_DEPRECATED
 /**
  * na_data_boxed_are_equal:
  * @a: the first #NADataBoxed object.
@@ -397,7 +387,7 @@ na_data_boxed_are_equal( const NADataBoxed *a, const NADataBoxed *b )
 
 	return( na_boxed_are_equal( NA_BOXED( a ), NA_BOXED( b )));
 }
-#endif /* NA_DISABLE_DEPRECATED */
+#endif /* NA_ENABLE_DEPRECATED */
 
 /**
  * na_data_boxed_is_default:
@@ -454,7 +444,7 @@ na_data_boxed_is_valid( const NADataBoxed *boxed )
 	return( is_valid );
 }
 
-#ifndef NA_DISABLE_DEPRECATED
+#ifdef NA_ENABLE_DEPRECATED
 /**
  * na_data_boxed_dump:
  * @boxed: this #NADataBoxed object.
@@ -584,7 +574,7 @@ na_data_boxed_set_from_void( NADataBoxed *boxed, const void *value )
 {
 	na_boxed_set_from_void( NA_BOXED( boxed ), value );
 }
-#endif /* NA_DISABLE_DEPRECATED */
+#endif /* NA_ENABLE_DEPRECATED */
 
 static GParamSpec *
 bool_spec( const NADataDef *def )
