@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -42,9 +42,9 @@
 static void        dump_entry( GConfEntry *entry, void *user_data );
 static GConfValue *read_value( GConfClient *gconf, const gchar *path, gboolean use_schema, GConfValueType type );
 
-#ifndef NA_DISABLE_DEPRECATED
+#ifdef NA_ENABLE_DEPRECATED
 static gboolean    sync_gconf( GConfClient *gconf, gchar **message );
-#endif /* NA_DISABLE_DEPRECATED */
+#endif /* NA_ENABLE_DEPRECATED */
 
 /**
  * na_gconf_utils_get_subdirs:
@@ -527,7 +527,7 @@ na_gconf_utils_read_string_list( GConfClient *gconf, const gchar *path )
 	return( list_strings );
 }
 
-#ifndef NA_DISABLE_DEPRECATED
+#ifdef NA_ENABLE_DEPRECATED
 /**
  * na_gconf_utils_write_bool:
  * @gconf: a GConfClient instance.
@@ -793,7 +793,7 @@ na_gconf_utils_slist_to_string( GSList *slist )
 
 	return( g_string_free( str, FALSE ));
 }
-#endif /* NA_DISABLE_DEPRECATED */
+#endif /* NA_ENABLE_DEPRECATED */
 
 static GConfValue *
 read_value( GConfClient *gconf, const gchar *path, gboolean use_schema, GConfValueType type )
@@ -828,7 +828,7 @@ read_value( GConfClient *gconf, const gchar *path, gboolean use_schema, GConfVal
 	return( value );
 }
 
-#ifndef NA_DISABLE_DEPRECATED
+#ifdef NA_ENABLE_DEPRECATED
 static gboolean
 sync_gconf( GConfClient *gconf, gchar **message )
 {
@@ -848,6 +848,6 @@ sync_gconf( GConfClient *gconf, gchar **message )
 
 	return( ret );
 }
-#endif /* NA_DISABLE_DEPRECATED */
+#endif /* NA_ENABLE_DEPRECATED */
 
 #endif /* HAVE_GCONF */
