@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,7 +32,13 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_GDBUS
+#include <gio/gio.h>
+#else
+# ifdef HAVE_DBUS_GLIB
 #include <dbus/dbus-glib.h>
+# endif
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -56,8 +62,7 @@ struct _NAIContextInterfacePrivate {
 	void *empty;						/* so that gcc -pedantic is happy */
 };
 
-static gboolean     st_initialized     = FALSE;
-static gboolean     st_finalized       = FALSE;
+static guint st_initializations = 0;	/* interface initialization count */
 
 static GType        register_type( void );
 static void         interface_base_init( NAIContextInterface *klass );
@@ -143,14 +148,14 @@ interface_base_init( NAIContextInterface *klass )
 {
 	static const gchar *thisfn = "na_icontext_interface_base_init";
 
-	if( !st_initialized ){
+	if( !st_initializations ){
 
 		g_debug( "%s: klass%p (%s)", thisfn, ( void * ) klass, G_OBJECT_CLASS_NAME( klass ));
 
 		klass->private = g_new0( NAIContextInterfacePrivate, 1 );
-
-		st_initialized = TRUE;
 	}
+
+	st_initializations += 1;
 }
 
 static void
@@ -158,12 +163,12 @@ interface_base_finalize( NAIContextInterface *klass )
 {
 	static const gchar *thisfn = "na_icontext_interface_base_finalize";
 
-	if( st_initialized && !st_finalized ){
+	st_initializations -= 1;
+
+	if( !st_initializations ){
 
 		g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
-		st_finalized = TRUE;
-
 		g_free( klass->private );
 	}
 }
@@ -598,6 +603,9 @@ is_candidate_for_show_if_registered( const NAIContext *object, guint target, GLi
 
 	if( name && strlen( name )){
 		ok = FALSE;
+#ifdef HAVE_GDBUS
+#else
+# ifdef HAVE_DBUS_GLIB
 		GError *error = NULL;
 		DBusGConnection *connection = dbus_g_bus_get( DBUS_BUS_SESSION, &error );
 
@@ -612,6 +620,8 @@ is_candidate_for_show_if_registered( const NAIContext *object, guint target, GLi
 			ok = ( proxy != NULL );
 			dbus_g_connection_unref( connection );
 		}
+# endif
+#endif
 	}
 
 	if( !ok ){
