@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,16 +44,16 @@ struct _NAIImporterInterfacePrivate {
 	void *empty;						/* so that gcc -pedantic is happy */
 };
 
-gboolean iimporter_initialized = FALSE;
-gboolean iimporter_finalized   = FALSE;
+static guint st_initializations = 0;	/* interface initialization count */
 
-static GType  register_type( void );
-static void   interface_base_init( NAIImporterInterface *klass );
-static void   interface_base_finalize( NAIImporterInterface *klass );
+static GType register_type( void );
+static void  interface_base_init( NAIImporterInterface *klass );
+static void  interface_base_finalize( NAIImporterInterface *klass );
+static guint iimporter_get_version( const NAIImporter *instance );
 
-static guint  iimporter_get_version( const NAIImporter *instance );
-
-static void   renumber_label_item( NAIImporterManageImportModeParms *parms );
+#ifdef NA_ENABLE_DEPRECATED
+static void  renumber_label_item( NAIImporterManageImportModeParms *parms );
+#endif
 
 /**
  * na_iimporter_get_type:
@@ -109,7 +109,7 @@ interface_base_init( NAIImporterInterface *klass )
 {
 	static const gchar *thisfn = "na_iimporter_interface_base_init";
 
-	if( !iimporter_initialized ){
+	if( !st_initializations ){
 
 		g_debug( "%s: klass%p (%s)", thisfn, ( void * ) klass, G_OBJECT_CLASS_NAME( klass ));
 
@@ -117,9 +117,9 @@ interface_base_init( NAIImporterInterface *klass )
 
 		klass->get_version = iimporter_get_version;
 		klass->import_from_uri = NULL;
-
-		iimporter_initialized = TRUE;
 	}
+
+	st_initializations += 1;
 }
 
 static void
@@ -127,12 +127,12 @@ interface_base_finalize( NAIImporterInterface *klass )
 {
 	static const gchar *thisfn = "na_iimporter_interface_base_finalize";
 
-	if( iimporter_initialized && !iimporter_finalized ){
+	st_initializations -= 1;
+
+	if( !st_initializations ){
 
 		g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
-		iimporter_finalized = TRUE;
-
 		g_free( klass->private );
 	}
 }
@@ -146,42 +146,45 @@ iimporter_get_version( const NAIImporter *instance )
 /**
  * na_iimporter_import_from_uri:
  * @importer: this #NAIImporter instance.
- * @parms: a #NAIImporterImportFromUriParms structure.
+ * @parms: a #NAIImporterImportFromUriParmsv2 structure.
  *
  * Tries to import a #NAObjectItem from the URI specified in @parms, returning
  * the result in <structfield>@parms->imported</structfield>.
  *
+ * Note that, starting with &prodname; 3.2, the @parms argument is no more a
+ * #NAIImporterImportFromUriParms pointer, but a #NAIImporterImportFromUriParmsv2
+ * one.
+ *
  * Returns: the return code of the operation.
  *
  * Since: 2.30
  */
 
 guint
-na_iimporter_import_from_uri( const NAIImporter *importer, NAIImporterImportFromUriParms *parms )
+na_iimporter_import_from_uri( const NAIImporter *importer, NAIImporterImportFromUriParmsv2 *parms )
 {
 	static const gchar *thisfn = "na_iimporter_import_from_uri";
 	guint code;
 
 	g_return_val_if_fail( NA_IS_IIMPORTER( importer ), IMPORTER_CODE_PROGRAM_ERROR );
+	g_return_val_if_fail( parms && parms->version == 2, IMPORTER_CODE_PROGRAM_ERROR );
 
 	code = IMPORTER_CODE_NOT_WILLING_TO;
 
-	if( iimporter_initialized && !iimporter_finalized ){
+	g_debug( "%s: importer=%p (%s), parms=%p", thisfn,
+			( void * ) importer, G_OBJECT_TYPE_NAME( importer), ( void * ) parms );
 
-		g_debug( "%s: importer=%p (%s), parms=%p", thisfn,
-				( void * ) importer, G_OBJECT_TYPE_NAME( importer), ( void * ) parms );
-
-		if( NA_IIMPORTER_GET_INTERFACE( importer )->import_from_uri ){
-			code = NA_IIMPORTER_GET_INTERFACE( importer )->import_from_uri( importer, parms );
-		}
+	if( NA_IIMPORTER_GET_INTERFACE( importer )->import_from_uri ){
+		code = NA_IIMPORTER_GET_INTERFACE( importer )->import_from_uri( importer, parms );
 	}
 
 	return( code );
 }
 
+#ifdef NA_ENABLE_DEPRECATED
 /**
  * na_iimporter_manage_import_mode:
- * @parms: a NAIImporterManageImportModeParms struct.
+ * @parms: a #NAIImporterManageImportModeParms struct.
  *
  * Returns: the #NAIImporterImportStatus status of the operation:
  *
@@ -210,6 +213,7 @@ na_iimporter_import_from_uri( const NAIImporter *importer, NAIImporterImportFrom
  * </itemizedlist>
  *
  * Since: 2.30
+ * Deprecated: 3.2
  */
 guint
 na_iimporter_manage_import_mode( NAIImporterManageImportModeParms *parms )
@@ -312,3 +316,4 @@ renumber_label_item( NAIImporterManageImportModeParms *parms )
 	g_free( tmp );
 	g_free( label );
 }
+#endif /* NA_ENABLE_DEPRECATED */
