@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -89,6 +89,7 @@ static void          dump_providers_list( GList *providers );
 static NAIOProvider *io_provider_new( const NAPivot *pivot, NAIIOProvider *module, const gchar *id );
 static GList        *io_providers_list_add_from_plugins( const NAPivot *pivot, GList *list );
 static GList        *io_providers_list_add_from_prefs( const NAPivot *pivot, GList *objects_list );
+static GSList       *io_providers_get_from_prefs( void );
 static GList        *io_providers_list_add_from_write_order( const NAPivot *pivot, GList *objects_list );
 static GList        *io_providers_list_append_object( const NAPivot *pivot, GList *list, NAIIOProvider *module, const gchar *id );
 static void          io_providers_list_set_module( const NAPivot *pivot, NAIOProvider *provider_object, NAIIOProvider *provider_module );
@@ -194,20 +195,23 @@ static void
 instance_constructed( GObject *object )
 {
 	static const gchar *thisfn = "na_io_provider_instance_constructed";
-	NAIOProvider *self;
+	NAIOProviderPrivate *priv;
 
 	g_return_if_fail( NA_IS_IO_PROVIDER( object ));
 
-	self = NA_IO_PROVIDER( object );
+	priv = NA_IO_PROVIDER( object )->private;
 
-	if( !self->private->dispose_has_run ){
-
-		g_debug( "%s: object=%p, id=%s", thisfn, ( void * ) object, self->private->id );
+	if( !priv->dispose_has_run ){
 
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->constructed ){
 			G_OBJECT_CLASS( st_parent_class )->constructed( object );
 		}
+
+		g_debug( "%s: object=%p (%s), id=%s",
+				thisfn,
+				( void * ) object, G_OBJECT_TYPE_NAME( object ),
+				priv->id );
 	}
 }
 
@@ -636,7 +640,7 @@ na_io_provider_load_items( const NAPivot *pivot, guint loadable_set, GSList **me
 			thisfn, ( void * ) pivot, loadable_set, ( void * ) messages );
 
 	/* get the global flat items list, as a merge of the list provided
-	 * each available and readable i/o provider
+	 * by each available and readable i/o provider
 	 */
 	flat = load_items_get_merged_list( pivot, loadable_set, messages );
 
@@ -753,7 +757,7 @@ io_providers_list_add_from_plugins( const NAPivot *pivot, GList *objects_list )
 	NAIIOProvider *provider_module;
 
 	merged = objects_list;
-	modules_list = na_pivot_get_providers( pivot, NA_IIO_PROVIDER_TYPE );
+	modules_list = na_pivot_get_providers( pivot, NA_TYPE_IIO_PROVIDER );
 
 	for( im = modules_list ; im ; im = im->next ){
 
@@ -797,7 +801,7 @@ io_providers_list_add_from_prefs( const NAPivot *pivot, GList *objects_list )
 	GSList *io_providers, *it;
 
 	merged = objects_list;
-	io_providers = na_iprefs_get_io_providers();
+	io_providers = io_providers_get_from_prefs();
 
 	for( it = io_providers ; it ; it = it->next ){
 		id = ( const gchar * ) it->data;
@@ -809,6 +813,56 @@ io_providers_list_add_from_prefs( const NAPivot *pivot, GList *objects_list )
 	return( merged );
 }
 
+/*
+ * io_providers_get_from_prefs:
+ *
+ * Searches in preferences system for all mentions of an i/o provider.
+ * This does not mean in any way that the i/o provider is active,
+ * available or so, but just that is mentioned here.
+ *
+ * I/o provider identifiers returned in the list are not supposed
+ * to be unique, nor sorted.
+ *
+ * Returns: a list of i/o provider identifiers found in preferences
+ * system; this list should be na_core_utils_slist_free() by the caller.
+ *
+ * Since: 3.1
+ */
+static GSList *
+io_providers_get_from_prefs( void )
+{
+	GSList *providers;
+	GSList *write_order, *groups;
+	GSList *it;
+	const gchar *name;
+	gchar *group_prefix;
+	guint prefix_len;
+
+	providers = NULL;
+
+	write_order = na_settings_get_string_list( NA_IPREFS_IO_PROVIDERS_WRITE_ORDER, NULL, NULL );
+	for( it = write_order ; it ; it = it->next ){
+		name = ( const gchar * ) it->data;
+		providers = g_slist_prepend( providers, g_strdup( name ));
+	}
+	na_core_utils_slist_free( write_order );
+
+	groups = na_settings_get_groups();
+
+	group_prefix = g_strdup_printf( "%s ", NA_IPREFS_IO_PROVIDER_GROUP );
+	prefix_len = strlen( group_prefix );
+	for( it = groups ; it ; it = it->next ){
+		name = ( const gchar * ) it->data;
+		if( g_str_has_prefix( name, group_prefix )){
+			providers = g_slist_prepend( providers, g_strdup( name+prefix_len ));
+		}
+	}
+	g_free( group_prefix );
+	na_core_utils_slist_free( groups );
+
+	return( providers );
+}
+
 /*
  * adding from write-order means we only create NAIOProvider objects
  * without having any pointer to the underlying NAIIOProvider (if it exists)
@@ -1092,6 +1146,7 @@ load_items_hierarchy_build( GList **tree, GSList *level_zero, gboolean list_if_e
 			}
 		}
 	}
+
 	/* if level-zero list is empty,
 	 * we consider that all items are at the same level
 	 */
@@ -1196,6 +1251,10 @@ na_io_provider_write_item( const NAIOProvider *provider, const NAObjectItem *ite
 
 	ret = NA_IIO_PROVIDER_GET_INTERFACE( provider->private->provider )->write_item( provider->private->provider, item, messages );
 
+	if( ret == NA_IIO_PROVIDER_CODE_OK ){
+		na_object_set_provider( item, provider );
+	}
+
 	return( ret );
 }
 
