@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -76,14 +76,14 @@ static void         object_dump( const NAObject *object );
 static gboolean     object_are_equal( const NAObject *a, const NAObject *b );
 static gboolean     object_is_valid( const NAObject *object );
 
-static void         ifactory_object_iface_init( NAIFactoryObjectInterface *iface );
+static void         ifactory_object_iface_init( NAIFactoryObjectInterface *iface, void *user_data );
 static guint        ifactory_object_get_version( const NAIFactoryObject *instance );
 static NADataGroup *ifactory_object_get_groups( const NAIFactoryObject *instance );
 static void         ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider *reader, void *reader_data, GSList **messages );
 static guint        ifactory_object_write_start( NAIFactoryObject *instance, const NAIFactoryProvider *writer, void *writer_data, GSList **messages );
 static guint        ifactory_object_write_done( NAIFactoryObject *instance, const NAIFactoryProvider *writer, void *writer_data, GSList **messages );
 
-static void         icontext_iface_init( NAIContextInterface *iface );
+static void         icontext_iface_init( NAIContextInterface *iface, void *user_data );
 static gboolean     icontext_is_candidate( NAIContext *object, guint target, GList *selection );
 
 static NAObjectProfile *read_done_convert_v1_to_v2( NAIFactoryObject *instance );
@@ -139,11 +139,11 @@ register_type( void )
 
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( NA_OBJECT_ITEM_TYPE, "NAObjectAction", &info, 0 );
+	type = g_type_register_static( NA_TYPE_OBJECT_ITEM, "NAObjectAction", &info, 0 );
 
-	g_type_add_interface_static( type, NA_ICONTEXT_TYPE, &icontext_iface_info );
+	g_type_add_interface_static( type, NA_TYPE_ICONTEXT, &icontext_iface_info );
 
-	g_type_add_interface_static( type, NA_IFACTORY_OBJECT_TYPE, &ifactory_object_iface_info );
+	g_type_add_interface_static( type, NA_TYPE_IFACTORY_OBJECT, &ifactory_object_iface_info );
 
 	return( type );
 }
@@ -351,11 +351,11 @@ object_is_valid( const NAObject *object )
 }
 
 static void
-ifactory_object_iface_init( NAIFactoryObjectInterface *iface )
+ifactory_object_iface_init( NAIFactoryObjectInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "na_object_action_ifactory_object_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 
 	iface->get_version = ifactory_object_get_version;
 	iface->get_groups = ifactory_object_get_groups;
@@ -378,8 +378,8 @@ ifactory_object_get_groups( const NAIFactoryObject *instance )
 
 /*
  * at this time, we don't yet have read the profiles as this will be
- * triggered by ifactory_provider_read_done - we so just be able to deal with
- * action-specific properties (not check for profiles consistency)
+ * triggered by ifactory_provider_read_done - we so just are able to deal with
+ * action-specific properties (not to check for profiles consistency)
  */
 static void
 ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider *reader, void *reader_data, GSList **messages )
@@ -389,8 +389,6 @@ ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider
 
 	g_debug( "na_object_action_ifactory_object_read_done: instance=%p", ( void * ) instance );
 
-	na_factory_object_dump( instance );
-
 	na_object_item_deals_with_version( NA_OBJECT_ITEM( instance ));
 
 	/* should attach a new profile if we detect a pre-v2 action
@@ -437,11 +435,11 @@ ifactory_object_write_done( NAIFactoryObject *instance, const NAIFactoryProvider
 }
 
 static void
-icontext_iface_init( NAIContextInterface *iface )
+icontext_iface_init( NAIContextInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "na_object_action_icontext_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 
 	iface->is_candidate = icontext_is_candidate;
 }
@@ -609,7 +607,7 @@ na_object_action_new( void )
 {
 	NAObjectAction *action;
 
-	action = g_object_new( NA_OBJECT_ACTION_TYPE, NULL );
+	action = g_object_new( NA_TYPE_OBJECT_ACTION, NULL );
 
 	return( action );
 }
@@ -631,7 +629,7 @@ na_object_action_new_with_profile( void )
 
 	action = na_object_action_new();
 	profile = na_object_profile_new();
-	na_object_action_attach_profile( action, profile );
+	na_object_attach_profile( action, profile );
 
 	return( action );
 }
@@ -659,7 +657,7 @@ na_object_action_new_with_defaults( void )
 	na_factory_object_set_defaults( NA_IFACTORY_OBJECT( action ));
 
 	profile = na_object_profile_new_with_defaults();
-	na_object_action_attach_profile( action, profile );
+	na_object_attach_profile( action, profile );
 
 	return( action );
 }
