@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,6 +38,14 @@
 #include <api/na-data-def.h>
 #include <api/na-data-types.h>
 
+/*
+ * As of 3.2 non copyables data are:
+ * - the type
+ * - non localized icon name
+ * - subitems pointers list and string list
+ * - read-only status
+ */
+
 NADataDef data_def_item [] = {
 
 	/* this data is marked as non readable as it has to be read specifically
@@ -320,7 +328,7 @@ NADataDef data_def_item [] = {
 				NA_DATA_TYPE_BOOLEAN,
 				"false",
 				FALSE,
-				TRUE,
+				FALSE,
 				FALSE,
 				FALSE,
 				FALSE,
