@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -69,7 +69,7 @@ static void     instance_dispose( GObject *object );
 static void     instance_finalize( GObject *object );
 
 static void     object_dump( const NAObject *object );
-static void     object_copy( NAObject*target, const NAObject *source, gboolean recursive );
+static void     object_copy( NAObject*target, const NAObject *source, guint mode );
 static gboolean object_are_equal( const NAObject *a, const NAObject *b );
 static gboolean object_is_valid( const NAObject *object );
 
@@ -77,7 +77,7 @@ static gchar   *object_id_new_id( const NAObjectId *item, const NAObjectId *new_
 
 static void     count_items_rec( GList *items, gint *menus, gint *actions, gint *profiles, gboolean recurse );
 static GSList  *get_children_slist( const NAObjectItem *item );
-static void     copy_children( NAObjectItem *target, const NAObjectItem *source );
+static void     copy_children( NAObjectItem *target, const NAObjectItem *source, guint mode );
 
 GType
 na_object_item_get_type( void )
@@ -111,7 +111,7 @@ register_type( void )
 
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( NA_OBJECT_ID_TYPE, "NAObjectItem", &info, 0 );
+	type = g_type_register_static( NA_TYPE_OBJECT_ID, "NAObjectItem", &info, 0 );
 
 	return( type );
 }
@@ -225,7 +225,7 @@ object_dump( const NAObject *object )
 }
 
 static void
-object_copy( NAObject *target, const NAObject *source, gboolean recursive )
+object_copy( NAObject *target, const NAObject *source, guint mode )
 {
 	static const gchar *thisfn = "na_object_item_object_copy";
 	void *provider;
@@ -239,8 +239,10 @@ object_copy( NAObject *target, const NAObject *source, gboolean recursive )
 
 	if( !dest->private->dispose_has_run && !src->private->dispose_has_run ){
 
-		if( recursive ){
-			copy_children( dest, src );
+		if( mode == DUPLICATE_REC ||
+			( mode == DUPLICATE_OBJECT && G_OBJECT_TYPE( source ) == NA_TYPE_OBJECT_ACTION )){
+
+				copy_children( dest, src, mode );
 		}
 
 		provider = na_object_get_provider( source );
@@ -262,7 +264,7 @@ object_copy( NAObject *target, const NAObject *source, gboolean recursive )
 
 		/* chain up to the parent class */
 		if( NA_OBJECT_CLASS( st_parent_class )->copy ){
-			NA_OBJECT_CLASS( st_parent_class )->copy( target, source, recursive );
+			NA_OBJECT_CLASS( st_parent_class )->copy( target, source, mode );
 		}
 	}
 }
@@ -717,8 +719,16 @@ count_items_rec( GList *items, gint *menus, gint *actions, gint *profiles, gbool
 GList *
 na_object_item_copyref_items( GList *items )
 {
-	GList *copy = g_list_copy( items );
-	g_list_foreach( copy, ( GFunc ) na_object_object_ref, NULL );
+	GList *copy = NULL;
+
+	if( items ){
+		copy = g_list_copy( items );
+		g_list_foreach( copy, ( GFunc ) na_object_object_ref, NULL );
+		g_debug( "na_object_item_copyref_items: list at %p contains %s at %p (ref_count=%d)",
+				( void * ) copy,
+				G_OBJECT_TYPE_NAME( copy->data ), ( void * ) copy->data, G_OBJECT( copy->data )->ref_count );
+	}
+
 	return( copy );
 }
 
@@ -735,8 +745,13 @@ na_object_item_copyref_items( GList *items )
 GList *
 na_object_item_free_items( GList *items )
 {
-	g_list_foreach( items, ( GFunc ) na_object_object_unref, NULL );
-	g_list_free( items );
+	if( items ){
+		g_debug( "na_object_item_free_items: freeing list at %p which contains %s at %p (ref_count=%d)",
+				( void * ) items,
+				G_OBJECT_TYPE_NAME( items->data ), ( void * ) items->data, G_OBJECT( items->data )->ref_count );
+		g_list_foreach( items, ( GFunc ) na_object_object_unref, NULL );
+		g_list_free( items );
+	}
 	return( NULL );
 }
 
@@ -825,8 +840,12 @@ get_children_slist( const NAObjectItem *item )
 	return( g_slist_reverse( slist ));
 }
 
+/*
+ * only copy children if mode is 'full recursive'
+ * or mode is 'duplicate object' and this is an action with profiles
+ */
 static void
-copy_children( NAObjectItem *target, const NAObjectItem *source )
+copy_children( NAObjectItem *target, const NAObjectItem *source, guint mode )
 {
 	static const gchar *thisfn = "na_object_item_copy_children";
 	GList *tgt_children, *src_children, *ic;
@@ -835,14 +854,15 @@ copy_children( NAObjectItem *target, const NAObjectItem *source )
 	tgt_children = na_object_get_items( target );
 	if( tgt_children ){
 		g_warning( "%s: target_children=%p (count=%d)",
-				thisfn, ( void * ) tgt_children, g_list_length( tgt_children ));
+				thisfn,
+				( void * ) tgt_children, g_list_length( tgt_children ));
 		g_return_if_fail( tgt_children == NULL );
 	}
 
 	src_children = na_object_get_items( source );
 	for( ic = src_children ; ic ; ic = ic->next ){
 
-		dup = ( NAObject * ) na_object_duplicate( ic->data );
+		dup = ( NAObject * ) na_object_duplicate( ic->data, mode );
 		na_object_set_parent( dup, target );
 		tgt_children = g_list_prepend( tgt_children, dup );
 	}
