@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -80,13 +80,13 @@ static void         instance_finalize( GObject *object );
 static void         object_dump( const NAObject *object );
 static gboolean     object_is_valid( const NAObject *object );
 
-static void         ifactory_object_iface_init( NAIFactoryObjectInterface *iface );
+static void         ifactory_object_iface_init( NAIFactoryObjectInterface *iface, void *user_data );
 static guint        ifactory_object_get_version( const NAIFactoryObject *instance );
 static NADataGroup *ifactory_object_get_groups( const NAIFactoryObject *instance );
 static void         ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider *reader, void *reader_data, GSList **messages );
 static guint        ifactory_object_write_done( NAIFactoryObject *instance, const NAIFactoryProvider *writer, void *writer_data, GSList **messages );
 
-static void         icontext_iface_init( NAIContextInterface *iface );
+static void         icontext_iface_init( NAIContextInterface *iface, void *user_data );
 static gboolean     icontext_is_candidate( NAIContext *object, guint target, GList *selection );
 
 static gboolean     convert_pre_v3_parameters( NAObjectProfile *profile );
@@ -143,11 +143,11 @@ register_type( void )
 
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( NA_OBJECT_ID_TYPE, "NAObjectProfile", &info, 0 );
+	type = g_type_register_static( NA_TYPE_OBJECT_ID, "NAObjectProfile", &info, 0 );
 
-	g_type_add_interface_static( type, NA_ICONTEXT_TYPE, &icontext_iface_info );
+	g_type_add_interface_static( type, NA_TYPE_ICONTEXT, &icontext_iface_info );
 
-	g_type_add_interface_static( type, NA_IFACTORY_OBJECT_TYPE, &ifactory_object_iface_info );
+	g_type_add_interface_static( type, NA_TYPE_IFACTORY_OBJECT, &ifactory_object_iface_info );
 
 	return( type );
 }
@@ -317,11 +317,11 @@ object_is_valid( const NAObject *object )
 }
 
 static void
-ifactory_object_iface_init( NAIFactoryObjectInterface *iface )
+ifactory_object_iface_init( NAIFactoryObjectInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "na_object_profile_ifactory_object_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 
 	iface->get_version = ifactory_object_get_version;
 	iface->get_groups = ifactory_object_get_groups;
@@ -350,8 +350,6 @@ ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider
 
 	g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
 
-	na_factory_object_dump( instance );
-
 	/* converts pre-v3 data
 	 */
 	action = NA_OBJECT_ACTION( na_object_get_parent( instance ));
@@ -375,11 +373,11 @@ ifactory_object_write_done( NAIFactoryObject *instance, const NAIFactoryProvider
 }
 
 static void
-icontext_iface_init( NAIContextInterface *iface )
+icontext_iface_init( NAIContextInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "na_object_profile_icontext_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 
 	iface->is_candidate = icontext_is_candidate;
 }
@@ -748,7 +746,7 @@ na_object_profile_new( void )
 {
 	NAObjectProfile *profile;
 
-	profile = g_object_new( NA_OBJECT_PROFILE_TYPE, NULL );
+	profile = g_object_new( NA_TYPE_OBJECT_PROFILE, NULL );
 
 	return( profile );
 }
