@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -58,14 +58,14 @@ static void     instance_finalize( GObject *object );
 
 static void     object_dump( const NAObject *object );
 
-static void     iduplicable_iface_init( NAIDuplicableInterface *iface );
-static void     iduplicable_copy( NAIDuplicable *target, const NAIDuplicable *source );
+static void     iduplicable_iface_init( NAIDuplicableInterface *iface, void *user_data );
+static void     iduplicable_copy( NAIDuplicable *target, const NAIDuplicable *source, guint mode );
 static gboolean iduplicable_are_equal( const NAIDuplicable *a, const NAIDuplicable *b );
 static gboolean iduplicable_is_valid( const NAIDuplicable *object );
 
 static void     check_status_down_rec( const NAObject *object );
 static void     check_status_up_rec( const NAObject *object, gboolean was_modified, gboolean was_valid );
-static void     v_copy( NAObject *target, const NAObject *source, gboolean recursive );
+static void     v_copy( NAObject *target, const NAObject *source, guint mode );
 static gboolean v_are_equal( const NAObject *a, const NAObject *b );
 static gboolean v_is_valid( const NAObject *a );
 static void     dump_tree( GList *tree, gint level );
@@ -110,7 +110,7 @@ register_type( void )
 
 	type = g_type_register_static( G_TYPE_OBJECT, "NAObject", &info, 0 );
 
-	g_type_add_interface_static( type, NA_IDUPLICABLE_TYPE, &iduplicable_iface_info );
+	g_type_add_interface_static( type, NA_TYPE_IDUPLICABLE, &iduplicable_iface_info );
 
 	return( type );
 }
@@ -142,23 +142,18 @@ class_init( NAObjectClass *klass )
 static void
 instance_init( GTypeInstance *instance, gpointer klass )
 {
-	static const gchar *thisfn = "na_object_instance_init";
 	NAObject *self;
 
 	g_return_if_fail( NA_IS_OBJECT( instance ));
 
 	self = NA_OBJECT( instance );
 
-	g_debug( "%s: instance=%p (%s), klass=%p",
-			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
-
 	self->private = g_new0( NAObjectPrivate, 1 );
 }
 
 static void
 instance_dispose( GObject *object )
 {
-	static const gchar *thisfn = "na_object_instance_dispose";
 	NAObject *self;
 
 	g_return_if_fail( NA_IS_OBJECT( object ));
@@ -166,7 +161,6 @@ instance_dispose( GObject *object )
 	self = NA_OBJECT( object );
 
 	if( !self->private->dispose_has_run ){
-		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
 
 		self->private->dispose_has_run = TRUE;
 
@@ -182,15 +176,12 @@ instance_dispose( GObject *object )
 static void
 instance_finalize( GObject *object )
 {
-	static const gchar *thisfn = "na_object_instance_finalize";
 	NAObject *self;
 
 	g_return_if_fail( NA_IS_OBJECT( object ));
 
 	self = NA_OBJECT( object );
 
-	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 	g_free( self->private );
 
 	if( NA_IS_IFACTORY_OBJECT( object )){
@@ -217,11 +208,11 @@ object_dump( const NAObject *object )
 }
 
 static void
-iduplicable_iface_init( NAIDuplicableInterface *iface )
+iduplicable_iface_init( NAIDuplicableInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "na_object_iduplicable_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 
 	iface->copy = iduplicable_copy;
 	iface->are_equal = iduplicable_are_equal;
@@ -233,7 +224,7 @@ iduplicable_iface_init( NAIDuplicableInterface *iface )
  * it recursively copies @source to @target
  */
 static void
-iduplicable_copy( NAIDuplicable *target, const NAIDuplicable *source )
+iduplicable_copy( NAIDuplicable *target, const NAIDuplicable *source, guint mode )
 {
 	static const gchar *thisfn = "na_object_iduplicable_copy";
 	NAObject *dest, *src;
@@ -247,8 +238,11 @@ iduplicable_copy( NAIDuplicable *target, const NAIDuplicable *source )
 	if( !dest->private->dispose_has_run &&
 		!src->private->dispose_has_run ){
 
-		g_debug( "%s: target=%p (%s), source=%p (%s)",
-				thisfn, ( void * ) dest, G_OBJECT_TYPE_NAME( dest ), ( void * ) src, G_OBJECT_TYPE_NAME( src ));
+		g_debug( "%s: target=%p (%s), source=%p (%s), mode=%d",
+				thisfn,
+				( void * ) dest, G_OBJECT_TYPE_NAME( dest ),
+				( void * ) src, G_OBJECT_TYPE_NAME( src ),
+				mode );
 
 		if( NA_IS_IFACTORY_OBJECT( target )){
 			na_factory_object_copy( NA_IFACTORY_OBJECT( target ), NA_IFACTORY_OBJECT( source ));
@@ -258,7 +252,7 @@ iduplicable_copy( NAIDuplicable *target, const NAIDuplicable *source )
 			na_icontext_copy( NA_ICONTEXT( target ), NA_ICONTEXT( source ));
 		}
 
-		v_copy( dest, src, TRUE );
+		v_copy( dest, src, mode );
 	}
 }
 
@@ -440,10 +434,10 @@ check_status_up_rec( const NAObject *object, gboolean was_modified, gboolean was
 }
 
 static void
-v_copy( NAObject *target, const NAObject *source, gboolean recursive )
+v_copy( NAObject *target, const NAObject *source, guint mode )
 {
 	if( NA_OBJECT_GET_CLASS( target )->copy ){
-		NA_OBJECT_GET_CLASS( target )->copy( target, source, recursive );
+		NA_OBJECT_GET_CLASS( target )->copy( target, source, mode );
 	}
 }
 
@@ -664,7 +658,7 @@ na_object_object_unref( NAObject *object )
 	}
 }
 
-#ifndef NA_DISABLE_DEPRECATED
+#ifdef NA_ENABLE_DEPRECATED
 /*
  * build the class hierarchy
  * returns a list of GObjectClass, which starts with NAObject,
@@ -679,7 +673,7 @@ build_class_hierarchy( const NAObject *object )
 	hierarchy = NULL;
 	class = G_OBJECT_GET_CLASS( object );
 
-	while( G_OBJECT_CLASS_TYPE( class ) != NA_OBJECT_TYPE ){
+	while( G_OBJECT_CLASS_TYPE( class ) != NA_TYPE_OBJECT ){
 
 		hierarchy = g_list_prepend( hierarchy, class );
 		class = g_type_class_peek_parent( class );
@@ -732,7 +726,7 @@ na_object_free_hierarchy( GList *hierarchy )
 {
 	g_list_free( hierarchy );
 }
-#endif /* NA_DISABLE_DEPRECATED */
+#endif /* NA_ENABLE_DEPRECATED */
 
 /**
  * na_object_object_debug_invalid:
