@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,7 +38,6 @@
 #include <api/na-timeout.h>
 
 #include "na-io-provider.h"
-#include "na-iprefs.h"
 #include "na-module.h"
 #include "na-pivot.h"
 
@@ -186,7 +185,7 @@ class_init( NAPivotClass *klass )
 	 */
 	st_signals[ ITEMS_CHANGED ] = g_signal_new(
 				PIVOT_SIGNAL_ITEMS_CHANGED,
-				NA_PIVOT_TYPE,
+				NA_TYPE_PIVOT,
 				G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 				0,									/* class offset */
 				NULL,								/* accumulator */
@@ -228,26 +227,27 @@ static void
 instance_constructed( GObject *object )
 {
 	static const gchar *thisfn = "na_pivot_instance_constructed";
-	NAPivot *self;
+	NAPivotPrivate *priv;
 
 	g_return_if_fail( NA_IS_PIVOT( object ));
-	self = NA_PIVOT( object );
 
-	if( !self->private->dispose_has_run ){
+	priv = NA_PIVOT( object )->private;
 
-		g_debug( "%s: object=%p", thisfn, ( void * ) object );
-
-		self->private->modules = na_module_load_modules();
-
-		/* force class initialization and io-factory registration
-		 */
-		g_object_unref( na_object_action_new_with_profile());
-		g_object_unref( na_object_menu_new());
+	if( !priv->dispose_has_run ){
 
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->constructed ){
 			G_OBJECT_CLASS( st_parent_class )->constructed( object );
 		}
+
+		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
+		priv->modules = na_module_load_modules();
+
+		/* force class initialization and io-factory registration
+		 */
+		g_object_unref( na_object_action_new_with_profile());
+		g_object_unref( na_object_menu_new());
 	}
 }
 
@@ -391,7 +391,7 @@ na_pivot_new( void )
 
 	g_debug( "%s", thisfn );
 
-	pivot = g_object_new( NA_PIVOT_TYPE, NULL );
+	pivot = g_object_new( NA_TYPE_PIVOT, NULL );
 
 	return( pivot );
 }
@@ -426,7 +426,7 @@ na_pivot_dump( const NAPivot *pivot )
  * na_pivot_get_providers:
  * @pivot: this #NAPivot instance.
  * @type: the type of searched interface.
- * For now, we only have NA_IIO_PROVIDER_TYPE interfaces.
+ * For now, we only have NA_TYPE_IIO_PROVIDER interfaces.
  *
  * Returns: a newly allocated list of providers of the required interface.
  *
