@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -147,19 +147,20 @@ static const KeyDef st_def_keys[] = {
 	{ NA_IPREFS_TRY_EXEC_WSP,                     GROUP_NACT,    NA_DATA_TYPE_UINT_LIST,   "" },
 	{ NA_IPREFS_TRY_EXEC_URI,                     GROUP_NACT,    NA_DATA_TYPE_STRING,      "file:///bin" },
 	{ NA_IPREFS_EXPORT_ASK_USER_WSP,              GROUP_NACT,    NA_DATA_TYPE_UINT_LIST,   "" },
-	{ NA_IPREFS_EXPORT_ASK_USER_LAST_FORMAT,      GROUP_NACT,    NA_DATA_TYPE_STRING,      NA_IPREFS_DEFAULT_EXPORT_FORMAT },
+	{ NA_IPREFS_EXPORT_ASK_USER_LAST_FORMAT,      GROUP_NACT,    NA_DATA_TYPE_STRING,      "Desktop1" },
 	{ NA_IPREFS_EXPORT_ASK_USER_KEEP_LAST_CHOICE, GROUP_NACT,    NA_DATA_TYPE_BOOLEAN,     "false" },
 	{ NA_IPREFS_EXPORT_ASSISTANT_WSP,             GROUP_NACT,    NA_DATA_TYPE_UINT_LIST,   "" },
 	{ NA_IPREFS_EXPORT_ASSISTANT_URI,             GROUP_NACT,    NA_DATA_TYPE_STRING,      "file:///tmp" },
-	{ NA_IPREFS_EXPORT_PREFERRED_FORMAT,          GROUP_NACT,    NA_DATA_TYPE_STRING,      NA_IPREFS_DEFAULT_EXPORT_FORMAT },
+	{ NA_IPREFS_EXPORT_ASSISTANT_PANED,           GROUP_NACT,    NA_DATA_TYPE_UINT,        "200" },
+	{ NA_IPREFS_EXPORT_PREFERRED_FORMAT,          GROUP_NACT,    NA_DATA_TYPE_STRING,      "Ask" },
 	{ NA_IPREFS_FOLDER_CHOOSER_WSP,               GROUP_NACT,    NA_DATA_TYPE_UINT_LIST,   "" },
 	{ NA_IPREFS_FOLDER_CHOOSER_URI,               GROUP_NACT,    NA_DATA_TYPE_STRING,      "file:///" },
 	{ NA_IPREFS_IMPORT_ASK_USER_WSP,              GROUP_NACT,    NA_DATA_TYPE_UINT_LIST,   "" },
-	{ NA_IPREFS_IMPORT_ASK_USER_LAST_MODE,        GROUP_NACT,    NA_DATA_TYPE_STRING,      NA_IPREFS_DEFAULT_IMPORT_MODE },
+	{ NA_IPREFS_IMPORT_ASK_USER_LAST_MODE,        GROUP_NACT,    NA_DATA_TYPE_STRING,      "NoImport" },
 	{ NA_IPREFS_IMPORT_ASSISTANT_WSP,             GROUP_NACT,    NA_DATA_TYPE_UINT_LIST,   "" },
 	{ NA_IPREFS_IMPORT_ASSISTANT_URI,             GROUP_NACT,    NA_DATA_TYPE_STRING,      "file:///tmp" },
 	{ NA_IPREFS_IMPORT_ASK_USER_KEEP_LAST_CHOICE, GROUP_NACT,    NA_DATA_TYPE_BOOLEAN,     "false" },
-	{ NA_IPREFS_IMPORT_PREFERRED_MODE,            GROUP_NACT,    NA_DATA_TYPE_STRING,      NA_IPREFS_DEFAULT_IMPORT_MODE },
+	{ NA_IPREFS_IMPORT_PREFERRED_MODE,            GROUP_NACT,    NA_DATA_TYPE_STRING,      "Ask" },
 	{ NA_IPREFS_IO_PROVIDERS_WRITE_ORDER,         GROUP_NACT,    NA_DATA_TYPE_STRING_LIST, "" },
 	{ NA_IPREFS_ICON_CHOOSER_URI,                 GROUP_NACT,    NA_DATA_TYPE_STRING,      "file:///" },
 	{ NA_IPREFS_ICON_CHOOSER_PANED,               GROUP_NACT,    NA_DATA_TYPE_UINT,        "200" },
@@ -167,10 +168,11 @@ static const KeyDef st_def_keys[] = {
 	{ NA_IPREFS_ITEMS_ADD_ABOUT_ITEM,             GROUP_RUNTIME, NA_DATA_TYPE_BOOLEAN,     "true" },
 	{ NA_IPREFS_ITEMS_CREATE_ROOT_MENU,           GROUP_RUNTIME, NA_DATA_TYPE_BOOLEAN,     "true" },
 	{ NA_IPREFS_ITEMS_LEVEL_ZERO_ORDER,           GROUP_RUNTIME, NA_DATA_TYPE_STRING_LIST, "" },
-	{ NA_IPREFS_ITEMS_LIST_ORDER_MODE,            GROUP_RUNTIME, NA_DATA_TYPE_STRING,      NA_IPREFS_DEFAULT_LIST_ORDER_MODE },
+	{ NA_IPREFS_ITEMS_LIST_ORDER_MODE,            GROUP_RUNTIME, NA_DATA_TYPE_STRING,      "AscendingOrder" },
 	{ NA_IPREFS_MAIN_PANED,                       GROUP_NACT,    NA_DATA_TYPE_UINT,        "200" },
 	{ NA_IPREFS_MAIN_SAVE_AUTO,                   GROUP_NACT,    NA_DATA_TYPE_BOOLEAN,     "false" },
 	{ NA_IPREFS_MAIN_SAVE_PERIOD,                 GROUP_NACT,    NA_DATA_TYPE_UINT,        "5" },
+	{ NA_IPREFS_MAIN_TABS_POS,                    GROUP_NACT,    NA_DATA_TYPE_STRING,      "Top" },
 	{ NA_IPREFS_MAIN_TOOLBAR_EDIT_DISPLAY,        GROUP_NACT,    NA_DATA_TYPE_BOOLEAN,     "true" },
 	{ NA_IPREFS_MAIN_TOOLBAR_FILE_DISPLAY,        GROUP_NACT,    NA_DATA_TYPE_BOOLEAN,     "true" },
 	{ NA_IPREFS_MAIN_TOOLBAR_HELP_DISPLAY,        GROUP_NACT,    NA_DATA_TYPE_BOOLEAN,     "true" },
@@ -778,6 +780,24 @@ na_settings_set_string( const gchar *key, const gchar *value )
 	return( set_key_value( NULL, key, value ));
 }
 
+/**
+ * na_settings_set_string_ex:
+ * @group: the group in the keyed file;
+ * @key: the key whose value is to be returned.
+ * @value: the string to be written.
+ *
+ * This function writes @value as a user preference.
+ *
+ * Returns: %TRUE is the writing has been successful, %FALSE else.
+ *
+ * Since: 3.2
+ */
+gboolean
+na_settings_set_string_ex( const gchar *group, const gchar *key, const gchar *value )
+{
+	return( set_key_value( group, key, value ));
+}
+
 /**
  * na_settings_set_string_list:
  * @key: the key whose value is to be returned.
@@ -809,6 +829,31 @@ na_settings_set_string_list( const gchar *key, const GSList *value )
 	return( ok );
 }
 
+/**
+ * na_settings_set_int_ex:
+ * @group: the group in the keyed file;
+ * @key: the key whose value is to be returned.
+ * @value: the unsigned integer to be written.
+ *
+ * This function writes @value as a user preference.
+ *
+ * Returns: %TRUE is the writing has been successful, %FALSE else.
+ *
+ * Since: 3.2
+ */
+gboolean
+na_settings_set_int_ex( const gchar *group, const gchar *key, int value )
+{
+	gchar *string;
+	gboolean ok;
+
+	string = g_strdup_printf( "%d", value );
+	ok = set_key_value( group, key, string );
+	g_free( string );
+
+	return( ok );
+}
+
 /**
  * na_settings_set_uint:
  * @key: the key whose value is to be returned.
