@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -203,7 +203,7 @@ nadp_desktop_file_new( void )
 {
 	NadpDesktopFile *ndf;
 
-	ndf = g_object_new( NADP_DESKTOP_FILE_TYPE, NULL );
+	ndf = g_object_new( NADP_TYPE_DESKTOP_FILE, NULL );
 
 	return( ndf );
 }
@@ -402,7 +402,7 @@ ndf_new( const gchar *uri )
 {
 	NadpDesktopFile *ndf;
 
-	ndf = g_object_new( NADP_DESKTOP_FILE_TYPE, NULL );
+	ndf = g_object_new( NADP_TYPE_DESKTOP_FILE, NULL );
 
 	ndf->private->id = uri2id( uri );
 	ndf->private->uri = g_strdup( uri );
