@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -71,7 +71,11 @@ static void            desktop_weak_notify( NadpDesktopFile *ndf, GObject *item
 static void            write_start_write_type( NadpDesktopFile *ndp, NAObjectItem *item );
 static void            write_done_write_subitems_list( NadpDesktopFile *ndp, NAObjectItem *item );
 
-static ExportFormatFn *find_export_format_fn( GQuark format );
+static ExportFormatFn *find_export_format_fn( const gchar *format );
+
+#ifdef NA_ENABLE_DEPRECATED
+static ExportFormatFn *find_export_format_fn_from_quark( GQuark format );
+#endif
 
 /*
  * This is implementation of NAIIOProvider::is_willing_to_write method
@@ -344,12 +348,12 @@ nadp_iio_provider_duplicate_data( const NAIIOProvider *provider, NAObjectItem *d
 /**
  * nadp_writer_iexporter_export_to_buffer:
  * @instance: this #NAIExporter instance.
- * @parms: a #NAIExporterBufferParms structure.
+ * @parms: a #NAIExporterBufferParmsv2 structure.
  *
  * Export the specified 'item' to a newly allocated buffer.
  */
 guint
-nadp_writer_iexporter_export_to_buffer( const NAIExporter *instance, NAIExporterBufferParms *parms )
+nadp_writer_iexporter_export_to_buffer( const NAIExporter *instance, NAIExporterBufferParmsv2 *parms )
 {
 	static const gchar *thisfn = "nadp_writer_iexporter_export_to_buffer";
 	guint code, write_code;
@@ -367,7 +371,16 @@ nadp_writer_iexporter_export_to_buffer( const NAIExporter *instance, NAIExporter
 	}
 
 	if( code == NA_IEXPORTER_CODE_OK ){
+
+#ifdef NA_ENABLE_DEPRECATED
+		if( parms->version == 1 ){
+			fmt = find_export_format_fn_from_quark((( NAIExporterBufferParms * ) parms )->format );
+		} else {
+			fmt = find_export_format_fn( parms->format );
+		}
+#else
 		fmt = find_export_format_fn( parms->format );
+#endif
 
 		if( !fmt ){
 			code = NA_IEXPORTER_CODE_INVALID_FORMAT;
@@ -395,12 +408,12 @@ nadp_writer_iexporter_export_to_buffer( const NAIExporter *instance, NAIExporter
 /**
  * nadp_writer_iexporter_export_to_file:
  * @instance: this #NAIExporter instance.
- * @parms: a #NAIExporterFileParms structure.
+ * @parms: a #NAIExporterFileParmsv2 structure.
  *
  * Export the specified 'item' to a newly created file.
  */
 guint
-nadp_writer_iexporter_export_to_file( const NAIExporter *instance, NAIExporterFileParms *parms )
+nadp_writer_iexporter_export_to_file( const NAIExporter *instance, NAIExporterFileParmsv2 *parms )
 {
 	static const gchar *thisfn = "nadp_writer_iexporter_export_to_file";
 	guint code, write_code;
@@ -418,7 +431,16 @@ nadp_writer_iexporter_export_to_file( const NAIExporter *instance, NAIExporterFi
 	}
 
 	if( code == NA_IEXPORTER_CODE_OK ){
+
+#ifdef NA_ENABLE_DEPRECATED
+		if( parms->version == 1 ){
+			fmt = find_export_format_fn_from_quark((( NAIExporterFileParms * ) parms )->format );
+		} else {
+			fmt = find_export_format_fn( parms->format );
+		}
+#else
 		fmt = find_export_format_fn( parms->format );
+#endif
 
 		if( !fmt ){
 			code = NA_IEXPORTER_CODE_INVALID_FORMAT;
@@ -614,7 +636,27 @@ write_done_write_subitems_list( NadpDesktopFile *ndp, NAObjectItem *item )
 }
 
 static ExportFormatFn *
-find_export_format_fn( GQuark format )
+find_export_format_fn( const gchar *format )
+{
+	ExportFormatFn *found;
+	ExportFormatFn *i;
+
+	found = NULL;
+	i = st_export_format_fn;
+
+	while( i->format && !found ){
+		if( !strcmp( i->format, format )){
+			found = i;
+		}
+		i++;
+	}
+
+	return( found );
+}
+
+#ifdef NA_ENABLE_DEPRECATED
+static ExportFormatFn *
+find_export_format_fn_from_quark( GQuark format )
 {
 	ExportFormatFn *found;
 	ExportFormatFn *i;
@@ -631,3 +673,4 @@ find_export_format_fn( GQuark format )
 
 	return( found );
 }
+#endif
