@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -74,25 +74,25 @@ typedef struct {
  * to import. We thus have one NAXMLReader object per import operation.
  */
 struct _NAXMLReaderPrivate {
-	gboolean                       dispose_has_run;
+	gboolean                         dispose_has_run;
 
 	/* data provided by the caller
 	 */
-	NAIImporter                   *importer;
-	NAIImporterImportFromUriParms *parms;
+	NAIImporter                     *importer;
+	NAIImporterImportFromUriParmsv2 *parms;
 
 	/* data dynamically set during the import operation
 	 */
-	gboolean                       type_found;
-	GList                         *nodes;
-	GList                         *dealt;
-	RootNodeStr                   *root_node_str;
-	gchar                         *item_id;
+	gboolean                         type_found;
+	GList                           *nodes;
+	GList                           *dealt;
+	RootNodeStr                     *root_node_str;
+	gchar                           *item_id;
 
 	/* following values are reset and reused while iterating on each
 	 * element nodes of the imported item (cf. reset_node_data())
 	 */
-	gboolean                       node_ok;
+	gboolean                         node_ok;
 };
 
 extern NAXMLKeyStr naxml_schema_key_schema_str[];
@@ -173,7 +173,6 @@ static gchar        *build_root_node_list( void );
 static gchar        *get_value_from_child_node( xmlNode *node, const gchar *child );
 static gchar        *get_value_from_child_child_node( xmlNode *node, const gchar *first, const gchar *second );
 static gboolean      is_profile_path( NAXMLReader *reader, xmlChar *text );
-static guint         manage_import_mode( NAXMLReader *reader );
 static void          reset_node_data( NAXMLReader *reader );
 static xmlNode      *search_for_child_node( xmlNode *node, const gchar *key );
 static int           strxcmp( const xmlChar *a, const char *b );
@@ -308,29 +307,32 @@ reader_new( void )
 /**
  * naxml_reader_import_uri:
  * @instance: the #NAIImporter provider.
- * @parms: a #NAIImporterUriParms structure.
+ * @parms: a #NAIImporterImportFromUriParmsv2 structure.
  *
  * Imports an item.
  *
  * Returns: the import operation code.
  *
- * If we not found at least a well-formed XML document with a known root node,
+ * If we do not found at least a well-formed XML document with a known root node,
  *  then we do not return any error message at all, but just the 'unwilling to'
  *  code.
+ *
+ * Starting with N-A 3.2, we only honor the version 2 of #NAIImporter interface,
+ * thus no more checking here against possible duplicate identifiers.
  */
 guint
-naxml_reader_import_from_uri( const NAIImporter *instance, NAIImporterImportFromUriParms *parms )
+naxml_reader_import_from_uri( const NAIImporter *instance, void *parms_ptr )
 {
 	static const gchar *thisfn = "naxml_reader_import_from_uri";
 	NAXMLReader *reader;
+	NAIImporterImportFromUriParmsv2* parms;
 	guint code;
 
-	g_debug( "%s: instance=%p, parms=%p", thisfn, ( void * ) instance, ( void * ) parms );
+	g_debug( "%s: instance=%p, parms=%p", thisfn, ( void * ) instance, parms_ptr );
 
 	g_return_val_if_fail( NA_IS_IIMPORTER( instance ), IMPORTER_CODE_PROGRAM_ERROR );
 
-	parms->exist = FALSE;
-	parms->import_mode = IMPORTER_MODE_NO_IMPORT;
+	parms = ( NAIImporterImportFromUriParmsv2 * ) parms_ptr;
 	parms->imported = NULL;
 
 	if( !na_core_utils_file_is_loadable( parms->uri )){
@@ -347,11 +349,6 @@ naxml_reader_import_from_uri( const NAIImporter *instance, NAIImporterImportFrom
 		na_core_utils_slist_add_message( &reader->private->parms->messages, ERR_NOT_IOXML );
 	}
 
-	if( code == IMPORTER_CODE_OK ){
-		g_return_val_if_fail( NA_IS_OBJECT_ITEM( parms->imported ), IMPORTER_CODE_PROGRAM_ERROR );
-		code = manage_import_mode( reader );
-	}
-
 	g_object_unref( reader );
 
 	if( code == IMPORTER_CODE_OK ){
@@ -1393,30 +1390,6 @@ is_profile_path( NAXMLReader *reader, xmlChar *text )
 	return( is_profile );
 }
 
-static guint
-manage_import_mode( NAXMLReader *reader )
-{
-	NAIImporterManageImportModeParms parms;
-	guint code;
-
-	parms.version = 1;
-	parms.imported = reader->private->parms->imported;
-	parms.check_fn = reader->private->parms->check_fn;
-	parms.check_fn_data = reader->private->parms->check_fn_data;
-	parms.ask_fn = reader->private->parms->ask_fn;
-	parms.ask_fn_data = reader->private->parms->ask_fn_data;
-	parms.asked_mode = reader->private->parms->asked_mode;
-	parms.messages = reader->private->parms->messages;
-
-	code = na_iimporter_manage_import_mode( &parms );
-
-	reader->private->parms->exist = parms.exist;
-	reader->private->parms->import_mode = parms.import_mode;
-	reader->private->parms->messages = parms.messages;
-
-	return( code );
-}
-
 /*
  * data are reset before first run on nodes for an item
  */
