@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,11 +34,10 @@
 
 #include <glib/gi18n.h>
 #include <string.h>
-#include <unique/unique.h>
 
 #include "base-application.h"
-#include "base-window.h"
-#include "egg-sm-client.h"
+#include "base-isession.h"
+#include "base-iunique.h"
 
 /* private class data
  */
@@ -60,15 +59,7 @@ struct _BaseApplicationPrivate {
 	gchar        *description;
 	gchar        *icon_name;
 	gchar        *unique_app_name;
-
-	/* internals
-	 */
-	EggSMClient  *sm_client;
-	gulong        sm_client_quit_handler_id;
-	gulong        sm_client_quit_requested_handler_id;
-	BaseBuilder  *builder;
-	BaseWindow   *main_window;
-	UniqueApp    *unique_app_handle;
+	int           code;
 };
 
 /* instance properties
@@ -82,32 +73,41 @@ enum {
 	BASE_PROP_APPLICATION_NAME_ID,
 	BASE_PROP_DESCRIPTION_ID,
 	BASE_PROP_ICON_NAME_ID,
-	BASE_PROP_UNIQUE_APP_NAME_ID,
+	BASE_PROP_UNIQUE_NAME_ID,
+	BASE_PROP_CODE_ID,
 
 	BASE_PROP_N_PROPERTIES
 };
 
 static GObjectClass *st_parent_class = NULL;
 
-static GType          register_type( void );
-static void           class_init( BaseApplicationClass *klass );
-static void           instance_init( GTypeInstance *instance, gpointer klass );
-static void           instance_get_property( GObject *object, guint property_id, GValue *value, GParamSpec *spec );
-static void           instance_set_property( GObject *object, guint property_id, const GValue *value, GParamSpec *spec );
-static void           instance_dispose( GObject *application );
-static void           instance_finalize( GObject *application );
+static GType        register_type( void );
+static void         class_init( BaseApplicationClass *klass );
+static void         isession_iface_init( BaseISessionInterface *iface, void *user_data );
+static void         iunique_iface_init( BaseIUniqueInterface *iface, void *user_data );
+static const gchar *iunique_get_application_name( const BaseIUnique *instance );
+static void         instance_init( GTypeInstance *instance, gpointer klass );
+static void         instance_get_property( GObject *object, guint property_id, GValue *value, GParamSpec *spec );
+static void         instance_set_property( GObject *object, guint property_id, const GValue *value, GParamSpec *spec );
+static void         instance_dispose( GObject *application );
+static void         instance_finalize( GObject *application );
 
-static gboolean       appli_initialize_gtk( BaseApplication *application, int *code );
-static gboolean       appli_initialize_manage_options( const BaseApplication *application, int *code );
-static gboolean       appli_initialize_unique_app( BaseApplication *application, int *code );
-#if 0
-static UniqueResponse on_unique_message_received( UniqueApp *app, UniqueCommand command, UniqueMessageData *message, guint time, gpointer user_data );
-#endif
-static gboolean       appli_initialize_session_manager( BaseApplication *application, int *code );
-static void           session_manager_client_quit_cb( EggSMClient *client, BaseApplication *application );
-static void           session_manager_client_quit_requested_cb( EggSMClient *client, BaseApplication *application );
-static gboolean       appli_initialize_base_application( BaseApplication *application, int *code );
+static gboolean     init_i18n( BaseApplication *application );
+static gboolean     init_application_name( BaseApplication *application );
+static gboolean     init_gtk( BaseApplication *application );
+static gboolean     v_manage_options( BaseApplication *application );
+static gboolean     init_unique_manager( BaseApplication *application );
+static gboolean     init_session_manager( BaseApplication *application );
+static gboolean     init_icon_name( BaseApplication *application );
+static gboolean     v_init_application( BaseApplication *application );
+static gboolean     v_create_windows( BaseApplication *application );
 
+/*
+ * the BaseISessionInterface interface is registered here because
+ * the interface requires its implementation to be of BaseApplication
+ * type. So we have to first register the type class before trying to
+ * register the type interface.
+ */
 GType
 base_application_get_type( void )
 {
@@ -138,12 +138,28 @@ register_type( void )
 		( GInstanceInitFunc ) instance_init
 	};
 
+	static const GInterfaceInfo isession_iface_info = {
+		( GInterfaceInitFunc ) isession_iface_init,
+		NULL,
+		NULL
+	};
+
+	static const GInterfaceInfo iunique_iface_info = {
+		( GInterfaceInitFunc ) iunique_iface_init,
+		NULL,
+		NULL
+	};
+
 	g_debug( "%s", thisfn );
 
 	g_type_init();
 
 	type = g_type_register_static( G_TYPE_OBJECT, "BaseApplication", &info, 0 );
 
+	g_type_add_interface_static( type, BASE_TYPE_ISESSION, &isession_iface_info );
+
+	g_type_add_interface_static( type, BASE_TYPE_IUNIQUE, &iunique_iface_info );
+
 	return( type );
 }
 
@@ -210,18 +226,54 @@ class_init( BaseApplicationClass *klass )
 					"",
 					G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
 
-	g_object_class_install_property( object_class, BASE_PROP_UNIQUE_APP_NAME_ID,
+	g_object_class_install_property( object_class, BASE_PROP_UNIQUE_NAME_ID,
 			g_param_spec_string(
-					BASE_PROP_UNIQUE_APP_NAME,
+					BASE_PROP_UNIQUE_NAME,
 					_( "UniqueApp name" ),
 					_( "The Unique name of the application" ),
 					"",
 					G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
 
+	g_object_class_install_property( object_class, BASE_PROP_CODE_ID,
+			g_param_spec_int(
+					BASE_PROP_CODE,
+					_( "Return code" ),
+					_( "The return code of the application" ),
+					-127, 127, 0,
+					G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
+
 	klass->private = g_new0( BaseApplicationClassPrivate, 1 );
 
 	klass->manage_options = NULL;
-	klass->main_window_new = NULL;
+	klass->init_application = NULL;
+	klass->create_windows = NULL;
+}
+
+static void
+isession_iface_init( BaseISessionInterface *iface, void *user_data )
+{
+	static const gchar *thisfn = "base_application_isession_iface_init";
+
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
+}
+
+static void
+iunique_iface_init( BaseIUniqueInterface *iface, void *user_data )
+{
+	static const gchar *thisfn = "base_application_iunique_iface_init";
+
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
+
+	iface->get_application_name = iunique_get_application_name;
+}
+
+static const gchar *
+iunique_get_application_name( const BaseIUnique *instance )
+{
+	g_return_val_if_fail( BASE_IS_IUNIQUE( instance ), NULL );
+	g_return_val_if_fail( BASE_IS_APPLICATION( instance ), NULL );
+
+	return( BASE_APPLICATION( instance )->private->application_name );
 }
 
 static void
@@ -277,10 +329,14 @@ instance_get_property( GObject *object, guint property_id, GValue *value, GParam
 				g_value_set_string( value, self->private->icon_name );
 				break;
 
-			case BASE_PROP_UNIQUE_APP_NAME_ID:
+			case BASE_PROP_UNIQUE_NAME_ID:
 				g_value_set_string( value, self->private->unique_app_name );
 				break;
 
+			case BASE_PROP_CODE_ID:
+				g_value_set_int( value, self->private->code );
+				break;
+
 			default:
 				G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
 				break;
@@ -329,11 +385,15 @@ instance_set_property( GObject *object, guint property_id, const GValue *value,
 				self->private->icon_name = g_value_dup_string( value );
 				break;
 
-			case BASE_PROP_UNIQUE_APP_NAME_ID:
+			case BASE_PROP_UNIQUE_NAME_ID:
 				g_free( self->private->unique_app_name );
 				self->private->unique_app_name = g_value_dup_string( value );
 				break;
 
+			case BASE_PROP_CODE_ID:
+				self->private->code = g_value_get_int( value );
+				break;
+
 			default:
 				G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, spec );
 				break;
@@ -357,28 +417,6 @@ instance_dispose( GObject *application )
 
 		self->private->dispose_has_run = TRUE;
 
-		if( UNIQUE_IS_APP( self->private->unique_app_handle )){
-			g_object_unref( self->private->unique_app_handle );
-		}
-
-		if( GTK_IS_BUILDER( self->private->builder )){
-			g_object_unref( self->private->builder );
-		}
-
-		if( self->private->sm_client_quit_handler_id &&
-			g_signal_handler_is_connected( self->private->sm_client, self->private->sm_client_quit_handler_id )){
-				g_signal_handler_disconnect( self->private->sm_client, self->private->sm_client_quit_handler_id  );
-		}
-
-		if( self->private->sm_client_quit_requested_handler_id &&
-			g_signal_handler_is_connected( self->private->sm_client, self->private->sm_client_quit_requested_handler_id )){
-				g_signal_handler_disconnect( self->private->sm_client, self->private->sm_client_quit_requested_handler_id  );
-		}
-
-		if( self->private->sm_client ){
-			g_object_unref( self->private->sm_client );
-		}
-
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->dispose ){
 			G_OBJECT_CLASS( st_parent_class )->dispose( application );
@@ -402,6 +440,7 @@ instance_finalize( GObject *application )
 	g_free( self->private->description );
 	g_free( self->private->icon_name );
 	g_free( self->private->unique_app_name );
+	g_strfreev( self->private->argv );
 
 	g_free( self->private );
 
@@ -412,7 +451,7 @@ instance_finalize( GObject *application )
 }
 
 /**
- * base_application_run:
+ * base_application_run_with_args:
  * @application: this #BaseApplication -derived instance.
  *
  * Starts and runs the application.
@@ -427,81 +466,57 @@ instance_finalize( GObject *application )
  * seldomly needed to override this in a derived class.
  */
 int
-base_application_run( BaseApplication *application )
+base_application_run_with_args( BaseApplication *application, int argc, GStrv argv )
 {
-	static const gchar *thisfn = "base_application_run";
-	int code;
-	GtkWindow *gtk_toplevel;
+	static const gchar *thisfn = "base_application_run_with_args";
+	BaseApplicationPrivate *priv;
 
-	g_return_val_if_fail( BASE_IS_APPLICATION( application ), BASE_EXIT_CODE_START_FAIL );
+	g_return_val_if_fail( BASE_IS_APPLICATION( application ), BASE_EXIT_CODE_PROGRAM );
 
-	code = BASE_EXIT_CODE_START_FAIL;
+	priv = application->private;
 
-	if( !application->private->dispose_has_run ){
-		g_debug( "%s: application=%p", thisfn, ( void * ) application );
+	if( !priv->dispose_has_run ){
 
-		code = BASE_EXIT_CODE_OK;
-		application->private->main_window = NULL;
+		g_debug( "%s: application=%p (%s), argc=%d",
+				thisfn,
+				( void * ) application, G_OBJECT_TYPE_NAME( application ),
+				argc );
 
-		if( appli_initialize_gtk( application, &code ) &&
-			appli_initialize_manage_options( application, &code ) &&
-			appli_initialize_unique_app( application, &code ) &&
-			appli_initialize_session_manager( application, &code ) &&
-			appli_initialize_base_application( application, &code )){
+		priv->argc = argc;
+		priv->argv = g_strdupv( argv );
+		priv->code = BASE_EXIT_CODE_OK;
 
+		if( init_i18n( application ) &&
+			init_application_name( application ) &&
+			init_gtk( application ) &&
+			v_manage_options( application ) &&
+			init_unique_manager( application ) &&
+			init_session_manager( application ) &&
+			init_icon_name( application ) &&
+			v_init_application( application ) &&
+			v_create_windows( application )){
 
-			if( BASE_APPLICATION_GET_CLASS( application )->main_window_new ){
-				application->private->main_window =
-						( BaseWindow * ) BASE_APPLICATION_GET_CLASS( application )->main_window_new( application, &code );
-			} else {
-				code = BASE_EXIT_CODE_MAIN_WINDOW;
-			}
-
-			if( application->private->main_window ){
-				g_return_val_if_fail( BASE_IS_WINDOW( application->private->main_window ), BASE_EXIT_CODE_START_FAIL );
-
-				if( base_window_init( application->private->main_window )){
-					gtk_toplevel = base_window_get_gtk_toplevel( application->private->main_window );
-					g_return_val_if_fail( gtk_toplevel, BASE_EXIT_CODE_START_FAIL );
-					g_return_val_if_fail( GTK_IS_WINDOW( gtk_toplevel ), BASE_EXIT_CODE_START_FAIL );
-
-					if( application->private->unique_app_handle ){
-						unique_app_watch_window( application->private->unique_app_handle, gtk_toplevel );
-					}
-
-					g_debug( "%s: invoking base_window_run", thisfn );
-					code = base_window_run( application->private->main_window );
-
-				} else {
-					g_debug( "%s: base_window_init has returned FALSE", thisfn );
-					code = BASE_EXIT_CODE_INIT_FAIL;
-				}
-			}
+			g_debug( "%s: entering gtk_main", thisfn );
+			gtk_main();
 		}
 	}
 
-	return( code );
+	return( priv->code );
 }
 
 /*
- * pre-gtk initialization
+ * i18n initialization
  *
- * Returns: %TRUE to continue the execution, %FALSE to temrinate the program.
+ * Returns: %TRUE to continue the execution, %FALSE to terminate the program.
  * The program exit code will be taken from @code.
  */
 static gboolean
-appli_initialize_gtk( BaseApplication *application, int *code )
+init_i18n( BaseApplication *application )
 {
-	static const gchar *thisfn = "base_application_appli_initialize_gtk";
-	gchar *name;
-	gboolean ret;
-	char *parameter_string;
-	GError *error;
+	static const gchar *thisfn = "base_application_init_i18n";
 
-	g_debug( "%s: application=%p, code=%p (%d)", thisfn, ( void * ) application, ( void * ) code, *code );
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
 
-	/* initialize i18n
-	 */
 #ifdef ENABLE_NLS
 	bindtextdomain( GETTEXT_PACKAGE, GNOMELOCALEDIR );
 # ifdef HAVE_BIND_TEXTDOMAIN_CODESET
@@ -510,199 +525,206 @@ appli_initialize_gtk( BaseApplication *application, int *code )
 	textdomain( GETTEXT_PACKAGE );
 #endif
 
+	return( TRUE );
+}
+
+/*
+ * From GLib Reference Manual:
+ * Sets a human-readable name for the application.
+ * This name should be localized if possible, and is intended for display to the user.
+ *
+ * This application name is supposed to have been set as a property when
+ * the BaseApplication-derived has been instanciated.
+ */
+static gboolean
+init_application_name( BaseApplication *application )
+{
+	static const gchar *thisfn = "base_application_init_application_name";
+	gchar *name;
+	gboolean ret;
+
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
+
+	ret = TRUE;
+
 	/* setup default Gtk+ application name
 	 * must have been set at instanciation time by the derived class
 	 */
 	name = base_application_get_application_name( application );
 	if( name && g_utf8_strlen( name, -1 )){
 		g_set_application_name( name );
+
+	} else {
+		application->private->code = BASE_EXIT_CODE_APPLICATION_NAME;
+		ret = FALSE;
 	}
 	g_free( name );
 
+	return( ret );
+}
+
+/*
+ * pre-gtk initialization
+ */
+static gboolean
+init_gtk( BaseApplication *application )
+{
+	static const gchar *thisfn = "base_application_init_gtk";
+	gboolean ret;
+	char *parameter_string;
+	GError *error;
+
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
+
 	/* manage command-line arguments
 	 */
 	if( application->private->options ){
 		parameter_string = g_strdup( g_get_application_name());
 		error = NULL;
 		ret = gtk_init_with_args(
-				&application->private->argc, ( char *** ) &application->private->argv,
-				parameter_string, application->private->options, GETTEXT_PACKAGE, &error );
+				&application->private->argc,
+				( char *** ) &application->private->argv,
+				parameter_string,
+				application->private->options,
+				GETTEXT_PACKAGE,
+				&error );
 		if( error ){
 			g_warning( "%s: %s", thisfn, error->message );
 			g_error_free( error );
 			ret = FALSE;
-			*code = BASE_EXIT_CODE_ARGS;
+			application->private->code = BASE_EXIT_CODE_ARGS;
 		}
 		g_free( parameter_string );
 
 	} else {
-		ret = gtk_init_check( &application->private->argc, ( char *** ) &application->private->argv );
+		ret = gtk_init_check(
+				&application->private->argc,
+				( char *** ) &application->private->argv );
+		if( !ret ){
+			g_warning( "%s", _( "Unable to interpret command-line arguments" ));
+			application->private->code = BASE_EXIT_CODE_ARGS;
+		}
 	}
 
 	return( ret );
 }
 
 static gboolean
-appli_initialize_manage_options( const BaseApplication *application, int *code )
+v_manage_options( BaseApplication *application )
 {
-	static const gchar *thisfn = "base_application_appli_initialize_manage_options";
+	static const gchar *thisfn = "base_application_v_manage_options";
 	gboolean ret;
 
-	g_debug( "%s: application=%p, code=%p (%d)", thisfn, ( void * ) application, ( void * ) code, *code );
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
 
 	ret = TRUE;
 
 	if( BASE_APPLICATION_GET_CLASS( application )->manage_options ){
-		ret = BASE_APPLICATION_GET_CLASS( application )->manage_options( application, code );
+		ret = BASE_APPLICATION_GET_CLASS( application )->manage_options( application );
 	}
 
 	return( ret );
 }
 
+/*
+ * Initialize BaseIUnique interface for the instance
+ */
 static gboolean
-appli_initialize_unique_app( BaseApplication *application, int *code )
+init_unique_manager( BaseApplication *application )
 {
-	static const gchar *thisfn = "base_application_appli_initialize_unique_app";
+	static const gchar *thisfn = "base_application_init_unique_manager";
 	gboolean ret;
-	gboolean is_first;
-	gchar *msg;
 
-	g_debug( "%s: application=%p, code=%p (%d)", thisfn, ( void * ) application, ( void * ) code, *code );
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
 
-	ret = TRUE;
+	ret = base_iunique_init_with_name(
+			BASE_IUNIQUE( application ),
+			application->private->unique_app_name );
 
-	if( application->private->unique_app_name &&
-		strlen( application->private->unique_app_name )){
-
-			application->private->unique_app_handle = unique_app_new( application->private->unique_app_name, NULL );
-			is_first = !unique_app_is_running( application->private->unique_app_handle );
-
-			if( !is_first ){
-				unique_app_send_message( application->private->unique_app_handle, UNIQUE_ACTIVATE, NULL );
-				/* i18n: application name */
-				msg = g_strdup_printf(
-						_( "Another instance of %s is already running.\n"
-							"Please switch back to it." ), application->private->application_name );
-				base_window_display_error_dlg( NULL, _( "The application is not unique" ), msg );
-				g_free( msg );
-				ret = FALSE;
-				*code = BASE_EXIT_CODE_UNIQUE_APP;
-#if 0
-			/* default from libunique is actually to activate the first window
-			 * so we rely on the default..
-			 */
-			} else {
-				g_signal_connect( application->private->unique_app_handle,
-						"message-received", G_CALLBACK( on_unique_message_received ), application );
-#endif
-			}
+	if( !ret ){
+		application->private->code = BASE_EXIT_CODE_UNIQUE_APP;
 	}
 
 	return( ret );
 }
 
-#if 0
-static UniqueResponse
-on_unique_message_received(
-		UniqueApp *app, UniqueCommand command, UniqueMessageData *message, guint time, gpointer user_data )
-{
-	static const gchar *thisfn = "base_application_check_for_unique_app";
-	UniqueResponse resp = UNIQUE_RESPONSE_OK;
-
-	switch( command ){
-		case UNIQUE_ACTIVATE:
-			g_debug( "%s: received message UNIQUE_ACTIVATE", thisfn );
-			break;
-		default:
-			resp = UNIQUE_RESPONSE_PASSTHROUGH;
-			break;
-	}
-
-	return( resp );
-}
-#endif
-
+/*
+ * Relying on session manager to have a chance to save the modifications
+ * before exiting a session
+ */
 static gboolean
-appli_initialize_session_manager( BaseApplication *application, int *code )
+init_session_manager( BaseApplication *application )
 {
-	static const gchar *thisfn = "base_application_do_initialize_session_manager";
+	static const gchar *thisfn = "base_application_init_session_manager";
 
-	g_debug( "%s: application=%p, code=%p (%d)", thisfn, ( void * ) application, ( void * ) code, *code );
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
 
-	egg_sm_client_set_mode( EGG_SM_CLIENT_MODE_NO_RESTART );
-	application->private->sm_client = egg_sm_client_get();
-	egg_sm_client_startup();
-	g_debug( "%s: sm_client=%p", thisfn, ( void * ) application->private->sm_client );
-
-	application->private->sm_client_quit_handler_id =
-			g_signal_connect( application->private->sm_client,
-					"quit-requested", G_CALLBACK( session_manager_client_quit_requested_cb ), application );
-
-	application->private->sm_client_quit_requested_handler_id =
-			g_signal_connect( application->private->sm_client,
-					"quit", G_CALLBACK( session_manager_client_quit_cb ), application );
+	base_isession_init( BASE_ISESSION( application ));
 
 	return( TRUE );
 }
 
-static void
-session_manager_client_quit_cb( EggSMClient *client, BaseApplication *application )
-{
-	static const gchar *thisfn = "base_application_session_manager_client_quit_cb";
-
-	g_return_if_fail( BASE_IS_APPLICATION( application ));
-
-	g_debug( "%s: client=%p, application=%p", thisfn, ( void * ) client, ( void * ) application );
-
-	if( application->private->main_window ){
-
-			g_return_if_fail( BASE_IS_WINDOW( application->private->main_window ));
-			g_object_unref( application->private->main_window );
-			application->private->main_window = NULL;
-	}
-}
-
-static void
-session_manager_client_quit_requested_cb( EggSMClient *client, BaseApplication *application )
-{
-	static const gchar *thisfn = "base_application_session_manager_client_quit_requested_cb";
-	gboolean willing_to = TRUE;
-
-	g_return_if_fail( BASE_IS_APPLICATION( application ));
-
-	g_debug( "%s: client=%p, application=%p", thisfn, ( void * ) client, ( void * ) application );
-
-	if( application->private->main_window ){
-
-			g_return_if_fail( BASE_IS_WINDOW( application->private->main_window ));
-			willing_to = base_window_is_willing_to_quit( application->private->main_window );
-	}
-
-	egg_sm_client_will_quit( client, willing_to );
-}
-
+/*
+ * From GTK+ Reference Manual:
+ * Sets an icon to be used as fallback for windows that haven't had
+ * gtk_window_set_icon_list() called on them from a named themed icon.
+ *
+ * This icon name is supposed to have been set as a property when
+ * the BaseApplication-derived has been instanciated.
+ */
 static gboolean
-appli_initialize_base_application( BaseApplication *application, int *code )
+init_icon_name( BaseApplication *application )
 {
-	static const gchar *thisfn = "base_application_appli_initialize_base_application";
+	static const gchar *thisfn = "base_application_init_icon_name";
 
-	g_debug( "%s: application=%p, code=%p (%d)", thisfn, ( void * ) application, ( void * ) code, *code );
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
 
 	/* setup default application icon
 	 */
-	if( application->private->icon_name &&
-		g_utf8_strlen( application->private->icon_name, -1 )){
-
+	if( application->private->icon_name && g_utf8_strlen( application->private->icon_name, -1 )){
 			gtk_window_set_default_icon_name( application->private->icon_name );
+
+	} else {
+		g_warning( "%s: no default icon name", thisfn );
 	}
 
-	/* allocate the common BaseBuilder instance
-	 */
-	application->private->builder = base_builder_new();
-
 	return( TRUE );
 }
 
+static gboolean
+v_init_application( BaseApplication *application )
+{
+	static const gchar *thisfn = "base_application_v_init_application";
+	gboolean ret;
+
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
+
+	ret = TRUE;
+
+	if( BASE_APPLICATION_GET_CLASS( application )->init_application ){
+		ret = BASE_APPLICATION_GET_CLASS( application )->init_application( application );
+	}
+
+	return( ret );
+}
+
+static gboolean
+v_create_windows( BaseApplication *application )
+{
+	static const gchar *thisfn = "base_application_v_create_windows";
+	gboolean ret;
+
+	g_debug( "%s: application=%p", thisfn, ( void * ) application );
+
+	ret = TRUE;
+
+	if( BASE_APPLICATION_GET_CLASS( application )->create_windows ){
+		ret = BASE_APPLICATION_GET_CLASS( application )->create_windows( application );
+	}
+
+	return( ret );
+}
+
 /**
  * base_application_get_application_name:
  * @application: this #BaseApplication instance.
@@ -726,22 +748,20 @@ base_application_get_application_name( const BaseApplication *application )
 }
 
 /**
- * base_application_get_builder:
+ * base_application_is_willing_to_quit:
  * @application: this #BaseApplication instance.
  *
- * Returns: the default #BaseBuilder object for the application.
+ * Returns: %TRUE if the application is willing to quit, %FALSE else.
+ *
+ * This function is typically called from the derived application, when
+ * the user has required the termination, in order to let it handle
+ * unsaved modifications.
  */
-BaseBuilder *
-base_application_get_builder( const BaseApplication *application )
+gboolean
+base_application_is_willing_to_quit( const BaseApplication *application )
 {
-	BaseBuilder *builder = NULL;
+	g_return_val_if_fail( BASE_IS_APPLICATION( application ), TRUE );
+	g_return_val_if_fail( BASE_IS_ISESSION( application ), TRUE );
 
-	g_return_val_if_fail( BASE_IS_APPLICATION( application ), NULL );
-
-	if( !application->private->dispose_has_run ){
-
-		builder = application->private->builder;
-	}
-
-	return( builder );
+	return( base_isession_is_willing_to_quit( BASE_ISESSION( application )));
 }
