@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -39,12 +39,22 @@
 
 #include "base-gtk-utils.h"
 
-#define NACT_PROP_TOGGLE_BUTTON				"nact-prop-toggle-button"
-#define NACT_PROP_TOGGLE_HANDLER			"nact-prop-toggle-handler"
-#define NACT_PROP_TOGGLE_USER_DATA			"nact-prop-toggle-user-data"
-
 #define DEFAULT_WIDTH		22
 
+typedef struct {
+	GtkWidget *table;
+	guint      rows;
+	guint      ir;
+	guint      columns;
+	guint      ic;
+	GtkWidget *grid;
+}
+	TableToGridData;
+
+#if GTK_CHECK_VERSION( 3,0,0 )
+static void table_to_grid_foreach_cb( GtkWidget *widget, TableToGridData *ttg );
+#endif
+
 /**
  * base_gtk_utils_position_window:
  * @window: this #BaseWindow-derived window.
@@ -103,58 +113,7 @@ base_gtk_utils_save_window_position( const BaseWindow *window, const gchar *wsp_
 void
 base_gtk_utils_set_editable( GObject *widget, gboolean editable )
 {
-	GList *renderers, *irender;
-
-/* GtkComboBoxEntry is deprecated from Gtk+3
- * see. http://git.gnome.org/browse/gtk+/commit/?id=9612c648176378bf237ad0e1a8c6c995b0ca7c61
- * while 'has_entry' property exists since 2.24
- */
-#if GTK_CHECK_VERSION( 2, 24, 0 )
-	if( GTK_IS_COMBO_BOX( widget ) && gtk_combo_box_get_has_entry( GTK_COMBO_BOX( widget ))){
-#else
-	if( GTK_IS_COMBO_BOX_ENTRY( widget )){
-#endif
-		/* idem as GtkEntry */
-		gtk_editable_set_editable( GTK_EDITABLE( gtk_bin_get_child( GTK_BIN( widget ))), editable );
-		g_object_set( G_OBJECT( gtk_bin_get_child( GTK_BIN( widget ))), "can-focus", editable, NULL );
-		/* disable the listbox button itself */
-		gtk_combo_box_set_button_sensitivity( GTK_COMBO_BOX( widget ), editable ? GTK_SENSITIVITY_ON : GTK_SENSITIVITY_OFF );
-
-	} else if( GTK_IS_COMBO_BOX( widget )){
-		/* disable the listbox button itself */
-		gtk_combo_box_set_button_sensitivity( GTK_COMBO_BOX( widget ), editable ? GTK_SENSITIVITY_ON : GTK_SENSITIVITY_OFF );
-
-	} else if( GTK_IS_ENTRY( widget )){
-		gtk_editable_set_editable( GTK_EDITABLE( widget ), editable );
-		/* removing the frame leads to a disturbing modification of the
-		 * height of the control */
-		/*g_object_set( G_OBJECT( widget ), "has-frame", editable, NULL );*/
-		/* this prevents the caret to be displayed when we click in the entry */
-		g_object_set( G_OBJECT( widget ), "can-focus", editable, NULL );
-
-	} else if( GTK_IS_TEXT_VIEW( widget )){
-		g_object_set( G_OBJECT( widget ), "can-focus", editable, NULL );
-		gtk_text_view_set_editable( GTK_TEXT_VIEW( widget ), editable );
-
-	} else if( GTK_IS_TOGGLE_BUTTON( widget )){
-		/* transforms to a quasi standard GtkButton */
-		/*g_object_set( G_OBJECT( widget ), "draw-indicator", editable, NULL );*/
-		/* this at least prevent the keyboard focus to go to the button
-		 * (which is better than nothing) */
-		g_object_set( G_OBJECT( widget ), "can-focus", editable, NULL );
-
-	} else if( GTK_IS_TREE_VIEW_COLUMN( widget )){
-		renderers = gtk_cell_layout_get_cells( GTK_CELL_LAYOUT( GTK_TREE_VIEW_COLUMN( widget )));
-		for( irender = renderers ; irender ; irender = irender->next ){
-			if( GTK_IS_CELL_RENDERER_TEXT( irender->data )){
-				g_object_set( G_OBJECT( irender->data ), "editable", editable, "editable-set", TRUE, NULL );
-			}
-		}
-		g_list_free( renderers );
-
-	} else if( GTK_IS_BUTTON( widget )){
-		gtk_widget_set_sensitive( GTK_WIDGET( widget ), editable );
-	}
+	na_gtk_utils_set_editable( widget, editable );
 }
 
 /**
@@ -184,23 +143,7 @@ void
 base_gtk_utils_radio_set_initial_state( GtkRadioButton *button,
 		GCallback handler, void *user_data, gboolean editable, gboolean sensitive )
 {
-	GSList *group, *ig;
-	GtkRadioButton *other;
-
-	group = gtk_radio_button_get_group( button );
-
-	for( ig = group ; ig ; ig = ig->next ){
-		other = GTK_RADIO_BUTTON( ig->data );
-		g_object_set_data( G_OBJECT( other ), NACT_PROP_TOGGLE_BUTTON, button );
-		g_object_set_data( G_OBJECT( other ), NACT_PROP_TOGGLE_HANDLER, handler );
-		g_object_set_data( G_OBJECT( other ), NACT_PROP_TOGGLE_USER_DATA, user_data );
-		g_object_set_data( G_OBJECT( other ), NACT_PROP_TOGGLE_EDITABLE, GUINT_TO_POINTER( editable ));
-		base_gtk_utils_set_editable( G_OBJECT( other ), editable );
-		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( other ), FALSE );
-		gtk_widget_set_sensitive( GTK_WIDGET( other ), sensitive );
-	}
-
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), TRUE );
+	na_gtk_utils_radio_set_initial_state( button, handler, user_data, editable, sensitive );
 }
 
 /**
@@ -217,33 +160,7 @@ base_gtk_utils_radio_set_initial_state( GtkRadioButton *button,
 void
 base_gtk_utils_radio_reset_initial_state( GtkRadioButton *button, GCallback handler )
 {
-	GtkToggleButton *initial_button;
-	GCallback initial_handler;
-	gboolean active;
-	gboolean editable;
-	gpointer user_data;
-
-	active = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( button ));
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
-
-	if( active && !editable ){
-		initial_button = GTK_TOGGLE_BUTTON( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_BUTTON ));
-		initial_handler = G_CALLBACK( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_HANDLER ));
-		user_data = g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_USER_DATA );
-
-		if( handler ){
-			g_signal_handlers_block_by_func(( gpointer ) button, handler, user_data );
-		}
-		g_signal_handlers_block_by_func(( gpointer ) initial_button, initial_handler, user_data );
-
-		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), FALSE );
-		gtk_toggle_button_set_active( initial_button, TRUE );
-
-		g_signal_handlers_unblock_by_func(( gpointer ) initial_button, initial_handler, user_data );
-		if( handler ){
-			g_signal_handlers_unblock_by_func(( gpointer ) button, handler, user_data );
-		}
-	}
+	na_gtk_utils_radio_reset_initial_state( button, handler );
 }
 
 /**
@@ -279,9 +196,9 @@ base_gtk_utils_toggle_set_initial_state( BaseWindow *window,
 	if( button ){
 		base_window_signal_connect( window, G_OBJECT( button ), "toggled", handler );
 
-		g_object_set_data( G_OBJECT( button ), NACT_PROP_TOGGLE_HANDLER, handler );
-		g_object_set_data( G_OBJECT( button ), NACT_PROP_TOGGLE_USER_DATA, window );
-		g_object_set_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE, GUINT_TO_POINTER( editable ));
+		g_object_set_data( G_OBJECT( button ), NA_TOGGLE_DATA_HANDLER, handler );
+		g_object_set_data( G_OBJECT( button ), NA_TOGGLE_DATA_USER_DATA, window );
+		g_object_set_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE, GUINT_TO_POINTER( editable ));
 
 		base_gtk_utils_set_editable( G_OBJECT( button ), editable );
 		gtk_widget_set_sensitive( GTK_WIDGET( button ), sensitive );
@@ -307,12 +224,12 @@ base_gtk_utils_toggle_reset_initial_state( GtkToggleButton *button )
 	gpointer user_data;
 	gboolean active;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( !editable ){
 		active = gtk_toggle_button_get_active( button );
-		handler = G_CALLBACK( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_HANDLER ));
-		user_data = g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_USER_DATA );
+		handler = G_CALLBACK( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_HANDLER ));
+		user_data = g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_USER_DATA );
 
 		g_signal_handlers_block_by_func(( gpointer ) button, handler, user_data );
 		gtk_toggle_button_set_active( button, !active );
@@ -321,10 +238,10 @@ base_gtk_utils_toggle_reset_initial_state( GtkToggleButton *button )
 }
 
 /**
- * nact_utils_get_pixbuf:
+ * base_gtk_utils_get_pixbuf:
  * @name: the name of the file or an icon.
- * widget: the widget on which the imagecshould be rendered.
- * size: the desired size.
+ * @widget: the widget on which the image should be rendered.
+ * @size: the desired size.
  *
  * Returns a pixbuf for the given widget.
  */
@@ -387,17 +304,17 @@ base_gtk_utils_get_pixbuf( const gchar *name, GtkWidget *widget, GtkIconSize siz
 
 	if( !pixbuf ){
 		g_debug( "%s: null pixbuf, loading transparent image", thisfn );
-		pixbuf = gdk_pixbuf_new_from_file_at_size( PKGDATADIR "/transparent.png", width, height, NULL );
+		pixbuf = gdk_pixbuf_new_from_file_at_size( PKGUIDIR "/transparent.png", width, height, NULL );
 	}
 
 	return( pixbuf );
 }
 
 /**
- * nact_utils_render:
+ * base_gtk_utils_render:
  * @name: the name of the file or an icon, or %NULL.
- * widget: the widget on which the image should be rendered.
- * size: the desired size.
+ * @widget: the widget on which the image should be rendered.
+ * @size: the desired size.
  *
  * Displays the (maybe themed) image on the given widget.
  */
@@ -418,7 +335,7 @@ base_gtk_utils_render( const gchar *name, GtkImage *widget, GtkIconSize size )
 			width = DEFAULT_WIDTH;
 			height = DEFAULT_HEIGHT;
 		}
-		pixbuf = gdk_pixbuf_new_from_file_at_size( PKGDATADIR "/transparent.png", width, height, NULL );
+		pixbuf = gdk_pixbuf_new_from_file_at_size( PKGUIDIR "/transparent.png", width, height, NULL );
 	}
 
 	if( pixbuf ){
@@ -600,3 +517,81 @@ base_gtk_utils_select_dir( BaseWindow *window,
 
 	gtk_widget_destroy( dialog );
 }
+
+/*
+ * base_gtk_utils_table_to_grid:
+ * @window: the #BaseWindow container.
+ * @table_name: the name of the #GtkTable to be replaced.
+ *
+ * Dynamically replaces a GtkTable with a GtkGrid, doing its best in order
+ * to preserve order and name of all children.
+ *
+ * The caller has to take care of calling this function for Gtk 3.x, only
+ * replacing valuable GtkTables.
+ *
+ * This function should be called from on_base_initialize_gtk().
+ */
+void
+base_gtk_utils_table_to_grid( BaseWindow *window, const gchar *table_name )
+{
+#if GTK_CHECK_VERSION( 3,0,0 )
+	static const gchar *thisfn = "base_gtk_utils_table_to_grid";
+	TableToGridData ttg;
+	GtkWidget *parent;
+	guint col_spacing, row_spacing;
+
+	memset( &ttg, '\0', sizeof( TableToGridData ));
+
+	ttg.table = na_gtk_utils_find_widget_by_name( GTK_CONTAINER( base_window_get_gtk_toplevel( window )), table_name );
+	g_return_if_fail( ttg.table );
+	g_return_if_fail( GTK_IS_TABLE( ttg.table ));
+	g_debug( "%s: table=%p (%s)", thisfn, ( void * ) ttg.table, gtk_buildable_get_name( GTK_BUILDABLE( ttg.table )));
+
+	parent = gtk_widget_get_parent( ttg.table );
+
+#ifdef NA_MAINTAINER_MODE
+	na_gtk_utils_dump_children( GTK_CONTAINER( parent ));
+#endif
+
+	gtk_table_get_size( GTK_TABLE( ttg.table ), &ttg.rows, &ttg.columns );
+	col_spacing = gtk_table_get_default_col_spacing( GTK_TABLE( ttg.table ));
+	row_spacing = gtk_table_get_default_row_spacing( GTK_TABLE( ttg.table ));
+
+	ttg.grid = gtk_grid_new();
+	gtk_grid_set_column_spacing( GTK_GRID( ttg.grid ), col_spacing );
+	gtk_grid_set_row_spacing( GTK_GRID( ttg.grid ), row_spacing );
+
+	gtk_container_foreach( GTK_CONTAINER( ttg.table ), ( GtkCallback ) table_to_grid_foreach_cb, &ttg );
+	/*gtk_widget_unparent( ttg.table );*/
+
+	if( GTK_IS_ALIGNMENT( parent )){
+		gtk_container_remove( GTK_CONTAINER( parent ), ttg.table );
+		gtk_container_add( GTK_CONTAINER( parent ), ttg.grid );
+	} else {
+		g_warning( "%s: untreated parent of class %s", thisfn, G_OBJECT_TYPE_NAME( parent ));
+	}
+
+#ifdef NA_MAINTAINER_MODE
+	na_gtk_utils_dump_children( GTK_CONTAINER( parent ));
+#endif
+#endif
+}
+
+#if GTK_CHECK_VERSION( 3,0,0 )
+static void
+table_to_grid_foreach_cb( GtkWidget *widget, TableToGridData *ttg )
+{
+	static const gchar *thisfn = "base_gtk_utils_table_to_grid_foreach_cb";
+	guint left, top, x_options;
+
+	g_debug( "%s: widget=%p (%s)", thisfn, ( void * ) widget, gtk_buildable_get_name( GTK_BUILDABLE( widget )));
+
+	gtk_container_child_get( GTK_CONTAINER( ttg->table ), widget,
+			"left-attach", &left, "top-attach", &top, "x-options", &x_options, NULL );
+
+	gtk_widget_unparent( widget );
+
+	gtk_grid_attach( GTK_GRID( ttg->grid ), widget, left, top, 1, 1 );
+	gtk_widget_set_hexpand( widget, x_options & GTK_EXPAND );
+}
+#endif
