@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -80,7 +80,7 @@ static CapabilityTextStruct st_caps[] = {
 		{ NULL },
 };
 
-static const gchar  *st_xmlui_filename = PKGDATADIR "/nact-add-capability.ui";
+static const gchar  *st_xmlui_filename = PKGUIDIR "/nact-add-capability.ui";
 static const gchar  *st_toplevel_name  = "AddCapabilityDialog";
 static const gchar  *st_wsp_name       = NA_IPREFS_CAPABILITY_ADD_CAPABILITY_WSP;
 
@@ -89,12 +89,13 @@ static GObjectClass *st_parent_class   = NULL;
 static GType    register_type( void );
 static void     class_init( NactAddCapabilityDialogClass *klass );
 static void     instance_init( GTypeInstance *instance, gpointer klass );
+static void     instance_constructed( GObject *dialog );
 static void     instance_dispose( GObject *dialog );
 static void     instance_finalize( GObject *dialog );
 
-static void     on_base_initialize_gtk_toplevel( NactAddCapabilityDialog *editor, GtkDialog *toplevel );
-static void     on_base_initialize_base_window( NactAddCapabilityDialog *editor );
-static void     on_base_all_widgets_showed( NactAddCapabilityDialog *editor );
+static void     on_base_initialize_gtk( NactAddCapabilityDialog *editor, GtkDialog *toplevel, gpointer user_data );
+static void     on_base_initialize_window( NactAddCapabilityDialog *editor, gpointer user_data );
+static void     on_base_show_widgets( NactAddCapabilityDialog *editor, gpointer user_data );
 static gboolean on_button_press_event( GtkWidget *widget, GdkEventButton *event, NactAddCapabilityDialog *editor );
 static void     on_cancel_clicked( GtkButton *button, NactAddCapabilityDialog *editor );
 static void     on_ok_clicked( GtkButton *button, NactAddCapabilityDialog *editor );
@@ -139,7 +140,7 @@ register_type( void )
 
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( BASE_DIALOG_TYPE, "NactAddCapabilityDialog", &info, 0 );
+	type = g_type_register_static( BASE_TYPE_DIALOG, "NactAddCapabilityDialog", &info, 0 );
 
 	return( type );
 }
@@ -156,6 +157,7 @@ class_init( NactAddCapabilityDialogClass *klass )
 	st_parent_class = g_type_class_peek_parent( klass );
 
 	object_class = G_OBJECT_CLASS( klass );
+	object_class->constructed = instance_constructed;
 	object_class->dispose = instance_dispose;
 	object_class->finalize = instance_finalize;
 
@@ -179,19 +181,49 @@ instance_init( GTypeInstance *instance, gpointer klass )
 
 	self->private = g_new0( NactAddCapabilityDialogPrivate, 1 );
 
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_INITIALIZE_GTK, G_CALLBACK( on_base_initialize_gtk_toplevel ));
-
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_INITIALIZE_WINDOW, G_CALLBACK( on_base_initialize_base_window ));
-
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_ALL_WIDGETS_SHOWED, G_CALLBACK( on_base_all_widgets_showed));
-
 	self->private->dispose_has_run = FALSE;
 	self->private->capability = NULL;
 }
 
+static void
+instance_constructed( GObject *dialog )
+{
+	static const gchar *thisfn = "nact_add_capability_dialog_instance_constructed";
+	NactAddCapabilityDialogPrivate *priv;
+
+	g_return_if_fail( NACT_IS_ADD_CAPABILITY_DIALOG( dialog ));
+
+	priv = NACT_ADD_CAPABILITY_DIALOG( dialog )->private;
+
+	if( !priv->dispose_has_run ){
+
+		/* chain up to the parent class */
+		if( G_OBJECT_CLASS( st_parent_class )->constructed ){
+			G_OBJECT_CLASS( st_parent_class )->constructed( dialog );
+		}
+
+		g_debug( "%s: dialog=%p (%s)", thisfn, ( void * ) dialog, G_OBJECT_TYPE_NAME( dialog ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( dialog ),
+				G_OBJECT( dialog ),
+				BASE_SIGNAL_INITIALIZE_GTK,
+				G_CALLBACK( on_base_initialize_gtk ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( dialog ),
+				G_OBJECT( dialog ),
+				BASE_SIGNAL_INITIALIZE_WINDOW,
+				G_CALLBACK( on_base_initialize_window ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( dialog ),
+				G_OBJECT( dialog ),
+				BASE_SIGNAL_SHOW_WIDGETS,
+				G_CALLBACK( on_base_show_widgets ));
+	}
+}
+
 static void
 instance_dispose( GObject *dialog )
 {
@@ -268,7 +300,7 @@ nact_add_capability_dialog_run( BaseWindow *parent, GSList *capabilities )
 
 	g_return_val_if_fail( BASE_IS_WINDOW( parent ), NULL );
 
-	dialog = g_object_new( NACT_ADD_CAPABILITY_DIALOG_TYPE,
+	dialog = g_object_new( NACT_TYPE_ADD_CAPABILITY_DIALOG,
 					BASE_PROP_PARENT,         parent,
 					BASE_PROP_XMLUI_FILENAME, st_xmlui_filename,
 					BASE_PROP_TOPLEVEL_NAME,  st_toplevel_name,
@@ -288,9 +320,9 @@ nact_add_capability_dialog_run( BaseWindow *parent, GSList *capabilities )
 }
 
 static void
-on_base_initialize_gtk_toplevel( NactAddCapabilityDialog *dialog, GtkDialog *toplevel )
+on_base_initialize_gtk( NactAddCapabilityDialog *dialog, GtkDialog *toplevel, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_add_capability_dialog_on_base_initialize_gtk_toplevel";
+	static const gchar *thisfn = "nact_add_capability_dialog_on_base_initialize_gtk";
 	GtkTreeView *listview;
 	GtkTreeModel *model;
 	GtkTreeViewColumn *column;
@@ -300,7 +332,9 @@ on_base_initialize_gtk_toplevel( NactAddCapabilityDialog *dialog, GtkDialog *top
 	g_return_if_fail( NACT_IS_ADD_CAPABILITY_DIALOG( dialog ));
 
 	if( !dialog->private->dispose_has_run ){
-		g_debug( "%s: dialog=%p, toplevel=%p", thisfn, ( void * ) dialog, ( void * ) toplevel );
+
+		g_debug( "%s: dialog=%p, toplevel=%p, user_data=%p",
+				thisfn, ( void * ) dialog, ( void * ) toplevel, ( void * ) user_data );
 
 		listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( dialog ), "CapabilitiesTreeView" ));
 
@@ -341,9 +375,9 @@ on_base_initialize_gtk_toplevel( NactAddCapabilityDialog *dialog, GtkDialog *top
 }
 
 static void
-on_base_initialize_base_window( NactAddCapabilityDialog *dialog )
+on_base_initialize_window( NactAddCapabilityDialog *dialog, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_add_capability_dialog_on_base_initialize_base_window";
+	static const gchar *thisfn = "nact_add_capability_dialog_on_base_initialize_window";
 	GtkTreeView *listview;
 	GtkListStore *model;
 	GtkTreeIter row;
@@ -352,7 +386,8 @@ on_base_initialize_base_window( NactAddCapabilityDialog *dialog )
 	g_return_if_fail( NACT_IS_ADD_CAPABILITY_DIALOG( dialog ));
 
 	if( !dialog->private->dispose_has_run ){
-		g_debug( "%s: dialog=%p", thisfn, ( void * ) dialog );
+
+		g_debug( "%s: dialog=%p, user_data=%p", thisfn, ( void * ) dialog, ( void * ) user_data );
 
 		listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( dialog ), "CapabilitiesTreeView" ));
 		model = GTK_LIST_STORE( gtk_tree_view_get_model( listview ));
@@ -384,9 +419,9 @@ on_base_initialize_base_window( NactAddCapabilityDialog *dialog )
 }
 
 static void
-on_base_all_widgets_showed( NactAddCapabilityDialog *dialog )
+on_base_show_widgets( NactAddCapabilityDialog *dialog, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_add_capability_dialog_on_base_all_widgets_showed";
+	static const gchar *thisfn = "nact_add_capability_dialog_on_base_show_widgets";
 	GtkTreeView *listview;
 	GtkTreePath *path;
 	GtkTreeSelection *selection;
@@ -394,7 +429,8 @@ on_base_all_widgets_showed( NactAddCapabilityDialog *dialog )
 	g_return_if_fail( NACT_IS_ADD_CAPABILITY_DIALOG( dialog ));
 
 	if( !dialog->private->dispose_has_run ){
-		g_debug( "%s: dialog=%p", thisfn, ( void * ) dialog );
+
+		g_debug( "%s: dialog=%p, user_data=%p", thisfn, ( void * ) dialog, ( void * ) user_data );
 
 		listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( dialog ), "CapabilitiesTreeView" ));
 		path = gtk_tree_path_new_first();
@@ -509,16 +545,7 @@ try_for_send_ok( NactAddCapabilityDialog *dialog )
 
 	button = base_window_get_widget( BASE_WINDOW( dialog ), "OKButton" );
 
-/* gtk_widget_is_sensitive() appears with Gtk+ 2.17.5 released on 2009-07-18
- * see http://git.gnome.org/browse/gtk+/commit/?id=8f6017622937770082f7b49dfbe135fae5608704
- * GTK_WIDGET_IS_SENSITIVE macro is deprecated since 2.19.7 released on 2010-03-09
- * see http://git.gnome.org/browse/gtk+/commit/?id=a27d5a2c9eba7af5b056de32ff9b2b4dd1eb97e1
- */
-#if GTK_CHECK_VERSION( 2, 17, 5 )
 	is_sensitive = gtk_widget_is_sensitive( button );
-#else
-	is_sensitive = GTK_WIDGET_IS_SENSITIVE( button );
-#endif
 
 	if( is_sensitive ){
 		send_ok( dialog );
