@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,15 +32,18 @@
 #include <config.h>
 #endif
 
+#include <gdk/gdk.h>
 #include <glib/gi18n.h>
 #include <string.h>
 
 #include <api/na-object-api.h>
 #include <api/na-core-utils.h>
 
+#include <core/na-import-mode.h>
 #include <core/na-importer.h>
-#include <core/na-iprefs.h>
+#include <core/na-ioptions-list.h>
 #include <core/na-gtk-utils.h>
+#include <core/na-settings.h>
 
 #include "nact-application.h"
 #include "nact-assistant-import.h"
@@ -54,10 +57,9 @@
  *   0   Intro    Introduction
  *   1   Content  Selection of the files
  *   2   Content  Duplicate management: what to do with duplicates ?
- *   2   Confirm  Display the selected files before import
- *   3   Summary  Import is done: summary of the done operations
+ *   3   Confirm  Display the selected files before import
+ *   4   Summary  Import is done: summary of the done operations
  */
-
 enum {
 	ASSIST_PAGE_INTRO = 0,
 	ASSIST_PAGE_FILES_SELECTION,
@@ -66,6 +68,17 @@ enum {
 	ASSIST_PAGE_DONE
 };
 
+/* column ordering in the duplicates treeview
+ */
+enum {
+	IMAGE_COLUMN = 0,
+	LABEL_COLUMN,
+	TOOLTIP_COLUMN,
+	MODE_COLUMN,
+	INDEX_COLUMN,
+	N_COLUMN
+};
+
 /* private class data
  */
 struct _NactAssistantImportClassPrivate {
@@ -77,10 +90,13 @@ struct _NactAssistantImportClassPrivate {
 struct _NactAssistantImportPrivate {
 	gboolean     dispose_has_run;
 	GtkWidget   *file_chooser;
+	GtkTreeView *duplicates_listview;
+	NAIOption   *mode;
 	GList       *results;
+	GList       *overriden;
 };
 
-static const gchar        *st_xmlui_filename = PKGDATADIR "/nautilus-actions-config-tool.ui";
+static const gchar        *st_xmlui_filename = PKGUIDIR "/nact-assistant-import.ui";
 static const gchar        *st_toplevel_name  = "ImportAssistant";
 static const gchar        *st_wsp_name       = NA_IPREFS_IMPORT_ASSISTANT_WSP;
 
@@ -88,28 +104,33 @@ static BaseAssistantClass *st_parent_class   = NULL;
 
 static GType         register_type( void );
 static void          class_init( NactAssistantImportClass *klass );
+static void          ioptions_list_iface_init( NAIOptionsListInterface *iface, void *user_data );
+static GList        *ioptions_list_get_modes( const NAIOptionsList *instance, GtkWidget *container );
+static void          ioptions_list_free_modes( const NAIOptionsList *instance, GtkWidget *container, GList *modes );
+static NAIOption    *ioptions_list_get_ask_option( const NAIOptionsList *instance, GtkWidget *container );
 static void          instance_init( GTypeInstance *instance, gpointer klass );
 static void          instance_dispose( GObject *application );
 static void          instance_finalize( GObject *application );
 
+static void          on_base_initialize_gtk( NactAssistantImport *dialog );
+static void          create_duplicates_treeview_model( NactAssistantImport *dialog );
 static void          on_base_initialize_base_window( NactAssistantImport *dialog );
 static void          runtime_init_intro( NactAssistantImport *window, GtkAssistant *assistant );
 static void          runtime_init_file_selector( NactAssistantImport *window, GtkAssistant *assistant );
 static void          on_file_selection_changed( GtkFileChooser *chooser, gpointer user_data );
-static gboolean      has_readable_files( GSList *uris );
+static gboolean      has_loadable_files( GSList *uris );
 static void          runtime_init_duplicates( NactAssistantImport *window, GtkAssistant *assistant );
-static void          set_import_mode( GtkAssistant *assistant, gint mode );
 
 static void          assistant_prepare( BaseAssistant *window, GtkAssistant *assistant, GtkWidget *page );
 static void          prepare_confirm( NactAssistantImport *window, GtkAssistant *assistant, GtkWidget *page );
-static gint          get_import_mode( NactAssistantImport *window );
-static gchar        *add_import_mode( NactAssistantImport *window, const gchar *text );
 static void          assistant_apply( BaseAssistant *window, GtkAssistant *assistant );
 static NAObjectItem *check_for_existence( const NAObjectItem *, NactMainWindow *window );
 static void          prepare_importdone( NactAssistantImport *window, GtkAssistant *assistant, GtkWidget *page );
 static void          free_results( GList *list );
 
 static GtkWidget    *find_widget_from_page( GtkWidget *page, const gchar *name );
+static GtkTreeView  *get_duplicates_treeview_from_assistant_import( NactAssistantImport *window );
+static GtkTreeView  *get_duplicates_treeview_from_page( GtkWidget *page );
 
 GType
 nact_assistant_import_get_type( void )
@@ -141,9 +162,17 @@ register_type( void )
 		( GInstanceInitFunc ) instance_init
 	};
 
+	static const GInterfaceInfo ioptions_list_iface_info = {
+		( GInterfaceInitFunc ) ioptions_list_iface_init,
+		NULL,
+		NULL
+	};
+
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( BASE_ASSISTANT_TYPE, "NactAssistantImport", &info, 0 );
+	type = g_type_register_static( BASE_TYPE_ASSISTANT, "NactAssistantImport", &info, 0 );
+
+	g_type_add_interface_static( type, NA_TYPE_IOPTIONS_LIST, &ioptions_list_iface_info );
 
 	return( type );
 }
@@ -170,6 +199,42 @@ class_init( NactAssistantImportClass *klass )
 	assist_class->prepare = assistant_prepare;
 }
 
+static void
+ioptions_list_iface_init( NAIOptionsListInterface *iface, void *user_data )
+{
+	static const gchar *thisfn = "nact_assistant_import_ioptions_list_iface_init";
+
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
+
+	iface->get_options = ioptions_list_get_modes;
+	iface->free_options = ioptions_list_free_modes;
+	iface->get_ask_option = ioptions_list_get_ask_option;
+}
+
+static GList *
+ioptions_list_get_modes( const NAIOptionsList *instance, GtkWidget *container )
+{
+	GList *modes;
+
+	g_return_val_if_fail( NACT_IS_ASSISTANT_IMPORT( instance ), NULL );
+
+	modes = na_importer_get_modes();
+
+	return( modes );
+}
+
+static void
+ioptions_list_free_modes( const NAIOptionsList *instance, GtkWidget *container, GList *modes )
+{
+	na_importer_free_modes( modes );
+}
+
+static NAIOption *
+ioptions_list_get_ask_option( const NAIOptionsList *instance, GtkWidget *container )
+{
+	return( na_importer_get_ask_mode());
+}
+
 static void
 instance_init( GTypeInstance *instance, gpointer klass )
 {
@@ -187,8 +252,17 @@ instance_init( GTypeInstance *instance, gpointer klass )
 
 	self->private->results = NULL;
 
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_INITIALIZE_WINDOW, G_CALLBACK( on_base_initialize_base_window ));
+	base_window_signal_connect(
+			BASE_WINDOW( instance ),
+			G_OBJECT( instance ),
+			BASE_SIGNAL_INITIALIZE_GTK,
+			G_CALLBACK( on_base_initialize_gtk ));
+
+	base_window_signal_connect(
+			BASE_WINDOW( instance ),
+			G_OBJECT( instance ),
+			BASE_SIGNAL_INITIALIZE_WINDOW,
+			G_CALLBACK( on_base_initialize_base_window ));
 
 	self->private->dispose_has_run = FALSE;
 }
@@ -254,7 +328,7 @@ nact_assistant_import_run( BaseWindow *main_window )
 	esc_quit = na_settings_get_boolean( NA_IPREFS_ASSISTANT_ESC_QUIT, NULL, NULL );
 	esc_confirm = na_settings_get_boolean( NA_IPREFS_ASSISTANT_ESC_CONFIRM, NULL, NULL );
 
-	assistant = g_object_new( NACT_ASSISTANT_IMPORT_TYPE,
+	assistant = g_object_new( NACT_TYPE_ASSISTANT_IMPORT,
 			BASE_PROP_PARENT,          main_window,
 			BASE_PROP_HAS_OWN_BUILDER, TRUE,
 			BASE_PROP_XMLUI_FILENAME,  st_xmlui_filename,
@@ -267,6 +341,57 @@ nact_assistant_import_run( BaseWindow *main_window )
 	base_window_run( BASE_WINDOW( assistant ));
 }
 
+static void
+on_base_initialize_gtk( NactAssistantImport *dialog )
+{
+	static const gchar *thisfn = "nact_assistant_import_on_base_initialize_gtk";
+
+	g_return_if_fail( NACT_IS_ASSISTANT_IMPORT( dialog ));
+
+	if( !dialog->private->dispose_has_run ){
+		g_debug( "%s: dialog=%p", thisfn, ( void * ) dialog );
+
+#if !GTK_CHECK_VERSION( 3,0,0 )
+		guint padder = 8;
+		GtkAssistant *assistant = GTK_ASSISTANT( base_window_get_gtk_toplevel( BASE_WINDOW( dialog )));
+		/* selecting files */
+		GtkWidget *page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_FILES_SELECTION );
+		GtkWidget *container = find_widget_from_page( page, "p1-l2-alignment1" );
+		g_object_set( G_OBJECT( container ), "top_padding", padder, NULL );
+		/* managing duplicates */
+		page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_DUPLICATES );
+		container = find_widget_from_page( page, "p2-l2-alignment1" );
+		g_object_set( G_OBJECT( container ), "border_width", padder, NULL );
+		/* summary */
+		page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_CONFIRM );
+		container = find_widget_from_page( page, "p3-l2-alignment1" );
+		g_object_set( G_OBJECT( container ), "border_width", padder, NULL );
+		/* import is done */
+		page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_DONE );
+		container = find_widget_from_page( page, "p4-l2-alignment1" );
+		g_object_set( G_OBJECT( container ), "border_width", padder, NULL );
+#endif
+
+		create_duplicates_treeview_model( dialog );
+	}
+}
+
+static void
+create_duplicates_treeview_model( NactAssistantImport *dialog )
+{
+	static const gchar *thisfn = "nact_assistant_import_create_duplicates_treeview_model";
+
+	g_return_if_fail( NACT_IS_ASSISTANT_IMPORT( dialog ));
+	g_return_if_fail( !dialog->private->dispose_has_run );
+
+	g_debug( "%s: dialog=%p", thisfn, ( void * ) dialog );
+
+	dialog->private->duplicates_listview = get_duplicates_treeview_from_assistant_import( dialog );
+	g_return_if_fail( GTK_IS_TREE_VIEW( dialog->private->duplicates_listview ));
+
+	na_ioptions_list_gtk_init( NA_IOPTIONS_LIST( dialog ), GTK_WIDGET( dialog->private->duplicates_listview ), TRUE );
+}
+
 static void
 on_base_initialize_base_window( NactAssistantImport *dialog )
 {
@@ -315,7 +440,7 @@ runtime_init_file_selector( NactAssistantImport *window, GtkAssistant *assistant
 	page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_FILES_SELECTION );
 	g_return_if_fail( GTK_IS_CONTAINER( page ));
 
-	chooser = na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), "ImportFileChooser" );
+	chooser = na_gtk_utils_find_widget_by_name( GTK_CONTAINER( page ), "ImportFileChooser" );
 	g_return_if_fail( GTK_IS_FILE_CHOOSER( chooser ));
 
 	g_debug( "%s: window=%p, assistant=%p, page=%p, chooser=%p",
@@ -334,9 +459,11 @@ runtime_init_file_selector( NactAssistantImport *window, GtkAssistant *assistant
 			"selection-changed",
 			G_CALLBACK( on_file_selection_changed ));
 
-	gtk_assistant_set_page_complete( assistant, page, FALSE );
 	window->private->file_chooser = chooser;
+
+	gtk_assistant_set_page_complete( assistant, page, FALSE );
 }
+
 static void
 on_file_selection_changed( GtkFileChooser *chooser, gpointer user_data )
 {
@@ -354,7 +481,7 @@ on_file_selection_changed( GtkFileChooser *chooser, gpointer user_data )
 	if( pos == ASSIST_PAGE_FILES_SELECTION ){
 
 		uris = gtk_file_chooser_get_uris( chooser );
-		enabled = has_readable_files( uris );
+		enabled = has_loadable_files( uris );
 
 		if( enabled ){
 			/*
@@ -379,110 +506,56 @@ on_file_selection_changed( GtkFileChooser *chooser, gpointer user_data )
 }
 
 /*
- * enable forward button if current selection has at least one readable file
+ * enable forward button if current selection has at least one loadable file
  */
 static gboolean
-has_readable_files( GSList *uris )
+has_loadable_files( GSList *uris )
 {
-	static const gchar *thisfn = "nact_assistant_import_has_readable_files";
 	GSList *iuri;
-	int readables = 0;
 	gchar *uri;
-	GFile *file;
-	GFileInfo *info;
-	GFileType type;
-	GError *error = NULL;
-	gboolean readable;
+	int loadables = 0;
 
 	for( iuri = uris ; iuri ; iuri = iuri->next ){
-
 		uri = ( gchar * ) iuri->data;
+
 		if( !strlen( uri )){
 			continue;
 		}
 
-		file = g_file_new_for_uri( uri );
-		info = g_file_query_info( file,
-				G_FILE_ATTRIBUTE_ACCESS_CAN_READ "," G_FILE_ATTRIBUTE_STANDARD_TYPE,
-				G_FILE_QUERY_INFO_NONE, NULL, &error );
-
-		if( error ){
-			g_warning( "%s: g_file_query_info error: %s", thisfn, error->message );
-			g_error_free( error );
-			g_object_unref( file );
-			continue;
+		if( na_core_utils_file_is_loadable( uri )){
+			loadables += 1;
 		}
-
-		type = g_file_info_get_file_type( info );
-		if( type != G_FILE_TYPE_REGULAR ){
-			g_debug( "%s: %s is not a file", thisfn, uri );
-			g_object_unref( info );
-			continue;
-		}
-
-		readable = g_file_info_get_attribute_boolean( info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ );
-		if( !readable ){
-			g_debug( "%s: %s is not readable", thisfn, uri );
-			g_object_unref( info );
-			continue;
-		}
-
-		readables += 1;
-		g_object_unref( info );
 	}
 
-	return( readables > 0 );
+	return( loadables > 0 );
 }
 
 static void
 runtime_init_duplicates( NactAssistantImport *window, GtkAssistant *assistant )
 {
 	static const gchar *thisfn = "nact_assistant_import_runtime_init_duplicates";
+	gchar *import_mode;
 	GtkWidget *page;
-	guint mode;
+	gboolean mandatory;
+
+	g_return_if_fail( GTK_IS_TREE_VIEW( window->private->duplicates_listview ));
 
 	g_debug( "%s: window=%p, assistant=%p",
 			thisfn, ( void * ) window, ( void * ) assistant );
 
-	mode = na_iprefs_get_import_mode( NA_IPREFS_IMPORT_PREFERRED_MODE, NULL );
-	set_import_mode( assistant, mode );
+	import_mode = na_settings_get_string( NA_IPREFS_IMPORT_PREFERRED_MODE, NULL, &mandatory );
+	na_ioptions_list_set_editable(
+			NA_IOPTIONS_LIST( window ), GTK_WIDGET( window->private->duplicates_listview ),
+			!mandatory );
+	na_ioptions_list_set_default(
+			NA_IOPTIONS_LIST( window ), GTK_WIDGET( window->private->duplicates_listview ),
+			import_mode );
+	g_free( import_mode );
 
 	page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_DUPLICATES );
 	gtk_assistant_set_page_complete( assistant, page, TRUE );
 }
 
-static void
-set_import_mode( GtkAssistant *assistant, gint mode )
-{
-	GtkWidget *page;
-	GtkWidget *button;
-
-	page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_DUPLICATES );
-	g_return_if_fail( GTK_IS_CONTAINER( page ));
-
-	switch( mode ){
-		case IMPORTER_MODE_ASK:
-			button = na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), "AskButton" );
-			break;
-
-		case IMPORTER_MODE_RENUMBER:
-			button = na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), "RenumberButton" );
-			break;
-
-		case IMPORTER_MODE_OVERRIDE:
-			button = na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), "OverrideButton" );
-			break;
-
-		case IMPORTER_MODE_NO_IMPORT:
-		default:
-			button = na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), "NoImportButton" );
-			break;
-	}
-
-	g_return_if_fail( GTK_IS_TOGGLE_BUTTON( button ));
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), TRUE );
-}
-
 static void
 assistant_prepare( BaseAssistant *window, GtkAssistant *assistant, GtkWidget *page )
 {
@@ -514,128 +587,63 @@ prepare_confirm( NactAssistantImport *window, GtkAssistant *assistant, GtkWidget
 	static const gchar *thisfn = "nact_assistant_import_prepare_confirm";
 	gchar *text, *tmp;
 	GSList *uris, *is;
-	GtkLabel *confirm_label;
+	GtkWidget *label;
+	gchar *mode_label, *label2, *mode_description;
 
 	g_debug( "%s: window=%p, assistant=%p, page=%p",
 			thisfn, ( void * ) window, ( void * ) assistant, ( void * ) page );
 
-	/* i18n: the title of the confirm page of the import assistant */
-	text = g_strdup( _( "About to import selected files:" ));
-	tmp = g_strdup_printf( "<b>%s</b>\n\n", text );
-	g_free( text );
-	text = tmp;
+#if !GTK_CHECK_VERSION( 3,0,0 )
+	/* Note that, at least, in Gtk 2.20 (Ubuntu 10) and 2.22 (Fedora 14), GtkLabel
+	 * queues its resize (when the text is being set), but the actual resize does
+	 * not happen immediately - We have to wait until Gtk 3.0, most probably due
+	 * to the new width-for-height and height-for-width features...
+	 */
+	GtkWidget *vbox = find_widget_from_page( page, "p3-ConfirmVBox" );
+	gtk_container_set_resize_mode( GTK_CONTAINER( vbox ), GTK_RESIZE_IMMEDIATE );
+#endif
 
+	/* adding list of uris to import
+	 */
+	text = NULL;
 	uris = gtk_file_chooser_get_uris( GTK_FILE_CHOOSER( window->private->file_chooser ));
-
 	for( is = uris ; is ; is = is->next ){
-		tmp = g_strdup_printf( "%s\t%s\n", text, ( gchar * ) is->data );
-		g_free( text );
-		text = tmp;
+		g_debug( "%s: uri=%s", thisfn, ( const gchar * ) is->data );
+
+		if( text ){
+			tmp = g_strdup_printf( "%s\n%s", text, ( const gchar * ) is->data );
+			g_free( text );
+			text = tmp;
+
+		} else {
+			text = g_strdup(( const gchar * ) is->data );
+		}
 	}
-
-	tmp = add_import_mode( window, text );
+	label = find_widget_from_page( page, "p3-ConfirmFilesList" );
+	g_return_if_fail( GTK_IS_LABEL( label ));
+	gtk_label_set_text( GTK_LABEL( label ), text );
 	g_free( text );
-	text = tmp;
 
-	confirm_label = GTK_LABEL( find_widget_from_page( page, "AssistantImportConfirmLabel" ));
-	gtk_label_set_markup( confirm_label, text );
+	/* adding import mode
+	 */
+	label = find_widget_from_page( page, "p3-ConfirmImportMode" );
+	g_return_if_fail( GTK_IS_LABEL( label ));
+	window->private->mode = na_ioptions_list_get_selected(
+			NA_IOPTIONS_LIST( window ), GTK_WIDGET( window->private->duplicates_listview ));
+	g_return_if_fail( NA_IS_IMPORT_MODE( window->private->mode ));
+	mode_label = na_ioption_get_label( window->private->mode );
+	label2 = na_core_utils_str_remove_char( mode_label, "_" );
+	mode_description = na_ioption_get_description( window->private->mode );
+	text = g_markup_printf_escaped( "%s\n\n<span style=\"italic\">%s</span>", label2, mode_description );
+	gtk_label_set_markup( GTK_LABEL( label ), text );
 	g_free( text );
+	g_free( mode_description );
+	g_free( mode_label );
+	g_free( label2 );
 
 	gtk_assistant_set_page_complete( assistant, page, TRUE );
 }
 
-static gint
-get_import_mode( NactAssistantImport *window )
-{
-	GtkAssistant *assistant;
-	GtkWidget *page;
-	GtkToggleButton *renumber_button;
-	GtkToggleButton *override_button;
-	GtkToggleButton *ask_button;
-	gint mode;
-
-	mode = IMPORTER_MODE_NO_IMPORT;
-
-	assistant = GTK_ASSISTANT( base_window_get_gtk_toplevel( BASE_WINDOW( window )));
-	g_return_val_if_fail( GTK_IS_ASSISTANT( assistant ), mode );
-
-	page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_DUPLICATES );
-	g_return_val_if_fail( GTK_IS_CONTAINER( page ), mode );
-
-	renumber_button = GTK_TOGGLE_BUTTON( find_widget_from_page( page, "RenumberButton" ));
-	override_button = GTK_TOGGLE_BUTTON( find_widget_from_page( page, "OverrideButton" ));
-	ask_button = GTK_TOGGLE_BUTTON( find_widget_from_page( page, "AskButton" ));
-
-	if( gtk_toggle_button_get_active( renumber_button )){
-		mode = IMPORTER_MODE_RENUMBER;
-
-	} else if( gtk_toggle_button_get_active( override_button )){
-		mode = IMPORTER_MODE_OVERRIDE;
-
-	} else if( gtk_toggle_button_get_active( ask_button )){
-		mode = IMPORTER_MODE_ASK;
-	}
-
-	return( mode );
-}
-
-static gchar *
-add_import_mode( NactAssistantImport *window, const gchar *text )
-{
-	gint mode;
-	GtkAssistant *assistant;
-	GtkWidget *page;
-	gchar *label1, *label2, *label3;
-	gchar *result;
-
-	mode = get_import_mode( window );
-	label1 = NULL;
-	label2 = NULL;
-	result = NULL;
-
-	assistant = GTK_ASSISTANT( base_window_get_gtk_toplevel( BASE_WINDOW( window )));
-	g_return_val_if_fail( GTK_IS_ASSISTANT( assistant ), NULL );
-
-	page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_DUPLICATES );
-	g_return_val_if_fail( GTK_IS_CONTAINER( page ), NULL );
-
-	switch( mode ){
-		case IMPORTER_MODE_NO_IMPORT:
-			label1 = na_core_utils_str_remove_char( gtk_button_get_label( GTK_BUTTON( find_widget_from_page( page, "NoImportButton" ))), "_" );
-			label2 = g_strdup( gtk_label_get_text( GTK_LABEL( find_widget_from_page( page, "NoImportLabel"))));
-			break;
-
-		case IMPORTER_MODE_RENUMBER:
-			label1 = na_core_utils_str_remove_char( gtk_button_get_label( GTK_BUTTON( find_widget_from_page( page, "RenumberButton" ))), "_" );
-			label2 = g_strdup( gtk_label_get_text( GTK_LABEL( find_widget_from_page( page, "RenumberLabel"))));
-			break;
-
-		case IMPORTER_MODE_OVERRIDE:
-			label1 = na_core_utils_str_remove_char( gtk_button_get_label( GTK_BUTTON( find_widget_from_page( page, "OverrideButton" ))), "_" );
-			label2 = g_strdup( gtk_label_get_text( GTK_LABEL( find_widget_from_page( page, "OverrideLabel"))));
-			break;
-
-		case IMPORTER_MODE_ASK:
-			label1 = na_core_utils_str_remove_char( gtk_button_get_label( GTK_BUTTON( na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), "AskButton" ))), "_" );
-			label2 = g_strdup( gtk_label_get_text( GTK_LABEL( na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), "AskLabel"))));
-			break;
-
-		default:
-			break;
-	}
-
-	if( label1 ){
-		label3 = na_core_utils_str_add_prefix( "\t", label2 );
-		g_free( label2 );
-
-		result = g_strdup_printf( "%s\n<b>%s</b>\n\n%s", text, label1, label3 );
-		g_free( label3 );
-		g_free( label1 );
-	}
-
-	return( result );
-}
-
 /*
  * do import here
  */
@@ -646,8 +654,8 @@ assistant_apply( BaseAssistant *wnd, GtkAssistant *assistant )
 	NactAssistantImport *window;
 	NAImporterParms importer_parms;
 	BaseWindow *main_window;
-	GList *it;
-	GList *imported_items;
+	GList *import_results, *it;
+	GList *insertable_items, *overriden_items;
 	NAImporterResult *result;
 	NactApplication *application;
 	NAUpdater *updater;
@@ -657,30 +665,37 @@ assistant_apply( BaseAssistant *wnd, GtkAssistant *assistant )
 
 	g_debug( "%s: window=%p, assistant=%p", thisfn, ( void * ) wnd, ( void * ) assistant );
 	window = NACT_ASSISTANT_IMPORT( wnd );
-
-	imported_items = NULL;
-	memset( &importer_parms, '\0', sizeof( NAImporterParms ));
-
-	g_object_get( G_OBJECT( wnd ), BASE_PROP_PARENT, &main_window, NULL );
-	importer_parms.parent = base_window_get_gtk_toplevel( BASE_WINDOW( wnd ));
-	importer_parms.uris = gtk_file_chooser_get_uris( GTK_FILE_CHOOSER( window->private->file_chooser ));
-	importer_parms.mode = get_import_mode( window );
-	importer_parms.check_fn = ( NAIImporterCheckFn ) check_for_existence;
-	importer_parms.check_fn_data = main_window;
+	g_object_get( G_OBJECT( window ), BASE_PROP_PARENT, &main_window, NULL );
 	application = NACT_APPLICATION( base_window_get_application( main_window ));
 	updater = nact_application_get_updater( application );
 
-	na_importer_import_from_list( NA_PIVOT( updater ), &importer_parms );
+	memset( &importer_parms, '\0', sizeof( NAImporterParms ));
+	importer_parms.uris = gtk_file_chooser_get_uris( GTK_FILE_CHOOSER( window->private->file_chooser ));
+	importer_parms.check_fn = ( NAImporterCheckFn ) check_for_existence;
+	importer_parms.check_fn_data = main_window;
+	importer_parms.preferred_mode = na_import_mode_get_id( NA_IMPORT_MODE( window->private->mode ));
+	importer_parms.parent_toplevel = base_window_get_gtk_toplevel( BASE_WINDOW( wnd ));
 
-	for( it = importer_parms.results ; it ; it = it->next ){
+	import_results = na_importer_import_from_uris( NA_PIVOT( updater ), &importer_parms );
+
+	insertable_items = NULL;
+	overriden_items = NULL;
+
+	for( it = import_results ; it ; it = it->next ){
 		result = ( NAImporterResult * ) it->data;
 		if( result->imported ){
-			imported_items = g_list_prepend( imported_items, result->imported );
+
+			if( !result->exist || result->mode == IMPORTER_MODE_RENUMBER ){
+				insertable_items = g_list_prepend( insertable_items, result->imported );
+
+			} else if( result->mode == IMPORTER_MODE_OVERRIDE ){
+				overriden_items = g_list_prepend( overriden_items, result->imported );
+			}
 		}
 	}
 
 	na_core_utils_slist_free( importer_parms.uris );
-	window->private->results = importer_parms.results;
+	window->private->results = import_results;
 
 	/* then insert the list
 	 * assuring that actions will be inserted in the same order as uris
@@ -689,11 +704,20 @@ assistant_apply( BaseAssistant *wnd, GtkAssistant *assistant )
 	 * on the inserted objects; the pointers so remain valid even after
 	 * having released the imported_items list
 	 */
-	if( imported_items ){
-		imported_items = g_list_reverse( imported_items );
+	if( insertable_items ){
+		insertable_items = g_list_reverse( insertable_items );
 		items_view = nact_main_window_get_items_view( NACT_MAIN_WINDOW( main_window ));
-		nact_tree_ieditable_insert_items( NACT_TREE_IEDITABLE( items_view ), imported_items, NULL );
-		na_object_free_items( imported_items );
+		nact_tree_ieditable_insert_items( NACT_TREE_IEDITABLE( items_view ), insertable_items, NULL );
+		na_object_free_items( insertable_items );
+	}
+
+	/* contrarily, the tree store may or not take a new reference on overriding
+	 * items, so do not release it here
+	 */
+	if( overriden_items ){
+		items_view = nact_main_window_get_items_view( NACT_MAIN_WINDOW( main_window ));
+		nact_tree_ieditable_set_items( NACT_TREE_IEDITABLE( items_view ), overriden_items );
+		window->private->overriden = overriden_items;
 	}
 }
 
@@ -717,99 +741,118 @@ check_for_existence( const NAObjectItem *item, NactMainWindow *window )
 	return( exists );
 }
 
+/*
+ * summary page is a vbox inside of a scrolled window
+ * each line in this vbox is a GtkLabel
+ * Starting with 3.1.6, uri is displayed in red if an error has occured, or
+ * in blue.
+ */
 static void
 prepare_importdone( NactAssistantImport *window, GtkAssistant *assistant, GtkWidget *page )
 {
 	static const gchar *thisfn = "nact_assistant_import_prepare_importdone";
-	gchar *text, *tmp, *text2;
-	gchar *bname, *id, *label;
+	guint width;
+	GtkWidget *vbox;
+	GtkWidget *file_vbox, *file_uri, *file_report;
 	GList *is;
 	GSList *im;
 	NAImporterResult *result;
-	GFile *file;
-	guint mode;
-	GtkTextView *summary_textview;
-	GtkTextBuffer *summary_buffer;
-	GtkTextTag *title_tag;
-	GtkTextIter start, end;
-	gint title_len;
+	gchar *text, *id, *item_label, *text2, *tmp;
+	const gchar *color;
+	gchar *mode_id;
 
 	g_debug( "%s: window=%p, assistant=%p, page=%p",
 			thisfn, ( void * ) window, ( void * ) assistant, ( void * ) page );
 
-	/* i18n: result of the import assistant */
-	text = g_strdup( _( "Selected files have been proceeded :" ));
-	title_len = g_utf8_strlen( text, -1 );
-	tmp = g_strdup_printf( "%s\n\n", text );
-	g_free( text );
-	text = tmp;
+	width = 15;
+	vbox = find_widget_from_page( page, "p4-SummaryVBox" );
+	g_return_if_fail( GTK_IS_BOX( vbox ));
 
+#if !GTK_CHECK_VERSION( 3,0,0 )
+	/* Note that, at least, in Gtk 2.20 (Ubuntu 10) and 2.22 (Fedora 14), GtkLabel
+	 * queues its resize (when the text is being set), but the actual resize does
+	 * not happen immediately - We have to wait until Gtk 3.0, most probably due
+	 * to the new width-for-height and height-for-width features...
+	 */
+	gtk_container_set_resize_mode( GTK_CONTAINER( vbox ), GTK_RESIZE_IMMEDIATE );
+#endif
+
+	/* for each uri
+	 * 	- display the uri
+	 *  - display a brief import log
+	 */
 	for( is = window->private->results ; is ; is = is->next ){
 		result = ( NAImporterResult * ) is->data;
+		g_debug( "%s: uri=%s", thisfn, result->uri );
 
-		file = g_file_new_for_uri( result->uri );
-		bname = g_file_get_basename( file );
-		g_object_unref( file );
-		tmp = g_strdup_printf( "%s\t%s\n", text, bname );
+		/* display the uri
+		 */
+#if GTK_CHECK_VERSION( 3,0,0 )
+		file_vbox = gtk_box_new( GTK_ORIENTATION_VERTICAL, 4 );
+#else
+		file_vbox = gtk_vbox_new( FALSE, 4 );
+#endif
+		gtk_box_pack_start( GTK_BOX( vbox ), file_vbox, FALSE, FALSE, 0 );
+
+		color = result->imported ? "blue" : "red";
+		text = g_markup_printf_escaped( "<span foreground=\"%s\">%s</span>", color, result->uri );
+		file_uri = gtk_label_new( NULL );
+		gtk_label_set_markup( GTK_LABEL( file_uri ), text );
 		g_free( text );
-		text = tmp;
-		g_free( bname );
+		g_object_set( G_OBJECT( file_uri ), "xalign", 0, NULL );
+		g_object_set( G_OBJECT( file_uri ), "xpad", width, NULL );
+		gtk_box_pack_start( GTK_BOX( file_vbox ), file_uri, FALSE, FALSE, 0 );
 
+		/* display the import log
+		 */
 		if( result->imported ){
 			/* i18n: indicate that the file has been successfully imported */
-			tmp = g_strdup_printf( "%s\t\t%s\n", text, _( "Import OK" ));
-			g_free( text );
-			text = tmp;
+			text = g_strdup( _( "Import OK" ));
 			id = na_object_get_id( result->imported );
-			label = na_object_get_label( result->imported );
+			item_label = na_object_get_label( result->imported );
 			/* i18n: this is the globally unique identifier and the label of the newly imported action */
-			text2 = g_strdup_printf( _( "Id.: %s\t%s" ), id, label);
-			g_free( label );
+			text2 = g_strdup_printf( _( "Id.: %s\t%s" ), id, item_label);
+			g_free( item_label );
 			g_free( id );
-			tmp = g_strdup_printf( "%s\t\t%s\n", text, text2 );
+			tmp = g_strdup_printf( "%s\n%s", text, text2 );
 			g_free( text );
+			g_free( text2 );
 			text = tmp;
 
 		} else {
 			/* i18n: indicate that the file was not imported */
-			tmp = g_strdup_printf( "%s\t\t%s\n", text, _( "Not imported" ));
-			g_free( text );
-			text = tmp;
+			text = g_strdup( _( "Not imported" ));
 		}
 
-		/* add messages if any */
+		/* add messages if any
+		 */
 		for( im = result->messages ; im ; im = im->next ){
-			tmp = g_strdup_printf( "%s\t\t%s\n", text, ( const char * ) im->data );
+			tmp = g_strdup_printf( "%s\n%s", text, ( const char * ) im->data );
 			g_free( text );
 			text = tmp;
 		}
 
-		/* add a blank line between two actions */
-		tmp = g_strdup_printf( "%s\n", text );
-		g_free( text );
-		text = tmp;
+		file_report = gtk_label_new( text );
+		gtk_label_set_line_wrap( GTK_LABEL( file_report ), TRUE );
+		gtk_label_set_line_wrap_mode( GTK_LABEL( file_report ), PANGO_WRAP_WORD );
+		g_object_set( G_OBJECT( file_report ), "xalign", 0, NULL );
+		g_object_set( G_OBJECT( file_report ), "xpad", 2*width, NULL );
+		gtk_box_pack_start( GTK_BOX( file_vbox ), file_report, FALSE, FALSE, 0 );
 	}
 
-	summary_textview = GTK_TEXT_VIEW( na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), "AssistantImportSummaryTextView" ));
-	summary_buffer = gtk_text_view_get_buffer( summary_textview );
-	gtk_text_buffer_set_text( summary_buffer, text, -1 );
-	g_free( text );
+	mode_id = na_ioption_get_id( window->private->mode );
+	na_settings_set_string( NA_IPREFS_IMPORT_PREFERRED_MODE, mode_id );
+	g_free( mode_id );
 
-	title_tag = gtk_text_buffer_create_tag( summary_buffer, "title",
-			"weight", PANGO_WEIGHT_BOLD,
-			NULL );
+	/* release here our reference on overriding items
+	 */
+	if( window->private->overriden ){
+		na_object_free_items( window->private->overriden );
+	}
 
-	gtk_text_buffer_get_iter_at_offset( summary_buffer, &start, 0 );
-	gtk_text_buffer_get_iter_at_offset( summary_buffer, &end, title_len );
-	gtk_text_buffer_apply_tag( summary_buffer, title_tag, &start, &end );
-
-	g_object_unref( title_tag );
-
-	mode = get_import_mode( window );
-	na_iprefs_set_import_mode( NA_IPREFS_IMPORT_PREFERRED_MODE, mode );
-
-	gtk_assistant_set_page_complete( assistant, page, TRUE );
 	g_object_set( G_OBJECT( window ), BASE_PROP_WARN_ON_ESCAPE, FALSE, NULL );
+	gtk_assistant_set_page_complete( assistant, page, TRUE );
+	gtk_widget_show_all( page );
 }
 
 static void
@@ -831,7 +874,33 @@ find_widget_from_page( GtkWidget *page, const gchar *name )
 
 	g_return_val_if_fail( GTK_IS_CONTAINER( page ), NULL );
 
-	widget = na_gtk_utils_search_for_child_widget( GTK_CONTAINER( page ), name );
+	widget = na_gtk_utils_find_widget_by_name( GTK_CONTAINER( page ), name );
 
 	return( widget );
 }
+
+static GtkTreeView *
+get_duplicates_treeview_from_assistant_import( NactAssistantImport *window )
+{
+	GtkAssistant *assistant;
+	GtkWidget *page;
+
+	g_return_val_if_fail( NACT_IS_ASSISTANT_IMPORT( window ), NULL );
+
+	assistant = GTK_ASSISTANT( base_window_get_gtk_toplevel( BASE_WINDOW( window )));
+	page = gtk_assistant_get_nth_page( assistant, ASSIST_PAGE_DUPLICATES );
+
+	return( get_duplicates_treeview_from_page( page ));
+}
+
+static GtkTreeView *
+get_duplicates_treeview_from_page( GtkWidget *page )
+{
+	GtkWidget *listview;
+
+	listview = find_widget_from_page( page, "p2-AskTreeView" );
+
+	g_return_val_if_fail( GTK_IS_TREE_VIEW( listview ), NULL );
+
+	return( GTK_TREE_VIEW( listview ));
+}
