@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,10 +37,11 @@
 #include <api/na-object-api.h>
 
 #include <core/na-exporter.h>
-#include <core/na-iprefs.h>
+#include <core/na-export-format.h>
+#include <core/na-gtk-utils.h>
+#include <core/na-ioptions-list.h>
 
 #include "nact-application.h"
-#include "nact-export-format.h"
 #include "nact-export-ask.h"
 #include "base-gtk-utils.h"
 
@@ -57,13 +58,13 @@ struct _NactExportAskPrivate {
 	BaseWindow   *parent;
 	gboolean      preferences_locked;
 	NAObjectItem *item;
-	GQuark        format;
+	gchar        *format;
 	gboolean      format_mandatory;
 	gboolean      keep_last_choice;
 	gboolean      keep_last_choice_mandatory;
 };
 
-static const gchar     *st_xmlui_filename = PKGDATADIR "/nact-assistant-export.ui";
+static const gchar     *st_xmlui_filename = PKGUIDIR "/nact-assistant-export.ui";
 static const gchar     *st_toplevel_name  = "ExportAskDialog";
 static const gchar     *st_wsp_name       = NA_IPREFS_EXPORT_ASK_USER_WSP;
 
@@ -71,16 +72,20 @@ static BaseDialogClass *st_parent_class   = NULL;
 
 static GType    register_type( void );
 static void     class_init( NactExportAskClass *klass );
+static void     ioptions_list_iface_init( NAIOptionsListInterface *iface, void *user_data );
+static GList   *ioptions_list_get_formats( const NAIOptionsList *instance, GtkWidget *container );
+static void     ioptions_list_free_formats( const NAIOptionsList *instance, GtkWidget *container, GList *formats );
 static void     instance_init( GTypeInstance *instance, gpointer klass );
+static void     instance_constructed( GObject *dialog );
 static void     instance_dispose( GObject *dialog );
 static void     instance_finalize( GObject *dialog );
 
-static void     on_base_initialize_gtk_toplevel( NactExportAsk *editor, GtkDialog *toplevel );
-static void     on_base_initialize_base_window( NactExportAsk *editor );
+static void     on_base_initialize_gtk( NactExportAsk *editor, GtkDialog *toplevel, gpointer user_data );
+static void     on_base_initialize_window( NactExportAsk *editor, gpointer user_data );
 static void     keep_choice_on_toggled( GtkToggleButton *button, NactExportAsk *editor );
 static void     on_cancel_clicked( GtkButton *button, NactExportAsk *editor );
 static void     on_ok_clicked( GtkButton *button, NactExportAsk *editor );
-static GQuark   get_export_format( NactExportAsk *editor );
+static gchar   *get_export_format( NactExportAsk *editor );
 
 GType
 nact_export_ask_get_type( void )
@@ -112,9 +117,17 @@ register_type( void )
 		( GInstanceInitFunc ) instance_init
 	};
 
+	static const GInterfaceInfo ioptions_list_iface_info = {
+		( GInterfaceInitFunc ) ioptions_list_iface_init,
+		NULL,
+		NULL
+	};
+
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( BASE_DIALOG_TYPE, "NactExportAsk", &info, 0 );
+	type = g_type_register_static( BASE_TYPE_DIALOG, "NactExportAsk", &info, 0 );
+
+	g_type_add_interface_static( type, NA_TYPE_IOPTIONS_LIST, &ioptions_list_iface_info );
 
 	return( type );
 }
@@ -130,12 +143,48 @@ class_init( NactExportAskClass *klass )
 	st_parent_class = g_type_class_peek_parent( klass );
 
 	object_class = G_OBJECT_CLASS( klass );
+	object_class->constructed = instance_constructed;
 	object_class->dispose = instance_dispose;
 	object_class->finalize = instance_finalize;
 
 	klass->private = g_new0( NactExportAskClassPrivate, 1 );
 }
 
+static void
+ioptions_list_iface_init( NAIOptionsListInterface *iface, void *user_data )
+{
+	static const gchar *thisfn = "nact_assistant_export_ioptions_list_iface_init";
+
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
+
+	iface->get_options = ioptions_list_get_formats;
+	iface->free_options = ioptions_list_free_formats;
+}
+
+static GList *
+ioptions_list_get_formats( const NAIOptionsList *instance, GtkWidget *container )
+{
+	NactExportAsk *window;
+	NactApplication *application;
+	NAUpdater *updater;
+	GList *formats;
+
+	g_return_val_if_fail( NACT_IS_EXPORT_ASK( instance ), NULL );
+	window = NACT_EXPORT_ASK( instance );
+
+	application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+	updater = nact_application_get_updater( application );
+	formats = na_exporter_get_formats( NA_PIVOT( updater ));
+
+	return( formats );
+}
+
+static void
+ioptions_list_free_formats( const NAIOptionsList *instance, GtkWidget *container, GList *formats )
+{
+	na_exporter_free_formats( formats );
+}
+
 static void
 instance_init( GTypeInstance *instance, gpointer klass )
 {
@@ -151,15 +200,42 @@ instance_init( GTypeInstance *instance, gpointer klass )
 
 	self->private = g_new0( NactExportAskPrivate, 1 );
 
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_INITIALIZE_GTK, G_CALLBACK( on_base_initialize_gtk_toplevel ));
-
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_INITIALIZE_WINDOW, G_CALLBACK( on_base_initialize_base_window ));
-
 	self->private->dispose_has_run = FALSE;
 }
 
+static void
+instance_constructed( GObject *dialog )
+{
+	static const gchar *thisfn = "nact_export_ask_instance_constructed";
+	NactExportAskPrivate *priv;
+
+	g_return_if_fail( NACT_IS_EXPORT_ASK( dialog ));
+
+	priv = NACT_EXPORT_ASK( dialog )->private;
+
+	if( !priv->dispose_has_run ){
+
+		/* chain up to the parent class */
+		if( G_OBJECT_CLASS( st_parent_class )->constructed ){
+			G_OBJECT_CLASS( st_parent_class )->constructed( dialog );
+		}
+
+		g_debug( "%s: dialog=%p (%s)", thisfn, ( void * ) dialog, G_OBJECT_TYPE_NAME( dialog ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( dialog ),
+				G_OBJECT( dialog ),
+				BASE_SIGNAL_INITIALIZE_GTK,
+				G_CALLBACK( on_base_initialize_gtk ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( dialog ),
+				G_OBJECT( dialog ),
+				BASE_SIGNAL_INITIALIZE_WINDOW,
+				G_CALLBACK( on_base_initialize_window ));
+	}
+}
+
 static void
 instance_dispose( GObject *dialog )
 {
@@ -194,6 +270,8 @@ instance_finalize( GObject *dialog )
 
 	self = NACT_EXPORT_ASK( dialog );
 
+	g_free( self->private->format );
+
 	g_free( self->private );
 
 	/* chain call to parent class */
@@ -216,14 +294,14 @@ instance_finalize( GObject *dialog )
  * when the set export format is 'Ask me'. Each exported file runs this
  * dialog, unless the user selects the 'keep same choice' box.
  *
- * Returns: the mode chosen by the user as a #GQuark which identifies
- * the export mode.
+ * Returns: the export format chosen by the user as a newly allocated
+ * string which should be g_free() by the caller.
  * The function defaults to returning NA_IPREFS_DEFAULT_EXPORT_FORMAT.
  *
  * When the user selects 'Keep same choice without asking me', this choice
  * becomes his new preferred export format.
  */
-GQuark
+gchar *
 nact_export_ask_user( BaseWindow *parent, NAObjectItem *item, gboolean first )
 {
 	static const gchar *thisfn = "nact_export_ask_user";
@@ -231,27 +309,28 @@ nact_export_ask_user( BaseWindow *parent, NAObjectItem *item, gboolean first )
 	gboolean are_locked, mandatory;
 	gboolean keep, keep_mandatory;
 	int code;
+	gchar *format;
 
-	GQuark format = g_quark_from_static_string( NA_IPREFS_DEFAULT_EXPORT_FORMAT );
-
-	g_return_val_if_fail( BASE_IS_WINDOW( parent ), format );
+	g_return_val_if_fail( BASE_IS_WINDOW( parent ), NULL );
 
 	g_debug( "%s: parent=%p, item=%p (%s), first=%s",
-			thisfn, ( void * ) parent, ( void * ) item, G_OBJECT_TYPE_NAME( item ), first ? "True":"False" );
+			thisfn,
+			( void * ) parent,
+			( void * ) item, G_OBJECT_TYPE_NAME( item ),
+			first ? "True":"False" );
 
-	format = na_iprefs_get_export_format( NA_IPREFS_EXPORT_ASK_USER_LAST_FORMAT, &mandatory );
+	format = na_settings_get_string( NA_IPREFS_EXPORT_ASK_USER_LAST_FORMAT, NULL, &mandatory );
 	keep = na_settings_get_boolean( NA_IPREFS_EXPORT_ASK_USER_KEEP_LAST_CHOICE, NULL, &keep_mandatory );
 
 	if( first || !keep ){
-
-		editor = g_object_new( NACT_EXPORT_ASK_TYPE,
+		editor = g_object_new( NACT_TYPE_EXPORT_ASK,
 				BASE_PROP_PARENT,         parent,
 				BASE_PROP_XMLUI_FILENAME, st_xmlui_filename,
 				BASE_PROP_TOPLEVEL_NAME,  st_toplevel_name,
 				BASE_PROP_WSP_NAME,       st_wsp_name,
 				NULL );
 
-		editor->private->format = format;
+		editor->private->format = g_strdup( format );
 		editor->private->format_mandatory = mandatory;
 		editor->private->keep_last_choice = keep;
 		editor->private->keep_last_choice_mandatory = keep_mandatory;
@@ -260,17 +339,19 @@ nact_export_ask_user( BaseWindow *parent, NAObjectItem *item, gboolean first )
 
 		are_locked = na_settings_get_boolean( NA_IPREFS_ADMIN_PREFERENCES_LOCKED, NULL, &mandatory );
 		editor->private->preferences_locked = are_locked && mandatory;
-
 		code = base_window_run( BASE_WINDOW( editor ));
+
 		switch( code ){
 			case GTK_RESPONSE_OK:
+				g_free( format );
 				format = get_export_format( editor );
 				break;
 
 			case GTK_RESPONSE_CANCEL:
 			/* base_dialog::do_run only returns OK or CANCEL */
 			default:
-				format = IPREFS_EXPORT_NO_EXPORT;
+				g_free( format );
+				format = g_strdup( EXPORTER_FORMAT_NOEXPORT );
 				break;
 		}
 
@@ -281,24 +362,20 @@ nact_export_ask_user( BaseWindow *parent, NAObjectItem *item, gboolean first )
 }
 
 static void
-on_base_initialize_gtk_toplevel( NactExportAsk *editor, GtkDialog *toplevel )
+on_base_initialize_gtk( NactExportAsk *editor, GtkDialog *toplevel, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_export_ask_on_base_initialize_gtk_toplevel";
-	NactApplication *application;
-	NAUpdater *updater;
+	static const gchar *thisfn = "nact_export_ask_on_base_initialize_gtk";
 	GtkWidget *container;
 
 	g_return_if_fail( NACT_IS_EXPORT_ASK( editor ));
 
 	if( !editor->private->dispose_has_run ){
-		g_debug( "%s: editor=%p, toplevel=%p", thisfn, ( void * ) editor, ( void * ) toplevel );
 
-		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( editor )));
-		updater = nact_application_get_updater( application );
+		g_debug( "%s: dialog=%p, toplevel=%p, user_data=%p",
+				thisfn, ( void * ) editor, ( void * ) toplevel, ( void * ) user_data );
+
 		container = base_window_get_widget( BASE_WINDOW( editor ), "ExportFormatAskVBox" );
-
-		nact_export_format_init_display( container,
-				NA_PIVOT( updater ), EXPORT_FORMAT_DISPLAY_ASK, !editor->private->preferences_locked );
+		na_ioptions_list_gtk_init( NA_IOPTIONS_LIST( editor ), container, FALSE );
 
 #if !GTK_CHECK_VERSION( 2,22,0 )
 		gtk_dialog_set_has_separator( toplevel, FALSE );
@@ -307,21 +384,24 @@ on_base_initialize_gtk_toplevel( NactExportAsk *editor, GtkDialog *toplevel )
 }
 
 static void
-on_base_initialize_base_window( NactExportAsk *editor )
+on_base_initialize_window( NactExportAsk *editor, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_export_ask_on_base_initialize_base_window";
-	GtkWidget *container;
+	static const gchar *thisfn = "nact_export_ask_on_base_initialize_window";
 	gchar *item_label, *label;
 	GtkWidget *widget;
+	NactExportAskPrivate *priv;
 
 	g_return_if_fail( NACT_IS_EXPORT_ASK( editor ));
 
-	if( !editor->private->dispose_has_run ){
-		g_debug( "%s: editor=%p", thisfn, ( void * ) editor );
+	priv = editor->private;
 
-		item_label = na_object_get_label( editor->private->item );
+	if( !priv->dispose_has_run ){
 
-		if( NA_IS_OBJECT_ACTION( editor->private->item )){
+		g_debug( "%s: dialog=%p, user_data=%p", thisfn, ( void * ) editor, ( void * ) user_data );
+
+		item_label = na_object_get_label( priv->item );
+
+		if( NA_IS_OBJECT_ACTION( priv->item )){
 			/* i18n: The action <label> is about to be exported */
 			label = g_strdup_printf( _( "The action \"%s\" is about to be exported." ), item_label );
 		} else {
@@ -329,24 +409,35 @@ on_base_initialize_base_window( NactExportAsk *editor )
 			label = g_strdup_printf( _( "The menu \"%s\" is about to be exported." ), item_label );
 		}
 
-		widget = base_window_get_widget( BASE_WINDOW( editor ), "ExportAskLabel1" );
+		widget = base_window_get_widget( BASE_WINDOW( editor ), "ExportAskLabel" );
 		gtk_label_set_text( GTK_LABEL( widget ), label );
 		g_free( label );
 		g_free( item_label );
 
-		container = base_window_get_widget( BASE_WINDOW( editor ), "ExportFormatAskVBox" );
-		nact_export_format_select( container, !editor->private->format_mandatory, editor->private->format );
+		widget = base_window_get_widget( BASE_WINDOW( editor ), "ExportFormatAskVBox" );
+		na_ioptions_list_set_editable(
+				NA_IOPTIONS_LIST( editor ), widget,
+				!priv->format_mandatory && !priv->preferences_locked );
+		na_ioptions_list_set_default(
+				NA_IOPTIONS_LIST( editor ), widget,
+				priv->format );
 
 		base_gtk_utils_toggle_set_initial_state( BASE_WINDOW( editor ),
 				"AskKeepChoiceButton", G_CALLBACK( keep_choice_on_toggled ),
-				editor->private->keep_last_choice,
-				!editor->private->keep_last_choice_mandatory, !editor->private->preferences_locked );
+				priv->keep_last_choice,
+				!priv->keep_last_choice_mandatory, !priv->preferences_locked );
 
-		base_window_signal_connect_by_name( BASE_WINDOW( editor ),
-				"CancelButton", "clicked", G_CALLBACK( on_cancel_clicked ));
+		base_window_signal_connect_by_name(
+				BASE_WINDOW( editor ),
+				"CancelButton",
+				"clicked",
+				G_CALLBACK( on_cancel_clicked ));
 
-		base_window_signal_connect_by_name( BASE_WINDOW( editor ),
-				"OKButton", "clicked", G_CALLBACK( on_ok_clicked ));
+		base_window_signal_connect_by_name(
+				BASE_WINDOW( editor ),
+				"OKButton",
+				"clicked",
+				G_CALLBACK( on_ok_clicked ));
 	}
 }
 
@@ -355,7 +446,7 @@ keep_choice_on_toggled( GtkToggleButton *button, NactExportAsk *editor )
 {
 	gboolean editable;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->keep_last_choice = gtk_toggle_button_get_active( button );
@@ -391,22 +482,23 @@ on_ok_clicked( GtkButton *button, NactExportAsk *editor )
  * let the user go back in ask dialog box the next time he will have some
  * files to export
  */
-static GQuark
+static gchar *
 get_export_format( NactExportAsk *editor )
 {
-	GtkWidget *container;
-	NAExportFormat *format;
-	GQuark format_quark;
+	GtkWidget *widget;
+	NAIOption *format;
+	gchar *format_id;
 
-	container = base_window_get_widget( BASE_WINDOW( editor ), "ExportFormatAskVBox" );
-	format = nact_export_format_get_selected( container );
-	format_quark = na_export_format_get_quark( format );
+	widget = base_window_get_widget( BASE_WINDOW( editor ), "ExportFormatAskVBox" );
+	format = na_ioptions_list_get_selected( NA_IOPTIONS_LIST( editor ), widget );
+	g_return_val_if_fail( NA_IS_EXPORT_FORMAT( format ), 0 );
 
 	if( !editor->private->keep_last_choice_mandatory ){
 		na_settings_set_boolean( NA_IPREFS_EXPORT_ASK_USER_KEEP_LAST_CHOICE, editor->private->keep_last_choice );
 	}
 
-	na_iprefs_set_export_format( NA_IPREFS_EXPORT_ASK_USER_LAST_FORMAT, format_quark );
+	format_id = na_ioption_get_id( format );
+	na_settings_set_string( NA_IPREFS_EXPORT_ASK_USER_LAST_FORMAT, format_id );
 
-	return( format_quark );
+	return( format_id );
 }
