@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -84,39 +84,52 @@ enum {
 	N_COLUMN
 };
 
-static gboolean st_initialized = FALSE;
-static gboolean st_finalized = FALSE;
-static gboolean st_on_selection_change = FALSE;
+/* Pseudo-property, set against the instance
+ */
+typedef struct {
+	gboolean on_selection_change;
+}
+	IEnvironData;
 
-static GType    register_type( void );
-static void     interface_base_init( NactIEnvironmentTabInterface *klass );
-static void     interface_base_finalize( NactIEnvironmentTabInterface *klass );
+#define IENVIRON_TAB_PROP_DATA				"nact-ienviron-tab-data"
 
-static void     on_main_selection_changed( NactIEnvironmentTab *instance, GList *selected_items, gpointer user_data );
+static guint st_initializations = 0;		/* interface initialization count */
 
-static void     on_selcount_ope_changed( GtkComboBox *combo, NactIEnvironmentTab *instance );
-static void     on_selcount_int_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
-static void     on_selection_count_changed( NactIEnvironmentTab *instance );
-static void     on_show_always_toggled( GtkToggleButton *togglebutton, NactIEnvironmentTab *instance );
-static void     on_only_show_toggled( GtkToggleButton *togglebutton, NactIEnvironmentTab *instance );
-static void     on_do_not_show_toggled( GtkToggleButton *togglebutton, NactIEnvironmentTab *instance );
-static void     on_desktop_toggled( GtkCellRendererToggle *renderer, gchar *path, BaseWindow *window );
-static void     on_try_exec_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
-static void     on_try_exec_browse( GtkButton *button, NactIEnvironmentTab *instance );
-static void     on_show_if_registered_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
-static void     on_show_if_true_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
-static void     on_show_if_running_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
-static void     on_show_if_running_browse( GtkButton *button, NactIEnvironmentTab *instance );
+static GType         register_type( void );
+static void          interface_base_init( NactIEnvironmentTabInterface *klass );
+static void          interface_base_finalize( NactIEnvironmentTabInterface *klass );
 
-static void     init_selection_count_combobox( NactIEnvironmentTab *instance );
-static gchar   *get_selection_count_selection( NactIEnvironmentTab *instance );
-static void     set_selection_count_selection( NactIEnvironmentTab *instance, const gchar *ope, const gchar *uint );
-static void     dispose_selection_count_combobox( NactIEnvironmentTab *instance );
+static void          on_base_initialize_gtk( NactIEnvironmentTab *instance, GtkWindow *toplevel, gpointer user_data );
+static void          on_base_initialize_window( NactIEnvironmentTab *instance, gpointer user_data );
 
-static void     init_desktop_listview( NactIEnvironmentTab *instance );
-static void     raz_desktop_listview( NactIEnvironmentTab *instance );
-static void     setup_desktop_listview( NactIEnvironmentTab *instance, GSList *show );
-static void     dispose_desktop_listview( NactIEnvironmentTab *instance );
+static void          on_main_selection_changed( NactIEnvironmentTab *instance, GList *selected_items, gpointer user_data );
+
+static void          on_selcount_ope_changed( GtkComboBox *combo, NactIEnvironmentTab *instance );
+static void          on_selcount_int_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
+static void          on_selection_count_changed( NactIEnvironmentTab *instance );
+static void          on_show_always_toggled( GtkToggleButton *togglebutton, NactIEnvironmentTab *instance );
+static void          on_only_show_toggled( GtkToggleButton *togglebutton, NactIEnvironmentTab *instance );
+static void          on_do_not_show_toggled( GtkToggleButton *togglebutton, NactIEnvironmentTab *instance );
+static void          on_desktop_toggled( GtkCellRendererToggle *renderer, gchar *path, BaseWindow *window );
+static void          on_try_exec_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
+static void          on_try_exec_browse( GtkButton *button, NactIEnvironmentTab *instance );
+static void          on_show_if_registered_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
+static void          on_show_if_true_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
+static void          on_show_if_running_changed( GtkEntry *entry, NactIEnvironmentTab *instance );
+static void          on_show_if_running_browse( GtkButton *button, NactIEnvironmentTab *instance );
+
+static void          init_selection_count_combobox( NactIEnvironmentTab *instance );
+static gchar        *get_selection_count_selection( NactIEnvironmentTab *instance );
+static void          set_selection_count_selection( NactIEnvironmentTab *instance, const gchar *ope, const gchar *uint );
+static void          dispose_selection_count_combobox( NactIEnvironmentTab *instance );
+
+static void          init_desktop_listview( NactIEnvironmentTab *instance );
+static void          raz_desktop_listview( NactIEnvironmentTab *instance );
+static void          setup_desktop_listview( NactIEnvironmentTab *instance, GSList *show );
+static void          dispose_desktop_listview( NactIEnvironmentTab *instance );
+
+static IEnvironData *get_ienviron_data( NactIEnvironmentTab *instance );
+static void          on_instance_finalized( gpointer user_data, NactIEnvironmentTab *instance );
 
 GType
 nact_ienvironment_tab_get_type( void )
@@ -152,7 +165,7 @@ register_type( void )
 
 	type = g_type_register_static( G_TYPE_INTERFACE, "NactIEnvironmentTab", &info, 0 );
 
-	g_type_interface_add_prerequisite( type, BASE_WINDOW_TYPE );
+	g_type_interface_add_prerequisite( type, BASE_TYPE_WINDOW );
 
 	return( type );
 }
@@ -162,14 +175,14 @@ interface_base_init( NactIEnvironmentTabInterface *klass )
 {
 	static const gchar *thisfn = "nact_ienvironment_tab_interface_base_init";
 
-	if( !st_initialized ){
+	if( !st_initializations ){
 
 		g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
 		klass->private = g_new0( NactIEnvironmentTabInterfacePrivate, 1 );
-
-		st_initialized = TRUE;
 	}
+
+	st_initializations += 1;
 }
 
 static void
@@ -177,48 +190,89 @@ interface_base_finalize( NactIEnvironmentTabInterface *klass )
 {
 	static const gchar *thisfn = "nact_ienvironment_tab_interface_base_finalize";
 
-	if( st_initialized && !st_finalized ){
+	st_initializations -= 1;
+
+	if( !st_initializations ){
 
 		g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
-		st_finalized = TRUE;
-
 		g_free( klass->private );
 	}
 }
 
 /**
- * nact_ienvironment_tab_initial_load:
+ * nact_ienvironment_tab_init:
+ * @instance: this #NactIEnvironmentTab instance.
+ *
+ * Initialize the interface
+ * Connect to #BaseWindow signals
+ */
+void
+nact_ienvironment_tab_init( NactIEnvironmentTab *instance )
+{
+	static const gchar *thisfn = "nact_ienvironment_tab_init";
+	IEnvironData *data;
+
+	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( instance ));
+
+	g_debug( "%s: instance=%p (%s)",
+			thisfn,
+			( void * ) instance, G_OBJECT_TYPE_NAME( instance ));
+
+	base_window_signal_connect(
+			BASE_WINDOW( instance ),
+			G_OBJECT( instance ),
+			BASE_SIGNAL_INITIALIZE_GTK,
+			G_CALLBACK( on_base_initialize_gtk ));
+
+	base_window_signal_connect(
+			BASE_WINDOW( instance ),
+			G_OBJECT( instance ),
+			BASE_SIGNAL_INITIALIZE_WINDOW,
+			G_CALLBACK( on_base_initialize_window ));
+
+	nact_main_tab_init( NACT_MAIN_WINDOW( instance ), TAB_ENVIRONMENT );
+
+	data = get_ienviron_data( instance );
+	data->on_selection_change = FALSE;
+
+	g_object_weak_ref( G_OBJECT( instance ), ( GWeakNotify ) on_instance_finalized, NULL );
+}
+
+/*
+ * on_base_initialize_gtk:
  * @window: this #NactIEnvironmentTab instance.
  *
  * Initializes the tab widget at initial load.
  */
-void
-nact_ienvironment_tab_initial_load_toplevel( NactIEnvironmentTab *instance )
+static void
+on_base_initialize_gtk( NactIEnvironmentTab *instance, GtkWindow *toplevel, void *user_data )
 {
-	static const gchar *thisfn = "nact_ienvironment_tab_initial_load_toplevel";
+	static const gchar *thisfn = "nact_ienvironment_tab_on_base_initialize_gtk";
 
 	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( instance ));
 
-	if( st_initialized && !st_finalized ){
-		g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
+	g_debug( "%s: instance=%p (%s), toplevel=%p, user_data=%p",
+			thisfn,
+			( void * ) instance, G_OBJECT_TYPE_NAME( instance ),
+			( void * ) toplevel,
+			( void * ) user_data );
 
-		init_selection_count_combobox( instance );
-		init_desktop_listview( instance );
-	}
+	init_selection_count_combobox( instance );
+	init_desktop_listview( instance );
 }
 
-/**
- * nact_ienvironment_tab_runtime_init:
+/*
+ * on_base_initialize_window:
  * @window: this #NactIEnvironmentTab instance.
  *
  * Initializes the tab widget at each time the widget will be displayed.
  * Connect signals and setup runtime values.
  */
-void
-nact_ienvironment_tab_runtime_init_toplevel( NactIEnvironmentTab *instance )
+static void
+on_base_initialize_window( NactIEnvironmentTab *instance, void *user_data )
 {
-	static const gchar *thisfn = "nact_ienvironment_tab_runtime_init_toplevel";
+	static const gchar *thisfn = "nact_ienvironment_tab_on_base_initialize_window";
 	GtkTreeView *listview;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -229,102 +283,107 @@ nact_ienvironment_tab_runtime_init_toplevel( NactIEnvironmentTab *instance )
 
 	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( instance ));
 
-	if( st_initialized && !st_finalized ){
-		g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
+	g_debug( "%s: instance=%p (%s), user_data=%p",
+			thisfn,
+			( void * ) instance, G_OBJECT_TYPE_NAME( instance ),
+			( void * ) user_data );
 
-		base_window_signal_connect( BASE_WINDOW( instance ),
-				G_OBJECT( instance ), MAIN_SIGNAL_SELECTION_CHANGED, G_CALLBACK( on_main_selection_changed ));
+	base_window_signal_connect(
+			BASE_WINDOW( instance ),
+			G_OBJECT( instance ),
+			MAIN_SIGNAL_SELECTION_CHANGED,
+			G_CALLBACK( on_main_selection_changed ));
 
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"SelectionCountSigneCombobox", "changed", G_CALLBACK( on_selcount_ope_changed ));
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"SelectionCountSigneCombobox",
+			"changed",
+			G_CALLBACK( on_selcount_ope_changed ));
 
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"SelectionCountNumberEntry", "changed", G_CALLBACK( on_selcount_int_changed ));
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"SelectionCountNumberEntry",
+			"changed",
+			G_CALLBACK( on_selcount_int_changed ));
 
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"ShowAlwaysButton", "toggled", G_CALLBACK( on_show_always_toggled ));
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"ShowAlwaysButton",
+			"toggled",
+			G_CALLBACK( on_show_always_toggled ));
 
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"OnlyShowButton", "toggled", G_CALLBACK( on_only_show_toggled ));
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"OnlyShowButton",
+			"toggled",
+			G_CALLBACK( on_only_show_toggled ));
 
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"DoNotShowButton", "toggled", G_CALLBACK( on_do_not_show_toggled ));
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"DoNotShowButton",
+			"toggled",
+			G_CALLBACK( on_do_not_show_toggled ));
 
-		listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( instance ), "EnvironmentsDesktopTreeView" ));
-		model = gtk_tree_view_get_model( listview );
+	listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( instance ), "EnvironmentsDesktopTreeView" ));
+	model = gtk_tree_view_get_model( listview );
 
-		desktops = na_desktop_environment_get_known_list();
+	desktops = na_desktop_environment_get_known_list();
 
-		for( i = 0 ; desktops[i].id ; ++i ){
-			gtk_list_store_append( GTK_LIST_STORE( model ), &iter );
-			gtk_list_store_set(
-					GTK_LIST_STORE( model ),
-					&iter,
-					ENV_BOOL_COLUMN, FALSE,
-					ENV_LABEL_COLUMN, gettext( desktops[i].label ),
-					ENV_KEYWORD_COLUMN, desktops[i].id,
-					-1 );
-		}
-
-		column = gtk_tree_view_get_column( listview, ENV_BOOL_COLUMN );
-		renderers = gtk_cell_layout_get_cells( GTK_CELL_LAYOUT( column ));
-		base_window_signal_connect( BASE_WINDOW( instance ),
-				G_OBJECT( renderers->data ), "toggled", G_CALLBACK( on_desktop_toggled ));
-
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"TryExecEntry", "changed", G_CALLBACK( on_try_exec_changed ));
-
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"TryExecButton", "clicked", G_CALLBACK( on_try_exec_browse ));
-
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"ShowIfRegisteredEntry", "changed", G_CALLBACK( on_show_if_registered_changed ));
-
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"ShowIfTrueEntry", "changed", G_CALLBACK( on_show_if_true_changed ));
-
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"ShowIfRunningEntry", "changed", G_CALLBACK( on_show_if_running_changed ));
-
-		base_window_signal_connect_by_name( BASE_WINDOW( instance ),
-				"ShowIfRunningButton", "clicked", G_CALLBACK( on_show_if_running_browse ));
+	for( i = 0 ; desktops[i].id ; ++i ){
+		gtk_list_store_append( GTK_LIST_STORE( model ), &iter );
+		gtk_list_store_set(
+				GTK_LIST_STORE( model ),
+				&iter,
+				ENV_BOOL_COLUMN, FALSE,
+				ENV_LABEL_COLUMN, gettext( desktops[i].label ),
+				ENV_KEYWORD_COLUMN, desktops[i].id,
+				-1 );
 	}
-}
 
-void
-nact_ienvironment_tab_all_widgets_showed( NactIEnvironmentTab *instance )
-{
-	static const gchar *thisfn = "nact_ienvironment_tab_all_widgets_showed";
+	column = gtk_tree_view_get_column( listview, ENV_BOOL_COLUMN );
+	renderers = gtk_cell_layout_get_cells( GTK_CELL_LAYOUT( column ));
 
-	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( instance ));
+	base_window_signal_connect(
+			BASE_WINDOW( instance ),
+			G_OBJECT( renderers->data ),
+			"toggled",
+			G_CALLBACK( on_desktop_toggled ));
 
-	if( st_initialized && !st_finalized ){
-		g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
-	}
-}
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"TryExecEntry",
+			"changed",
+			G_CALLBACK( on_try_exec_changed ));
 
-/**
- * nact_ienvironment_tab_dispose:
- * @window: this #NactIEnvironmentTab instance.
- *
- * Called at instance_dispose time.
- */
-void
-nact_ienvironment_tab_dispose( NactIEnvironmentTab *instance )
-{
-	static const gchar *thisfn = "nact_ienvironment_tab_dispose";
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"TryExecButton",
+			"clicked",
+			G_CALLBACK( on_try_exec_browse ));
 
-	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( instance ));
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"ShowIfRegisteredEntry",
+			"changed",
+			G_CALLBACK( on_show_if_registered_changed ));
 
-	if( st_initialized && !st_finalized ){
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"ShowIfTrueEntry",
+			"changed",
+			G_CALLBACK( on_show_if_true_changed ));
 
-		g_debug( "%s: instance=%p", thisfn, ( void * ) instance );
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"ShowIfRunningEntry",
+			"changed",
+			G_CALLBACK( on_show_if_running_changed ));
 
-		st_on_selection_change = TRUE;
-
-		dispose_selection_count_combobox( instance );
-		dispose_desktop_listview( instance );
-	}
+	base_window_signal_connect_by_name(
+			BASE_WINDOW( instance ),
+			"ShowIfRunningButton",
+			"clicked",
+			G_CALLBACK( on_show_if_running_browse ));
 }
 
 static void
@@ -343,119 +402,121 @@ on_main_selection_changed( NactIEnvironmentTab *instance, GList *selected_items,
 	GtkWidget *browse_button;
 	GSList *desktops;
 	gchar *text;
+	IEnvironData *data;
 
 	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( instance ));
 
-	if( st_initialized && !st_finalized ){
-		g_debug( "%s: instance=%p, selected_items=%p (count=%d)",
-				thisfn, ( void * ) instance, ( void * ) selected_items, g_list_length( selected_items ));
+	g_debug( "%s: instance=%p (%s), selected_items=%p (count=%d)",
+			thisfn,
+			( void * ) instance, G_OBJECT_TYPE_NAME( instance ),
+			( void * ) selected_items, g_list_length( selected_items ));
 
-		g_object_get( G_OBJECT( instance ),
-				MAIN_PROP_CONTEXT, &context, MAIN_PROP_EDITABLE, &editable,
-				NULL );
+	g_object_get( G_OBJECT( instance ),
+			MAIN_PROP_CONTEXT, &context, MAIN_PROP_EDITABLE, &editable,
+			NULL );
 
-		enable_tab = ( context != NULL );
-		nact_main_tab_enable_page( NACT_MAIN_WINDOW( instance ), TAB_ENVIRONMENT, enable_tab );
+	enable_tab = ( context != NULL );
+	nact_main_tab_enable_page( NACT_MAIN_WINDOW( instance ), TAB_ENVIRONMENT, enable_tab );
 
-		st_on_selection_change = TRUE;
+	data = get_ienviron_data( instance );
+	data->on_selection_change = TRUE;
 
-		/* selection count
-		 */
-		sel_count = context ? na_object_get_selection_count( context ) : g_strdup( "" );
-		na_core_utils_selcount_get_ope_int( sel_count, &selcount_ope, &selcount_int );
-		set_selection_count_selection( instance, selcount_ope, selcount_int );
-		g_free( selcount_int );
-		g_free( selcount_ope );
-		g_free( sel_count );
+	/* selection count
+	 */
+	sel_count = context ? na_object_get_selection_count( context ) : g_strdup( "" );
+	na_core_utils_selcount_get_ope_int( sel_count, &selcount_ope, &selcount_int );
+	set_selection_count_selection( instance, selcount_ope, selcount_int );
+	g_free( selcount_int );
+	g_free( selcount_ope );
+	g_free( sel_count );
 
-		combo = base_window_get_widget( BASE_WINDOW( instance ), "SelectionCountSigneCombobox" );
-		base_gtk_utils_set_editable( G_OBJECT( combo ), editable );
+	combo = base_window_get_widget( BASE_WINDOW( instance ), "SelectionCountSigneCombobox" );
+	base_gtk_utils_set_editable( G_OBJECT( combo ), editable );
 
-		entry = base_window_get_widget( BASE_WINDOW( instance ), "SelectionCountNumberEntry" );
-		base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
+	entry = base_window_get_widget( BASE_WINDOW( instance ), "SelectionCountNumberEntry" );
+	base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
 
-		/* desktop environment
-		 */
-		raz_desktop_listview( instance );
+	/* desktop environment
+	 */
+	raz_desktop_listview( instance );
 
-		always_button = base_window_get_widget( BASE_WINDOW( instance ), "ShowAlwaysButton" );
-		show_button = base_window_get_widget( BASE_WINDOW( instance ), "OnlyShowButton" );
-		notshow_button = base_window_get_widget( BASE_WINDOW( instance ), "DoNotShowButton" );
+	always_button = base_window_get_widget( BASE_WINDOW( instance ), "ShowAlwaysButton" );
+	show_button = base_window_get_widget( BASE_WINDOW( instance ), "OnlyShowButton" );
+	notshow_button = base_window_get_widget( BASE_WINDOW( instance ), "DoNotShowButton" );
 
-		desktops = context ? na_object_get_only_show_in( context ) : NULL;
-		listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( instance ), "EnvironmentsDesktopTreeView" ));
-		gtk_toggle_button_set_inconsistent( GTK_TOGGLE_BUTTON( always_button ), context == NULL );
+	desktops = context ? na_object_get_only_show_in( context ) : NULL;
+	listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( instance ), "EnvironmentsDesktopTreeView" ));
+	gtk_toggle_button_set_inconsistent( GTK_TOGGLE_BUTTON( always_button ), context == NULL );
+
+	if( desktops && g_slist_length( desktops )){
+		base_gtk_utils_radio_set_initial_state(
+				GTK_RADIO_BUTTON( show_button ),
+				G_CALLBACK( on_only_show_toggled ), instance, editable, ( context != NULL ));
+		gtk_widget_set_sensitive( GTK_WIDGET( listview ), TRUE );
+
+	} else {
+		desktops = context ? na_object_get_not_show_in( context ) : NULL;
 
 		if( desktops && g_slist_length( desktops )){
 			base_gtk_utils_radio_set_initial_state(
-					GTK_RADIO_BUTTON( show_button ),
-					G_CALLBACK( on_only_show_toggled ), instance, editable, ( context != NULL ));
+					GTK_RADIO_BUTTON( notshow_button ),
+					G_CALLBACK( on_do_not_show_toggled ), instance, editable, ( context != NULL ));
 			gtk_widget_set_sensitive( GTK_WIDGET( listview ), TRUE );
 
 		} else {
-			desktops = context ? na_object_get_not_show_in( context ) : NULL;
-
-			if( desktops && g_slist_length( desktops )){
-				base_gtk_utils_radio_set_initial_state(
-						GTK_RADIO_BUTTON( notshow_button ),
-						G_CALLBACK( on_do_not_show_toggled ), instance, editable, ( context != NULL ));
-				gtk_widget_set_sensitive( GTK_WIDGET( listview ), TRUE );
-
-			} else {
-				base_gtk_utils_radio_set_initial_state(
-						GTK_RADIO_BUTTON( always_button ),
-						G_CALLBACK( on_show_always_toggled ), instance, editable, ( context != NULL ));
-				gtk_widget_set_sensitive( GTK_WIDGET( listview ), FALSE );
-				desktops = NULL;
-			}
+			base_gtk_utils_radio_set_initial_state(
+					GTK_RADIO_BUTTON( always_button ),
+					G_CALLBACK( on_show_always_toggled ), instance, editable, ( context != NULL ));
+			gtk_widget_set_sensitive( GTK_WIDGET( listview ), FALSE );
+			desktops = NULL;
 		}
+	}
 
-		setup_desktop_listview( instance, desktops );
+	setup_desktop_listview( instance, desktops );
 
-		/* execution environment
-		 */
-		entry = base_window_get_widget( BASE_WINDOW( instance ), "TryExecEntry" );
-		text = context ? na_object_get_try_exec( context ) : g_strdup( "" );
-		text = text && strlen( text ) ? text : g_strdup( "" );
-		gtk_entry_set_text( GTK_ENTRY( entry ), text );
-		g_free( text );
-		base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
+	/* execution environment
+	 */
+	entry = base_window_get_widget( BASE_WINDOW( instance ), "TryExecEntry" );
+	text = context ? na_object_get_try_exec( context ) : g_strdup( "" );
+	text = text && strlen( text ) ? text : g_strdup( "" );
+	gtk_entry_set_text( GTK_ENTRY( entry ), text );
+	g_free( text );
+	base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
 
-		browse_button = base_window_get_widget( BASE_WINDOW( instance ), "TryExecButton" );
-		base_gtk_utils_set_editable( G_OBJECT( browse_button ), editable );
+	browse_button = base_window_get_widget( BASE_WINDOW( instance ), "TryExecButton" );
+	base_gtk_utils_set_editable( G_OBJECT( browse_button ), editable );
 
-		entry = base_window_get_widget( BASE_WINDOW( instance ), "ShowIfRegisteredEntry" );
-		text = context ? na_object_get_show_if_registered( context ) : g_strdup( "" );
-		text = text && strlen( text ) ? text : g_strdup( "" );
-		gtk_entry_set_text( GTK_ENTRY( entry ), text );
-		g_free( text );
-		base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
+	entry = base_window_get_widget( BASE_WINDOW( instance ), "ShowIfRegisteredEntry" );
+	text = context ? na_object_get_show_if_registered( context ) : g_strdup( "" );
+	text = text && strlen( text ) ? text : g_strdup( "" );
+	gtk_entry_set_text( GTK_ENTRY( entry ), text );
+	g_free( text );
+	base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
 
-		entry = base_window_get_widget( BASE_WINDOW( instance ), "ShowIfTrueEntry" );
-		text = context ? na_object_get_show_if_true( context ) : g_strdup( "" );
-		text = text && strlen( text ) ? text : g_strdup( "" );
-		gtk_entry_set_text( GTK_ENTRY( entry ), text );
-		g_free( text );
-		base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
+	entry = base_window_get_widget( BASE_WINDOW( instance ), "ShowIfTrueEntry" );
+	text = context ? na_object_get_show_if_true( context ) : g_strdup( "" );
+	text = text && strlen( text ) ? text : g_strdup( "" );
+	gtk_entry_set_text( GTK_ENTRY( entry ), text );
+	g_free( text );
+	base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
 
-		entry = base_window_get_widget( BASE_WINDOW( instance ), "ShowIfRunningEntry" );
-		text = context ? na_object_get_show_if_running( context ) : g_strdup( "" );
-		text = text && strlen( text ) ? text : g_strdup( "" );
-		gtk_entry_set_text( GTK_ENTRY( entry ), text );
-		g_free( text );
-		base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
+	entry = base_window_get_widget( BASE_WINDOW( instance ), "ShowIfRunningEntry" );
+	text = context ? na_object_get_show_if_running( context ) : g_strdup( "" );
+	text = text && strlen( text ) ? text : g_strdup( "" );
+	gtk_entry_set_text( GTK_ENTRY( entry ), text );
+	g_free( text );
+	base_gtk_utils_set_editable( G_OBJECT( entry ), editable );
 
-		browse_button = base_window_get_widget( BASE_WINDOW( instance ), "ShowIfRunningButton" );
-		base_gtk_utils_set_editable( G_OBJECT( browse_button ), editable );
+	browse_button = base_window_get_widget( BASE_WINDOW( instance ), "ShowIfRunningButton" );
+	base_gtk_utils_set_editable( G_OBJECT( browse_button ), editable );
 
-		st_on_selection_change = FALSE;
+	data->on_selection_change = FALSE;
 
-		path = gtk_tree_path_new_first();
-		if( path ){
-			selection = gtk_tree_view_get_selection( listview );
-			gtk_tree_selection_select_path( selection, path );
-			gtk_tree_path_free( path );
-		}
+	path = gtk_tree_path_new_first();
+	if( path ){
+		selection = gtk_tree_view_get_selection( listview );
+		gtk_tree_selection_select_path( selection, path );
+		gtk_tree_path_free( path );
 	}
 }
 
@@ -476,8 +537,11 @@ on_selection_count_changed( NactIEnvironmentTab *instance )
 {
 	NAIContext *context;
 	gchar *selcount;
+	IEnvironData *data;
 
-	if( !st_on_selection_change ){
+	data = get_ienviron_data( instance );
+
+	if( !data->on_selection_change ){
 		g_object_get( G_OBJECT( instance ), MAIN_PROP_CONTEXT, &context, NULL );
 
 		if( context ){
@@ -511,6 +575,8 @@ on_show_always_toggled( GtkToggleButton *toggle_button, NactIEnvironmentTab *ins
 	gboolean active;
 	GtkTreeView *listview;
 
+	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( instance ));
+
 	g_debug( "%s: toggle_button=%p (active=%s), instance=%p",
 			thisfn,
 			( void * ) toggle_button, gtk_toggle_button_get_active( toggle_button ) ? "True":"False",
@@ -636,10 +702,13 @@ on_desktop_toggled( GtkCellRendererToggle *renderer, gchar *path, BaseWindow *wi
 	gboolean state;
 	gchar *desktop;
 	GtkWidget *show_button;
+	IEnvironData *data;
 
 	g_debug( "%s: renderer=%p, path=%s, window=%p", thisfn, ( void * ) renderer, path, ( void * ) window );
 
-	if( !st_on_selection_change ){
+	data = get_ienviron_data( NACT_IENVIRONMENT_TAB( window ));
+
+	if( !data->on_selection_change ){
 		g_object_get( G_OBJECT( window ),
 				MAIN_PROP_CONTEXT, &context, MAIN_PROP_EDITABLE, &editable,
 				NULL );
@@ -846,12 +915,14 @@ set_selection_count_selection( NactIEnvironmentTab *instance, const gchar *ope,
 static void
 dispose_selection_count_combobox( NactIEnvironmentTab *instance )
 {
-	GtkComboBox *combo;
+	GtkWidget *combo;
 	GtkTreeModel *model;
 
-	combo = GTK_COMBO_BOX( base_window_get_widget( BASE_WINDOW( instance ), "SelectionCountSigneCombobox" ));
-	model = gtk_combo_box_get_model( combo );
-	gtk_list_store_clear( GTK_LIST_STORE( model ));
+	combo = base_window_get_widget( BASE_WINDOW( instance ), "SelectionCountSigneCombobox" );
+	if( GTK_IS_COMBO_BOX( combo )){
+		model = gtk_combo_box_get_model( GTK_COMBO_BOX( combo ));
+		gtk_list_store_clear( GTK_LIST_STORE( model ));
+	}
 }
 
 static void
@@ -951,13 +1022,46 @@ setup_desktop_listview( NactIEnvironmentTab *instance, GSList *show )
 static void
 dispose_desktop_listview( NactIEnvironmentTab *instance )
 {
-	GtkTreeView *listview;
+	GtkWidget *listview;
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 
-	listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( instance ), "EnvironmentsDesktopTreeView" ));
-	model = gtk_tree_view_get_model( listview );
-	selection = gtk_tree_view_get_selection( listview );
-	gtk_tree_selection_unselect_all( selection );
-	gtk_list_store_clear( GTK_LIST_STORE( model ));
+	listview = base_window_get_widget( BASE_WINDOW( instance ), "EnvironmentsDesktopTreeView" );
+	if( GTK_IS_TREE_VIEW( listview )){
+		model = gtk_tree_view_get_model( GTK_TREE_VIEW( listview ));
+		selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( listview ));
+		gtk_tree_selection_unselect_all( selection );
+		gtk_list_store_clear( GTK_LIST_STORE( model ));
+	}
+}
+
+static IEnvironData *
+get_ienviron_data( NactIEnvironmentTab *instance )
+{
+	IEnvironData *data;
+
+	data = ( IEnvironData * ) g_object_get_data( G_OBJECT( instance ), IENVIRON_TAB_PROP_DATA );
+
+	if( !data ){
+		data = g_new0( IEnvironData, 1 );
+		g_object_set_data( G_OBJECT( instance ), IENVIRON_TAB_PROP_DATA, data );
+	}
+
+	return( data );
+}
+
+static void
+on_instance_finalized( gpointer user_data, NactIEnvironmentTab *instance )
+{
+	static const gchar *thisfn = "nact_ienvironment_tab_on_instance_finalized";
+	IEnvironData *data;
+
+	g_debug( "%s: instance=%p, user_data=%p", thisfn, ( void * ) instance, ( void * ) user_data );
+
+	data = get_ienviron_data( instance );
+	data->on_selection_change = TRUE;
+	dispose_selection_count_combobox( instance );
+	dispose_desktop_listview( instance );
+
+	g_free( data );
 }
