@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 #include <core/na-iprefs.h>
 #include <core/na-pivot.h>
 
-#include "base-marshal.h"
+#include "base-isession.h"
 #include "nact-iaction-tab.h"
 #include "nact-icommand-tab.h"
 #include "nact-ibasenames-tab.h"
@@ -55,6 +55,7 @@
 #include "nact-main-tab.h"
 #include "nact-main-statusbar.h"
 #include "nact-main-window.h"
+#include "nact-marshal.h"
 #include "nact-menubar.h"
 #include "nact-tree-view.h"
 #include "nact-confirm-logout.h"
@@ -148,10 +149,11 @@ enum {
 	MAIN_ITEM_UPDATED,
 	TAB_ITEM_UPDATED,
 	SELECTION_CHANGED,
+	CONTEXT_MENU,
 	LAST_SIGNAL
 };
 
-static const gchar     *st_xmlui_filename         = PKGDATADIR "/nautilus-actions-config-tool.ui";
+static const gchar     *st_xmlui_filename         = PKGUIDIR "/nautilus-actions-config-tool.ui";
 static const gchar     *st_toplevel_name          = "MainWindow";
 static const gchar     *st_wsp_name               = NA_IPREFS_MAIN_WINDOW_WSP;
 
@@ -161,26 +163,26 @@ static gint             st_signals[ LAST_SIGNAL ] = { 0 };
 
 static GType      register_type( void );
 static void       class_init( NactMainWindowClass *klass );
-static void       iaction_tab_iface_init( NactIActionTabInterface *iface );
-static void       icommand_tab_iface_init( NactICommandTabInterface *iface );
-static void       ibasenames_tab_iface_init( NactIBasenamesTabInterface *iface );
-static void       imimetypes_tab_iface_init( NactIMimetypesTabInterface *iface );
-static void       ifolders_tab_iface_init( NactIFoldersTabInterface *iface );
-static void       ischemes_tab_iface_init( NactISchemesTabInterface *iface );
-static void       icapabilities_tab_iface_init( NactICapabilitiesTabInterface *iface );
-static void       ienvironment_tab_iface_init( NactIEnvironmentTabInterface *iface );
-static void       iexecution_tab_iface_init( NactIExecutionTabInterface *iface );
-static void       iproperties_tab_iface_init( NactIPropertiesTabInterface *iface );
+static void       iaction_tab_iface_init( NactIActionTabInterface *iface, void *user_data );
+static void       icommand_tab_iface_init( NactICommandTabInterface *iface, void *user_data );
+static void       ibasenames_tab_iface_init( NactIBasenamesTabInterface *iface, void *user_data );
+static void       imimetypes_tab_iface_init( NactIMimetypesTabInterface *iface, void *user_data );
+static void       ifolders_tab_iface_init( NactIFoldersTabInterface *iface, void *user_data );
+static void       ischemes_tab_iface_init( NactISchemesTabInterface *iface, void *user_data );
+static void       icapabilities_tab_iface_init( NactICapabilitiesTabInterface *iface, void *user_data );
+static void       ienvironment_tab_iface_init( NactIEnvironmentTabInterface *iface, void *user_data );
+static void       iexecution_tab_iface_init( NactIExecutionTabInterface *iface, void *user_data );
+static void       iproperties_tab_iface_init( NactIPropertiesTabInterface *iface, void *user_data );
 static void       instance_init( GTypeInstance *instance, gpointer klass );
 static void       instance_get_property( GObject *object, guint property_id, GValue *value, GParamSpec *spec );
 static void       instance_set_property( GObject *object, guint property_id, const GValue *value, GParamSpec *spec );
-static void       instance_constructed( GObject *application );
-static void       instance_dispose( GObject *application );
-static void       instance_finalize( GObject *application );
+static void       instance_constructed( GObject *window );
+static void       instance_dispose( GObject *window );
+static void       instance_finalize( GObject *window );
 
-static void       on_base_initialize_gtk_toplevel( NactMainWindow *window, GtkWindow *toplevel, gpointer user_data );
-static void       on_base_initialize_base_window( NactMainWindow *window, gpointer user_data );
-static void       on_base_all_widgets_showed( NactMainWindow *window, gpointer user_data );
+static void       on_base_initialize_gtk( NactMainWindow *window, GtkWindow *toplevel, gpointer user_data );
+static void       on_base_initialize_window( NactMainWindow *window, gpointer user_data );
+static void       on_base_show_widgets( NactMainWindow *window, gpointer user_data );
 
 static void       on_block_items_changed_timeout( NactMainWindow *window );
 static void       on_tree_view_modified_status_changed( NactMainWindow *window, gboolean is_modified, gpointer user_data );
@@ -199,7 +201,7 @@ static gboolean   confirm_for_giveup_from_menu( const NactMainWindow *window );
 static void       load_or_reload_items( NactMainWindow *window );
 
 /* application termination */
-static gboolean   on_base_is_willing_to_quit( const BaseWindow *window, gconstpointer user_data );
+static gboolean   on_base_quit_requested( NactApplication *application, NactMainWindow *window );
 static gboolean   on_delete_event( GtkWidget *toplevel, GdkEvent *event, NactMainWindow *window );
 static gboolean   warn_modified( NactMainWindow *window );
 
@@ -295,27 +297,27 @@ register_type( void )
 
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( BASE_WINDOW_TYPE, "NactMainWindow", &info, 0 );
+	type = g_type_register_static( BASE_TYPE_WINDOW, "NactMainWindow", &info, 0 );
 
-	g_type_add_interface_static( type, NACT_IACTION_TAB_TYPE, &iaction_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_IACTION_TAB, &iaction_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_ICOMMAND_TAB_TYPE, &icommand_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_ICOMMAND_TAB, &icommand_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_IBASENAMES_TAB_TYPE, &ibasenames_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_IBASENAMES_TAB, &ibasenames_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_IMIMETYPES_TAB_TYPE, &imimetypes_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_IMIMETYPES_TAB, &imimetypes_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_IFOLDERS_TAB_TYPE, &ifolders_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_IFOLDERS_TAB, &ifolders_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_ISCHEMES_TAB_TYPE, &ischemes_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_ISCHEMES_TAB, &ischemes_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_ICAPABILITIES_TAB_TYPE, &icapabilities_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_ICAPABILITIES_TAB, &icapabilities_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_IENVIRONMENT_TAB_TYPE, &ienvironment_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_IENVIRONMENT_TAB, &ienvironment_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_IEXECUTION_TAB_TYPE, &iexecution_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_IEXECUTION_TAB, &iexecution_tab_iface_info );
 
-	g_type_add_interface_static( type, NACT_IPROPERTIES_TAB_TYPE, &iproperties_tab_iface_info );
+	g_type_add_interface_static( type, NACT_TYPE_IPROPERTIES_TAB, &iproperties_tab_iface_info );
 
 	return( type );
 }
@@ -340,36 +342,38 @@ class_init( NactMainWindowClass *klass )
 	g_object_class_install_property( object_class, MAIN_PROP_ITEM_ID,
 			g_param_spec_pointer(
 					MAIN_PROP_ITEM,
-					"Current NAObjectItem",
-					"A pointer to the currently edited NAObjectItem, an action or a menu",
+					_( "Current NAObjectItem" ),
+					_( "A pointer to the currently edited NAObjectItem, an action or a menu" ),
 					G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
 
 	g_object_class_install_property( object_class, MAIN_PROP_PROFILE_ID,
 			g_param_spec_pointer(
 					MAIN_PROP_PROFILE,
-					"Current NAObjectProfile",
-					"A pointer to the currently edited NAObjectProfile",
+					_( "Current NAObjectProfile" ),
+					_( "A pointer to the currently edited NAObjectProfile" ),
 					G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
 
 	g_object_class_install_property( object_class, MAIN_PROP_CONTEXT_ID,
 			g_param_spec_pointer(
 					MAIN_PROP_CONTEXT,
-					"Current NAIContext",
-					"A pointer to the currently edited NAIContext",
+					_( "Current NAIContext" ),
+					_( "A pointer to the currently edited NAIContext" ),
 					G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
 
 	g_object_class_install_property( object_class, MAIN_PROP_EDITABLE_ID,
 			g_param_spec_boolean(
 					MAIN_PROP_EDITABLE,
-					"Editable item ?",
-					"Whether the item will be able to be updated against its I/O provider", FALSE,
+					_( "Editable item ?" ),
+					_( "Whether the item will be able to be updated against its I/O provider" ),
+					FALSE,
 					G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
 
 	g_object_class_install_property( object_class, MAIN_PROP_REASON_ID,
 			g_param_spec_int(
 					MAIN_PROP_REASON,
-					"No edition reason",
-					"Why is this item not editable", 0, 255, 0,
+					_( "No edition reason" ),
+					_( "Why is this item not editable" ),
+					0, 255, 0,
 					G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
 
 	klass->private = g_new0( NactMainWindowClassPrivate, 1 );
@@ -451,86 +455,112 @@ class_init( NactMainWindowClass *klass )
 			G_TYPE_NONE,
 			1,
 			G_TYPE_POINTER );
+
+	/**
+	 * NactMainWindow::main-signal-open-popup
+	 *
+	 * This signal is emitted on the BaseWindow parent when the user right
+	 * clicks somewhere (on an active zone).
+	 *
+	 * Signal args:
+	 * - the GdkEvent
+	 * - the popup name to be opened.
+	 *
+	 * Handler prototype:
+	 * void ( *handler )( BaseWindow *window, GdkEvent *event, const gchar *popup_name, gpointer user_data );
+	 */
+	st_signals[ CONTEXT_MENU ] = g_signal_new(
+			MAIN_SIGNAL_CONTEXT_MENU,
+			G_TYPE_OBJECT,
+			G_SIGNAL_RUN_LAST,
+			0,
+			NULL,
+			NULL,
+			nact_cclosure_marshal_VOID__POINTER_STRING,
+			G_TYPE_NONE,
+			2,
+			G_TYPE_POINTER,
+			G_TYPE_STRING);
 }
 
 static void
-iaction_tab_iface_init( NactIActionTabInterface *iface )
+iaction_tab_iface_init( NactIActionTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_iaction_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-icommand_tab_iface_init( NactICommandTabInterface *iface )
+icommand_tab_iface_init( NactICommandTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_icommand_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-ibasenames_tab_iface_init( NactIBasenamesTabInterface *iface )
+ibasenames_tab_iface_init( NactIBasenamesTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_ibasenames_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-imimetypes_tab_iface_init( NactIMimetypesTabInterface *iface )
+imimetypes_tab_iface_init( NactIMimetypesTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_imimetypes_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-ifolders_tab_iface_init( NactIFoldersTabInterface *iface )
+ifolders_tab_iface_init( NactIFoldersTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_ifolders_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-ischemes_tab_iface_init( NactISchemesTabInterface *iface )
+ischemes_tab_iface_init( NactISchemesTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_ischemes_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-icapabilities_tab_iface_init( NactICapabilitiesTabInterface *iface )
+icapabilities_tab_iface_init( NactICapabilitiesTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_icapabilities_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-ienvironment_tab_iface_init( NactIEnvironmentTabInterface *iface )
+ienvironment_tab_iface_init( NactIEnvironmentTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_ienvironment_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-iexecution_tab_iface_init( NactIExecutionTabInterface *iface )
+iexecution_tab_iface_init( NactIExecutionTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_iexecution_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
-iproperties_tab_iface_init( NactIPropertiesTabInterface *iface )
+iproperties_tab_iface_init( NactIPropertiesTabInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_iproperties_tab_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
@@ -538,6 +568,7 @@ instance_init( GTypeInstance *instance, gpointer klass )
 {
 	static const gchar *thisfn = "nact_main_window_instance_init";
 	NactMainWindow *self;
+	NactMainWindowPrivate *priv;
 
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( instance ));
 
@@ -545,17 +576,16 @@ instance_init( GTypeInstance *instance, gpointer klass )
 			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
 
 	self = NACT_MAIN_WINDOW( instance );
-
 	self->private = g_new0( NactMainWindowPrivate, 1 );
-
-	self->private->dispose_has_run = FALSE;
+	priv = self->private;
+	priv->dispose_has_run = FALSE;
 
 	/* initialize timeout parameters when blocking 'pivot-items-changed' handler
 	 */
-	self->private->pivot_timeout.timeout = st_burst_timeout;
-	self->private->pivot_timeout.handler = ( NATimeoutFunc ) on_block_items_changed_timeout;
-	self->private->pivot_timeout.user_data = self;
-	self->private->pivot_timeout.source_id = 0;
+	priv->pivot_timeout.timeout = st_burst_timeout;
+	priv->pivot_timeout.handler = ( NATimeoutFunc ) on_block_items_changed_timeout;
+	priv->pivot_timeout.user_data = self;
+	priv->pivot_timeout.source_id = 0;
 }
 
 static void
@@ -638,47 +668,81 @@ static void
 instance_constructed( GObject *window )
 {
 	static const gchar *thisfn = "nact_main_window_instance_constructed";
-	NactMainWindow *self;
+	NactMainWindowPrivate *priv;
 	NactApplication *application;
 
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 
-	self = NACT_MAIN_WINDOW( window );
+	priv = NACT_MAIN_WINDOW( window )->private;
 
-	if( !self->private->dispose_has_run ){
-		g_debug( "%s: window=%p (%s)", thisfn, ( void * ) window, G_OBJECT_TYPE_NAME( window ));
-
-		/* first connect to BaseWindow signals
-		 * so that convenience objects instanciated later will have this same signal
-		 * triggered after the one of NactMainWindow
-		 */
-		base_window_signal_connect( BASE_WINDOW( window ),
-				G_OBJECT( window ), BASE_SIGNAL_INITIALIZE_GTK, G_CALLBACK( on_base_initialize_gtk_toplevel ));
-
-		base_window_signal_connect( BASE_WINDOW( window ),
-				G_OBJECT( window ), BASE_SIGNAL_INITIALIZE_WINDOW, G_CALLBACK( on_base_initialize_base_window ));
-
-		base_window_signal_connect( BASE_WINDOW( window ),
-				G_OBJECT( window ), BASE_SIGNAL_ALL_WIDGETS_SHOWED, G_CALLBACK( on_base_all_widgets_showed ));
-
-		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
-		self->private->updater = nact_application_get_updater( application );
-
-		self->private->pivot_handler_id = base_window_signal_connect( BASE_WINDOW( window ),
-				G_OBJECT( self->private->updater ), PIVOT_SIGNAL_ITEMS_CHANGED, G_CALLBACK( on_pivot_items_changed ));
-
-		base_window_signal_connect( BASE_WINDOW( window ),
-				G_OBJECT( window ), TAB_UPDATABLE_SIGNAL_ITEM_UPDATED, G_CALLBACK( on_tab_updatable_item_updated ));
-
-		/* create the menubar and other convenience objects
-		 */
-		self->private->menubar = nact_menubar_new( BASE_WINDOW( window ));
-		self->private->clipboard = nact_clipboard_new( BASE_WINDOW( window ));
+	if( !priv->dispose_has_run ){
 
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->constructed ){
 			G_OBJECT_CLASS( st_parent_class )->constructed( window );
 		}
+
+		g_debug( "%s: window=%p (%s)", thisfn, ( void * ) window, G_OBJECT_TYPE_NAME( window ));
+
+		/* connect to BaseWindow signals
+		 * so that convenience objects instanciated later will have this same signal
+		 * triggered after the one of NactMainWindow
+		 */
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( window ),
+				BASE_SIGNAL_INITIALIZE_GTK,
+				G_CALLBACK( on_base_initialize_gtk ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( window ),
+				BASE_SIGNAL_INITIALIZE_WINDOW,
+				G_CALLBACK( on_base_initialize_window ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( window ),
+				BASE_SIGNAL_SHOW_WIDGETS,
+				G_CALLBACK( on_base_show_widgets ));
+
+		/* monitor the items stored on the disk for modifications
+		 * outside of this application
+		 */
+		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+		priv->updater = nact_application_get_updater( application );
+
+		priv->pivot_handler_id = base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( priv->updater ),
+				PIVOT_SIGNAL_ITEMS_CHANGED,
+				G_CALLBACK( on_pivot_items_changed ));
+
+		/* monitor the updates which originates from each property tab
+		 */
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( window ),
+				TAB_UPDATABLE_SIGNAL_ITEM_UPDATED,
+				G_CALLBACK( on_tab_updatable_item_updated ));
+
+		/* create the menubar and other convenience objects
+		 */
+		priv->menubar = nact_menubar_new( BASE_WINDOW( window ));
+		priv->clipboard = nact_clipboard_new( BASE_WINDOW( window ));
+
+		/* initialize the notebook interfaces
+		 */
+		nact_iaction_tab_init( NACT_IACTION_TAB( window ));
+		nact_icommand_tab_init( NACT_ICOMMAND_TAB( window ));
+		nact_ibasenames_tab_init( NACT_IBASENAMES_TAB( window ));
+		nact_imimetypes_tab_init( NACT_IMIMETYPES_TAB( window ));
+		nact_ifolders_tab_init( NACT_IFOLDERS_TAB( window ));
+		nact_ischemes_tab_init( NACT_ISCHEMES_TAB( window ));
+		nact_icapabilities_tab_init( NACT_ICAPABILITIES_TAB( window ));
+		nact_ienvironment_tab_init( NACT_IENVIRONMENT_TAB( window ));
+		nact_iexecution_tab_init( NACT_IEXECUTION_TAB( window ));
+		nact_iproperties_tab_init( NACT_IPROPERTIES_TAB( window ));
 	}
 }
 
@@ -689,6 +753,7 @@ instance_dispose( GObject *window )
 	NactMainWindow *self;
 	GtkWidget *pane;
 	gint pos;
+	GtkNotebook *notebook;
 
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 
@@ -699,6 +764,8 @@ instance_dispose( GObject *window )
 
 		self->private->dispose_has_run = TRUE;
 
+		gtk_main_quit();
+
 		g_object_unref( self->private->clipboard );
 		g_object_unref( self->private->menubar );
 
@@ -706,16 +773,9 @@ instance_dispose( GObject *window )
 		pos = gtk_paned_get_position( GTK_PANED( pane ));
 		na_settings_set_uint( NA_IPREFS_MAIN_PANED, pos );
 
-		nact_iaction_tab_dispose( NACT_IACTION_TAB( window ));
-		nact_icommand_tab_dispose( NACT_ICOMMAND_TAB( window ));
-		nact_ibasenames_tab_dispose( NACT_IBASENAMES_TAB( window ));
-		nact_imimetypes_tab_dispose( NACT_IMIMETYPES_TAB( window ));
-		nact_ifolders_tab_dispose( NACT_IFOLDERS_TAB( window ));
-		nact_ischemes_tab_dispose( NACT_ISCHEMES_TAB( window ));
-		nact_icapabilities_tab_dispose( NACT_ICAPABILITIES_TAB( window ));
-		nact_ienvironment_tab_dispose( NACT_IENVIRONMENT_TAB( window ));
-		nact_iexecution_tab_dispose( NACT_IEXECUTION_TAB( window ));
-		nact_iproperties_tab_dispose( NACT_IPROPERTIES_TAB( window ));
+		notebook = GTK_NOTEBOOK( base_window_get_widget( BASE_WINDOW( window ), "MainNotebook" ));
+		pos = gtk_notebook_get_tab_pos( notebook );
+		na_iprefs_set_tabs_pos( pos );
 
 		/* unref items view at last as gtk_tree_model_store_clear() will
 		 * finalize all objects, thus invaliditing all our references
@@ -759,13 +819,19 @@ nact_main_window_new( const NactApplication *application )
 
 	g_return_val_if_fail( NACT_IS_APPLICATION( application ), NULL );
 
-	window = g_object_new( NACT_MAIN_WINDOW_TYPE,
-			BASE_PROP_APPLICATION,    application,
-			BASE_PROP_XMLUI_FILENAME, st_xmlui_filename,
-			BASE_PROP_TOPLEVEL_NAME,  st_toplevel_name,
-			BASE_PROP_WSP_NAME,       st_wsp_name,
+	window = g_object_new( NACT_TYPE_MAIN_WINDOW,
+			BASE_PROP_APPLICATION,        application,
+			BASE_PROP_XMLUI_FILENAME,     st_xmlui_filename,
+			BASE_PROP_TOPLEVEL_NAME,      st_toplevel_name,
+			BASE_PROP_WSP_NAME,           st_wsp_name,
+			BASE_PROP_DESTROY_ON_DISPOSE, TRUE,
 			NULL );
 
+	if( !base_window_init( BASE_WINDOW( window ))){
+		g_object_unref( window );
+		window = NULL;
+	}
+
 	return( window );
 }
 
@@ -776,59 +842,58 @@ nact_main_window_new( const NactApplication *application )
  * is the same than quitting the application
  */
 static void
-on_base_initialize_gtk_toplevel( NactMainWindow *window, GtkWindow *toplevel, gpointer user_data )
+on_base_initialize_gtk( NactMainWindow *window, GtkWindow *toplevel, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_main_window_on_base_initialize_gtk_toplevel";
+	static const gchar *thisfn = "nact_main_window_on_base_initialize_gtk";
 	GtkWidget *tree_parent;
+	GtkNotebook *notebook;
 
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 
 	if( !window->private->dispose_has_run ){
+
 		g_debug( "%s: window=%p, toplevel=%p, user_data=%p",
-				thisfn, ( void * ) window, ( void * ) toplevel, ( void * ) user_data );
+				thisfn,
+				( void * ) window,
+				( void * ) toplevel,
+				( void * ) user_data );
 
 		/* create the tree view which will create itself its own tree model
 		 */
 		tree_parent = base_window_get_widget( BASE_WINDOW( window ), "MainVBox" );
 		g_debug( "%s: tree_parent=%p (%s)", thisfn, ( void * ) tree_parent, G_OBJECT_TYPE_NAME( tree_parent ));
 		window->private->items_view = nact_tree_view_new(
-				BASE_WINDOW( window ), GTK_CONTAINER( tree_parent ),
-				"ActionsList", TREE_MODE_EDITION );
-
-		base_window_signal_connect( BASE_WINDOW( window ),
-				G_OBJECT( window ), TREE_SIGNAL_SELECTION_CHANGED, G_CALLBACK( on_tree_view_selection_changed ));
-
-		base_window_signal_connect( BASE_WINDOW( window ),
-				G_OBJECT( window ), TREE_SIGNAL_MODIFIED_STATUS_CHANGED, G_CALLBACK( on_tree_view_modified_status_changed ));
-
-		/* then initialize notebook
-		 */
-		nact_iaction_tab_initial_load_toplevel( NACT_IACTION_TAB( window ));
-		nact_icommand_tab_initial_load_toplevel( NACT_ICOMMAND_TAB( window ));
-		nact_ibasenames_tab_initial_load_toplevel( NACT_IBASENAMES_TAB( window ));
-		nact_imimetypes_tab_initial_load_toplevel( NACT_IMIMETYPES_TAB( window ));
-		nact_ifolders_tab_initial_load_toplevel( NACT_IFOLDERS_TAB( window ));
-		nact_ischemes_tab_initial_load_toplevel( NACT_ISCHEMES_TAB( window ));
-		nact_icapabilities_tab_initial_load_toplevel( NACT_ICAPABILITIES_TAB( window ));
-		nact_ienvironment_tab_initial_load_toplevel( NACT_IENVIRONMENT_TAB( window ));
-		nact_iexecution_tab_initial_load_toplevel( NACT_IEXECUTION_TAB( window ));
-		nact_iproperties_tab_initial_load_toplevel( NACT_IPROPERTIES_TAB( window ));
+				BASE_WINDOW( window ),
+				GTK_CONTAINER( tree_parent ),
+				"ActionsList",
+				TREE_MODE_EDITION );
 
 		nact_main_statusbar_initialize_gtk_toplevel( window );
+
+		/* enable popup menu on  the notebook
+		 */
+		notebook = GTK_NOTEBOOK( base_window_get_widget( BASE_WINDOW( window ), "MainNotebook" ));
+		gtk_notebook_popup_enable( notebook );
 	}
 }
 
 static void
-on_base_initialize_base_window( NactMainWindow *window, gpointer user_data )
+on_base_initialize_window( NactMainWindow *window, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_main_window_on_base_initialize_base_window";
+	static const gchar *thisfn = "nact_main_window_on_base_initialize_window";
 	guint pos;
 	GtkWidget *pane;
+	NactApplication *application;
+	GtkNotebook *notebook;
 
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 
 	if( !window->private->dispose_has_run ){
-		g_debug( "%s: window=%p, user_data=%p", thisfn, ( void * ) window, ( void * ) user_data );
+
+		g_debug( "%s: window=%p, user_data=%p",
+				thisfn,
+				( void * ) window,
+				( void * ) user_data );
 
 		pos = na_settings_get_uint( NA_IPREFS_MAIN_PANED, NULL, NULL );
 		if( pos ){
@@ -836,59 +901,56 @@ on_base_initialize_base_window( NactMainWindow *window, gpointer user_data )
 			gtk_paned_set_position( GTK_PANED( pane ), pos );
 		}
 
-		nact_iaction_tab_runtime_init_toplevel( NACT_IACTION_TAB( window ));
-		nact_icommand_tab_runtime_init_toplevel( NACT_ICOMMAND_TAB( window ));
-		nact_ibasenames_tab_runtime_init_toplevel( NACT_IBASENAMES_TAB( window ));
-		nact_imimetypes_tab_runtime_init_toplevel( NACT_IMIMETYPES_TAB( window ));
-		nact_ifolders_tab_runtime_init_toplevel( NACT_IFOLDERS_TAB( window ));
-		nact_ischemes_tab_runtime_init_toplevel( NACT_ISCHEMES_TAB( window ));
-		nact_icapabilities_tab_runtime_init_toplevel( NACT_ICAPABILITIES_TAB( window ));
-		nact_ienvironment_tab_runtime_init_toplevel( NACT_IENVIRONMENT_TAB( window ));
-		nact_iexecution_tab_runtime_init_toplevel( NACT_IEXECUTION_TAB( window ));
-		nact_iproperties_tab_runtime_init_toplevel( NACT_IPROPERTIES_TAB( window ));
-
 		/* terminate the application by clicking the top right [X] button
 		 */
-		base_window_signal_connect( BASE_WINDOW( window ),
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
 				G_OBJECT( base_window_get_gtk_toplevel( BASE_WINDOW( window ))),
-				"delete-event", G_CALLBACK( on_delete_event ));
+				"delete-event",
+				G_CALLBACK( on_delete_event ));
 
 		/* is willing to quit ?
+		 * connect to the signal of the BaseISession interface
 		 */
-		base_window_signal_connect( BASE_WINDOW( window ),
-				G_OBJECT( window ), BASE_SIGNAL_WILLING_TO_QUIT, G_CALLBACK( on_base_is_willing_to_quit ));
+		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( window )));
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( application ),
+				BASE_SIGNAL_QUIT_REQUESTED,
+				G_CALLBACK( on_base_quit_requested ));
+
+		/* connect to treeview signals
+		 */
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( window ),
+				MAIN_SIGNAL_SELECTION_CHANGED,
+				G_CALLBACK( on_tree_view_selection_changed ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( window ),
+				G_OBJECT( window ),
+				TREE_SIGNAL_MODIFIED_STATUS_CHANGED,
+				G_CALLBACK( on_tree_view_modified_status_changed ));
+
+		/* restore the notebook tabs position
+		 */
+		pos = na_iprefs_get_tabs_pos( NULL );
+		notebook = GTK_NOTEBOOK( base_window_get_widget( BASE_WINDOW( window ), "MainNotebook" ));
+		gtk_notebook_set_tab_pos( notebook, pos );
 	}
 }
 
 static void
-on_base_all_widgets_showed( NactMainWindow *window, gpointer user_data )
+on_base_show_widgets( NactMainWindow *window, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_main_window_on_base_all_widgets_showed";
+	static const gchar *thisfn = "nact_main_window_on_base_show_widgets";
 
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
-	g_return_if_fail( NACT_IS_IACTION_TAB( window ));
-	g_return_if_fail( NACT_IS_ICOMMAND_TAB( window ));
-	g_return_if_fail( NACT_IS_IBASENAMES_TAB( window ));
-	g_return_if_fail( NACT_IS_IMIMETYPES_TAB( window ));
-	g_return_if_fail( NACT_IS_IFOLDERS_TAB( window ));
-	g_return_if_fail( NACT_IS_ISCHEMES_TAB( window ));
-	g_return_if_fail( NACT_IS_IENVIRONMENT_TAB( window ));
-	g_return_if_fail( NACT_IS_IEXECUTION_TAB( window ));
-	g_return_if_fail( NACT_IS_IPROPERTIES_TAB( window ));
 
 	if( !window->private->dispose_has_run ){
-		g_debug( "%s: window=%p, user_data=%p", thisfn, ( void * ) window, ( void * ) user_data );
 
-		nact_iaction_tab_all_widgets_showed( NACT_IACTION_TAB( window ));
-		nact_icommand_tab_all_widgets_showed( NACT_ICOMMAND_TAB( window ));
-		nact_ibasenames_tab_all_widgets_showed( NACT_IBASENAMES_TAB( window ));
-		nact_imimetypes_tab_all_widgets_showed( NACT_IMIMETYPES_TAB( window ));
-		nact_ifolders_tab_all_widgets_showed( NACT_IFOLDERS_TAB( window ));
-		nact_ischemes_tab_all_widgets_showed( NACT_ISCHEMES_TAB( window ));
-		nact_icapabilities_tab_all_widgets_showed( NACT_ICAPABILITIES_TAB( window ));
-		nact_ienvironment_tab_all_widgets_showed( NACT_IENVIRONMENT_TAB( window ));
-		nact_iexecution_tab_all_widgets_showed( NACT_IEXECUTION_TAB( window ));
-		nact_iproperties_tab_all_widgets_showed( NACT_IPROPERTIES_TAB( window ));
+		g_debug( "%s: window=%p, user_data=%p", thisfn, ( void * ) window, ( void * ) user_data );
 
 		load_or_reload_items( window );
 	}
@@ -1040,9 +1102,6 @@ on_tree_view_selection_changed( NactMainWindow *window, GList *selected_items, g
 		}
 
 		setup_dialog_title( window );
-
-		g_signal_emit_by_name( G_OBJECT( window ),
-				MAIN_SIGNAL_SELECTION_CHANGED, na_object_copyref_items( selected_items ));
 	}
 }
 
@@ -1103,6 +1162,7 @@ setup_current_selection( NactMainWindow *window, NAObjectId *selected_row )
 
 	if( NA_IS_OBJECT_PROFILE( selected_row )){
 		window->private->current_profile = NA_OBJECT_PROFILE( selected_row );
+		window->private->current_context = NA_ICONTEXT( selected_row );
 		window->private->current_item = NA_OBJECT_ITEM( na_object_get_parent( selected_row ));
 
 	} else {
@@ -1308,20 +1368,21 @@ nact_main_window_quit( NactMainWindow *window )
  * this will also stop the emission of the signal (i.e. the first FALSE wins)
  */
 static gboolean
-on_base_is_willing_to_quit( const BaseWindow *window, gconstpointer user_data )
+on_base_quit_requested( NactApplication *application, NactMainWindow *window )
 {
-	static const gchar *thisfn = "nact_main_window_on_base_is_willing_to_quit";
+	static const gchar *thisfn = "nact_main_window_on_base_quit_requested";
 	gboolean willing_to;
 
 	g_return_val_if_fail( NACT_IS_MAIN_WINDOW( window ), TRUE );
 
 	willing_to = TRUE;
 
-	if( !NACT_MAIN_WINDOW( window )->private->dispose_has_run ){
-		g_debug( "%s (virtual): window=%p", thisfn, ( void * ) window );
+	if( !window->private->dispose_has_run ){
 
-		if( NACT_MAIN_WINDOW( window )->private->is_tree_modified ){
-			willing_to = nact_confirm_logout_run( NACT_MAIN_WINDOW( window ));
+		g_debug( "%s: application=%p, window=%p", thisfn, ( void * ) application, ( void * ) window );
+
+		if( window->private->is_tree_modified ){
+			willing_to = nact_confirm_logout_run( window );
 		}
 	}
 
