@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,7 +37,6 @@
 #include <api/na-core-utils.h>
 
 #include <core/na-io-provider.h>
-#include <core/na-iprefs.h>
 
 #include "nact-application.h"
 #include "nact-main-tab.h"
@@ -138,7 +137,9 @@ nact_menubar_edit_on_update_sensitivities( const NactMenubar *bar )
 		if( paste_into_enabled ){
 			selected_action = NA_OBJECT( bar->private->selected_items->data );
 			paste_into_enabled &= NA_IS_OBJECT_ACTION( selected_action );
-			paste_into_enabled &= na_object_is_finally_writable( selected_action, NULL );
+			if( paste_into_enabled ){
+				paste_into_enabled &= na_object_is_finally_writable( selected_action, NULL );
+			}
 		}
 	} else {
 		paste_into_enabled &= bar->private->has_writable_providers;
@@ -252,7 +253,6 @@ void
 nact_menubar_edit_on_copy( GtkAction *gtk_action, BaseWindow *window )
 {
 	static const gchar *thisfn = "nact_menubar_edit_on_copy";
-	GList *items;
 	NactClipboard *clipboard;
 
 	BAR_WINDOW_VOID( window );
@@ -261,11 +261,9 @@ nact_menubar_edit_on_copy( GtkAction *gtk_action, BaseWindow *window )
 	g_return_if_fail( GTK_IS_ACTION( gtk_action ));
 	g_return_if_fail( NACT_IS_MAIN_WINDOW( window ));
 
-	items = na_object_copyref_items( bar->private->selected_items );
 	clipboard = nact_main_window_get_clipboard( NACT_MAIN_WINDOW( window ));
-	nact_clipboard_primary_set( clipboard, items, CLIPBOARD_MODE_COPY );
+	nact_clipboard_primary_set( clipboard, bar->private->selected_items, CLIPBOARD_MODE_COPY );
 	update_clipboard_counters( window );
-	na_object_free_items( items );
 
 	g_signal_emit_by_name( bar, MENUBAR_SIGNAL_UPDATE_SENSITIVITIES );
 }
@@ -407,7 +405,7 @@ nact_menubar_edit_on_duplicate( GtkAction *gtk_action, BaseWindow *window )
 	items = na_object_copyref_items( bar->private->selected_items );
 
 	for( it = items ; it ; it = it->next ){
-		obj = NA_OBJECT( na_object_duplicate( it->data ));
+		obj = NA_OBJECT( na_object_duplicate( it->data, DUPLICATE_REC ));
 		action = NULL;
 
 		/* duplicating a profile
@@ -492,11 +490,18 @@ get_deletables( NAUpdater *updater, GList *selected, GSList **non_deletables )
 	GList *subitems;
 	GSList *sub_deletables;
 	guint reason;
+	NAObjectItem *item;
 
 	to_delete = NULL;
 	for( it = selected ; it ; it = it->next ){
 
-		if( !na_object_is_finally_writable( it->data, &reason )){
+		if( NA_IS_OBJECT_PROFILE( it->data )){
+			item = na_object_get_parent( it->data );
+		} else {
+			item = NA_OBJECT_ITEM( it->data );
+		}
+
+		if( !na_object_is_finally_writable( item, &reason )){
 			*non_deletables = g_slist_prepend(
 					*non_deletables, add_non_deletable_msg( NA_OBJECT_ITEM( it->data ), reason ));
 			continue;
