@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,11 +38,16 @@
 #include <api/na-iimporter.h>
 
 #include <core/na-desktop-environment.h>
+#include <core/na-exporter.h>
+#include <core/na-export-format.h>
+#include <core/na-gtk-utils.h>
+#include <core/na-import-mode.h>
+#include <core/na-importer.h>
+#include <core/na-ioptions-list.h>
 #include <core/na-iprefs.h>
 #include <core/na-tokens.h>
 
 #include "nact-application.h"
-#include "nact-export-format.h"
 #include "base-gtk-utils.h"
 #include "nact-schemes-list.h"
 #include "nact-providers-list.h"
@@ -110,61 +115,60 @@ enum {
 };
 
 /* i18n: the user is not willing to identify his current desktop environment,
- *       and prefers rely onthe runtime detection */
+ *       and prefers rely on the runtime detection */
 static const NADesktopEnv st_no_desktop     = { "None", N_( "Rely on runtime detection" ) };
 
-static const gchar       *st_xmlui_filename = PKGDATADIR "/nact-preferences.ui";
+static const gchar       *st_xmlui_filename = PKGUIDIR "/nact-preferences.ui";
 static const gchar       *st_toplevel_name  = "PreferencesDialog";
 static const gchar       *st_wsp_name       = NA_IPREFS_PREFERENCES_WSP;
 
 static GObjectClass      *st_parent_class   = NULL;
 static guint              st_last_tab       = 0;
 
-static GType    register_type( void );
-static void     class_init( NactPreferencesEditorClass *klass );
-static void     instance_init( GTypeInstance *instance, gpointer klass );
-static void     instance_dispose( GObject *dialog );
-static void     instance_finalize( GObject *dialog );
+static GType      register_type( void );
+static void       class_init( NactPreferencesEditorClass *klass );
+static void       ioptions_list_iface_init( NAIOptionsListInterface *iface, void *user_data );
+static GList     *ioptions_list_get_options( const NAIOptionsList *instance, GtkWidget *container );
+static void       ioptions_list_free_options( const NAIOptionsList *instance, GtkWidget *container, GList *options );
+static NAIOption *ioptions_list_get_ask_option( const NAIOptionsList *instance, GtkWidget *container );
+static void       instance_init( GTypeInstance *instance, gpointer klass );
+static void       instance_constructed( GObject *dialog );
+static void       instance_dispose( GObject *dialog );
+static void       instance_finalize( GObject *dialog );
 
-static void     on_base_initialize_gtk_toplevel( NactPreferencesEditor *editor, GtkDialog *toplevel );
-static void     on_base_initialize_base_window( NactPreferencesEditor *editor );
-static void     on_base_all_widgets_showed( NactPreferencesEditor *editor );
-static void     order_mode_setup( NactPreferencesEditor *editor );
-static void     order_mode_on_alpha_asc_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
-static void     order_mode_on_alpha_desc_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
-static void     order_mode_on_manual_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
-static void     order_mode_on_toggled( NactPreferencesEditor *editor, GtkToggleButton *togglebutton, GCallback cb, guint order_mode );
-static void     root_menu_setup( NactPreferencesEditor *editor );
-static void     root_menu_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
-static void     about_item_setup( NactPreferencesEditor *editor );
-static void     about_item_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
-static void     terminal_pattern_setup( NactPreferencesEditor *editor );
-static void     terminal_pattern_on_changed( GtkEntry *entry, NactPreferencesEditor *editor );
-static void     desktop_create_model( NactPreferencesEditor *editor );
-static void     desktop_setup( NactPreferencesEditor *editor );
-static void     desktop_on_changed( GtkComboBox *combo, NactPreferencesEditor *editor );
-static void     relabel_menu_setup( NactPreferencesEditor *editor );
-static void     relabel_menu_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
-static void     relabel_action_setup( NactPreferencesEditor *editor );
-static void     relabel_action_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
-static void     relabel_profile_setup( NactPreferencesEditor *editor );
-static void     relabel_profile_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
-static void     esc_quit_setup( NactPreferencesEditor *editor );
-static void     esc_quit_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
-static void     esc_confirm_setup( NactPreferencesEditor *editor );
-static void     esc_confirm_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
-static void     auto_save_setup( NactPreferencesEditor *editor );
-static void     auto_save_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
-static void     auto_save_period_on_change_value( GtkSpinButton *spinbutton, NactPreferencesEditor *editor );
-static void     import_mode_setup( NactPreferencesEditor *editor );
-static void     import_mode_on_ask_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
-static void     import_mode_on_override_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
-static void     import_mode_on_renumber_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
-static void     import_mode_on_noimport_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
-static void     import_mode_on_toggled( NactPreferencesEditor *editor, GtkToggleButton *togglebutton, GCallback cb, guint import_mode );
-static void     on_cancel_clicked( GtkButton *button, NactPreferencesEditor *editor );
-static void     on_ok_clicked( GtkButton *button, NactPreferencesEditor *editor );
-static void     on_dialog_ok( BaseDialog *dialog );
+static void       on_base_initialize_gtk( NactPreferencesEditor *editor, GtkDialog *toplevel, gpointer user_data );
+static void       on_base_initialize_window( NactPreferencesEditor *editor, gpointer user_data );
+static void       on_base_show_widgets( NactPreferencesEditor *editor, gpointer user_data );
+static void       order_mode_setup( NactPreferencesEditor *editor );
+static void       order_mode_on_alpha_asc_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
+static void       order_mode_on_alpha_desc_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
+static void       order_mode_on_manual_toggled( GtkToggleButton *togglebutton, NactPreferencesEditor *editor );
+static void       order_mode_on_toggled( NactPreferencesEditor *editor, GtkToggleButton *togglebutton, GCallback cb, guint order_mode );
+static void       root_menu_setup( NactPreferencesEditor *editor );
+static void       root_menu_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
+static void       about_item_setup( NactPreferencesEditor *editor );
+static void       about_item_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
+static void       terminal_pattern_setup( NactPreferencesEditor *editor );
+static void       terminal_pattern_on_changed( GtkEntry *entry, NactPreferencesEditor *editor );
+static void       desktop_create_model( NactPreferencesEditor *editor );
+static void       desktop_setup( NactPreferencesEditor *editor );
+static void       desktop_on_changed( GtkComboBox *combo, NactPreferencesEditor *editor );
+static void       relabel_menu_setup( NactPreferencesEditor *editor );
+static void       relabel_menu_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
+static void       relabel_action_setup( NactPreferencesEditor *editor );
+static void       relabel_action_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
+static void       relabel_profile_setup( NactPreferencesEditor *editor );
+static void       relabel_profile_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
+static void       esc_quit_setup( NactPreferencesEditor *editor );
+static void       esc_quit_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
+static void       esc_confirm_setup( NactPreferencesEditor *editor );
+static void       esc_confirm_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
+static void       auto_save_setup( NactPreferencesEditor *editor );
+static void       auto_save_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor );
+static void       auto_save_period_on_change_value( GtkSpinButton *spinbutton, NactPreferencesEditor *editor );
+static void       on_cancel_clicked( GtkButton *button, NactPreferencesEditor *editor );
+static void       on_ok_clicked( GtkButton *button, NactPreferencesEditor *editor );
+static void       on_dialog_ok( BaseDialog *dialog );
 
 GType
 nact_preferences_editor_get_type( void )
@@ -196,9 +200,17 @@ register_type( void )
 		( GInstanceInitFunc ) instance_init
 	};
 
+	static const GInterfaceInfo ioptions_list_iface_info = {
+		( GInterfaceInitFunc ) ioptions_list_iface_init,
+		NULL,
+		NULL
+	};
+
 	g_debug( "%s", thisfn );
 
-	type = g_type_register_static( BASE_DIALOG_TYPE, "NactPreferencesEditor", &info, 0 );
+	type = g_type_register_static( BASE_TYPE_DIALOG, "NactPreferencesEditor", &info, 0 );
+
+	g_type_add_interface_static( type, NA_TYPE_IOPTIONS_LIST, &ioptions_list_iface_info );
 
 	return( type );
 }
@@ -215,6 +227,7 @@ class_init( NactPreferencesEditorClass *klass )
 	st_parent_class = g_type_class_peek_parent( klass );
 
 	object_class = G_OBJECT_CLASS( klass );
+	object_class->constructed = instance_constructed;
 	object_class->dispose = instance_dispose;
 	object_class->finalize = instance_finalize;
 
@@ -224,6 +237,97 @@ class_init( NactPreferencesEditorClass *klass )
 	dialog_class->ok = on_dialog_ok;
 }
 
+static void
+ioptions_list_iface_init( NAIOptionsListInterface *iface, void *user_data )
+{
+	static const gchar *thisfn = "nact_assistant_export_ioptions_list_iface_init";
+
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
+
+	iface->get_options = ioptions_list_get_options;
+	iface->free_options = ioptions_list_free_options;
+	iface->get_ask_option = ioptions_list_get_ask_option;
+}
+
+/*
+ * ioptions_list_get_options, ioptions_list_free_options:
+ * manages import mode options or export format options depending of the
+ * current container
+ */
+static GList *
+ioptions_list_get_options( const NAIOptionsList *instance, GtkWidget *container )
+{
+	static const gchar *thisfn = "nact_preferences_editor_ioptions_list_get_options";
+	GList *options;
+	NactApplication *application;
+	NAUpdater *updater;
+
+	g_return_val_if_fail( NACT_IS_PREFERENCES_EDITOR( instance ), NULL );
+
+	options = NULL;
+
+	if( container == base_window_get_widget( BASE_WINDOW( instance ), "PreferencesExportFormatVBox" )){
+		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( instance )));
+		updater = nact_application_get_updater( application );
+		options = na_exporter_get_formats( NA_PIVOT( updater ));
+
+	} else if( container == base_window_get_widget( BASE_WINDOW( instance ), "PreferencesImportModeVBox" )){
+		options = na_importer_get_modes();
+
+	} else {
+		g_warning( "%s: container=%p (%s): unmanaged container",
+				thisfn,
+				( void * ) container, G_OBJECT_TYPE_NAME( container ));
+	}
+
+	return( options );
+}
+
+static void
+ioptions_list_free_options( const NAIOptionsList *instance, GtkWidget *container, GList *options )
+{
+	static const gchar *thisfn = "nact_preferences_editor_ioptions_list_free_options";
+
+	g_return_if_fail( NACT_IS_PREFERENCES_EDITOR( instance ));
+
+	if( container == base_window_get_widget( BASE_WINDOW( instance ), "PreferencesExportFormatVBox" )){
+		na_exporter_free_formats( options );
+
+	} else if( container == base_window_get_widget( BASE_WINDOW( instance ), "PreferencesImportModeVBox" )){
+		na_importer_free_modes( options );
+
+	} else {
+		g_warning( "%s: container=%p (%s): unmanaged container",
+				thisfn,
+				( void * ) container, G_OBJECT_TYPE_NAME( container ));
+	}
+}
+
+static NAIOption *
+ioptions_list_get_ask_option( const NAIOptionsList *instance, GtkWidget *container )
+{
+	static const gchar *thisfn = "nact_preferences_editor_ioptions_list_get_ask_option";
+	NAIOption *option;
+
+	g_return_val_if_fail( NACT_IS_PREFERENCES_EDITOR( instance ), NULL );
+
+	option = NULL;
+
+	if( container == base_window_get_widget( BASE_WINDOW( instance ), "PreferencesExportFormatVBox" )){
+		option = na_exporter_get_ask_option();
+
+	} else if( container == base_window_get_widget( BASE_WINDOW( instance ), "PreferencesImportModeVBox" )){
+		option = na_importer_get_ask_mode();
+
+	} else {
+		g_warning( "%s: container=%p (%s): unmanaged container",
+				thisfn,
+				( void * ) container, G_OBJECT_TYPE_NAME( container ));
+	}
+
+	return( option );
+}
+
 static void
 instance_init( GTypeInstance *instance, gpointer klass )
 {
@@ -238,18 +342,48 @@ instance_init( GTypeInstance *instance, gpointer klass )
 
 	self->private = g_new0( NactPreferencesEditorPrivate, 1 );
 
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_INITIALIZE_GTK, G_CALLBACK( on_base_initialize_gtk_toplevel ));
-
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_INITIALIZE_WINDOW, G_CALLBACK( on_base_initialize_base_window ));
-
-	base_window_signal_connect( BASE_WINDOW( instance ),
-			G_OBJECT( instance ), BASE_SIGNAL_ALL_WIDGETS_SHOWED, G_CALLBACK( on_base_all_widgets_showed));
-
 	self->private->dispose_has_run = FALSE;
 }
 
+static void
+instance_constructed( GObject *dialog )
+{
+	static const gchar *thisfn = "nact_preferences_editor_instance_constructed";
+	NactPreferencesEditorPrivate *priv;
+
+	g_return_if_fail( NACT_IS_PREFERENCES_EDITOR( dialog ));
+
+	priv = NACT_PREFERENCES_EDITOR( dialog )->private;
+
+	if( !priv->dispose_has_run ){
+
+		/* chain up to the parent class */
+		if( G_OBJECT_CLASS( st_parent_class )->constructed ){
+			G_OBJECT_CLASS( st_parent_class )->constructed( dialog );
+		}
+
+		g_debug( "%s: dialog=%p (%s)", thisfn, ( void * ) dialog, G_OBJECT_TYPE_NAME( dialog ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( dialog ),
+				G_OBJECT( dialog ),
+				BASE_SIGNAL_INITIALIZE_GTK,
+				G_CALLBACK( on_base_initialize_gtk ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( dialog ),
+				G_OBJECT( dialog ),
+				BASE_SIGNAL_INITIALIZE_WINDOW,
+				G_CALLBACK( on_base_initialize_window ));
+
+		base_window_signal_connect(
+				BASE_WINDOW( dialog ),
+				G_OBJECT( dialog ),
+				BASE_SIGNAL_SHOW_WIDGETS,
+				G_CALLBACK( on_base_show_widgets ));
+	}
+}
+
 static void
 instance_dispose( GObject *dialog )
 {
@@ -314,11 +448,17 @@ nact_preferences_editor_run( BaseWindow *parent )
 
 	g_debug( "%s: parent=%p (%s)", thisfn, ( void * ) parent, G_OBJECT_TYPE_NAME( parent ));
 
-	editor = g_object_new( NACT_PREFERENCES_EDITOR_TYPE,
-					BASE_PROP_PARENT,         parent,
-					BASE_PROP_XMLUI_FILENAME, st_xmlui_filename,
-					BASE_PROP_TOPLEVEL_NAME,  st_toplevel_name,
-					BASE_PROP_WSP_NAME,       st_wsp_name,
+	editor = g_object_new( NACT_TYPE_PREFERENCES_EDITOR,
+					BASE_PROP_PARENT,          parent,
+					BASE_PROP_XMLUI_FILENAME,  st_xmlui_filename,
+					/*
+					 * having our own builder let us, e.g., set a weak reference on
+					 * pixbufs allocated by plugins - but this way we are losing
+					 * mutualization of gtk initializations...
+					 */
+					/*BASE_PROP_HAS_OWN_BUILDER, TRUE,*/
+					BASE_PROP_TOPLEVEL_NAME,   st_toplevel_name,
+					BASE_PROP_WSP_NAME,        st_wsp_name,
 					NULL );
 
 	are_locked = na_settings_get_boolean( NA_IPREFS_ADMIN_PREFERENCES_LOCKED, NULL, &mandatory );
@@ -335,28 +475,26 @@ nact_preferences_editor_run( BaseWindow *parent )
 }
 
 static void
-on_base_initialize_gtk_toplevel( NactPreferencesEditor *editor, GtkDialog *toplevel )
+on_base_initialize_gtk( NactPreferencesEditor *editor, GtkDialog *toplevel, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_preferences_editor_on_base_initialize_gtk_toplevel";
-	NactApplication *application;
-	NAUpdater *updater;
+	static const gchar *thisfn = "nact_preferences_editor_on_base_initialize_gtk";
 	GtkWidget *container;
 	GtkTreeView *listview;
 
 	g_return_if_fail( NACT_IS_PREFERENCES_EDITOR( editor ));
 
 	if( !editor->private->dispose_has_run ){
-		g_debug( "%s: editor=%p, toplevel=%p", thisfn, ( void * ) editor, ( void * ) toplevel );
 
-		application = NACT_APPLICATION( base_window_get_application( BASE_WINDOW( editor )));
-		updater = nact_application_get_updater( application );
+		g_debug( "%s: dialog=%p, toplevel=%p, user_data=%p",
+				thisfn, ( void * ) editor, ( void * ) toplevel, ( void * ) user_data );
 
 		desktop_create_model( editor );
 
+		container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesImportModeVBox" );
+		na_ioptions_list_gtk_init( NA_IOPTIONS_LIST( editor ), container, TRUE );
+
 		container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesExportFormatVBox" );
-		nact_export_format_init_display(
-				container, NA_PIVOT( updater ),
-				EXPORT_FORMAT_DISPLAY_PREFERENCES, !editor->private->preferences_locked );
+		na_ioptions_list_gtk_init( NA_IOPTIONS_LIST( editor ), container, TRUE );
 
 		listview = GTK_TREE_VIEW( base_window_get_widget( BASE_WINDOW( editor ), "SchemesTreeView" ));
 		nact_schemes_list_create_model( listview, SCHEMES_LIST_FOR_PREFERENCES );
@@ -371,18 +509,20 @@ on_base_initialize_gtk_toplevel( NactPreferencesEditor *editor, GtkDialog *tople
 }
 
 static void
-on_base_initialize_base_window( NactPreferencesEditor *editor )
+on_base_initialize_window( NactPreferencesEditor *editor, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_preferences_editor_on_base_initialize_base_window";
+	static const gchar *thisfn = "nact_preferences_editor_on_base_initialize_window";
 	GtkWidget *container;
-	GQuark export_format;
 	GtkTreeView *listview;
 	GtkWidget *ok_button;
+	gchar *export_format;
+	gchar *import_mode;
 
 	g_return_if_fail( NACT_IS_PREFERENCES_EDITOR( editor ));
 
 	if( !editor->private->dispose_has_run ){
-		g_debug( "%s: editor=%p", thisfn, ( void * ) editor );
+
+		g_debug( "%s: dialog=%p, user_data=%p", thisfn, ( void * ) editor, ( void * ) user_data );
 
 		/* first tab: runtime preferences
 		 */
@@ -406,13 +546,27 @@ on_base_initialize_base_window( NactPreferencesEditor *editor )
 
 		/* fourth tab: import mode
 		 */
-		import_mode_setup( editor );
+		container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesImportModeVBox" );
+		import_mode = na_settings_get_string( NA_IPREFS_IMPORT_PREFERRED_MODE, NULL, &editor->private->import_mode_mandatory );
+		na_ioptions_list_set_editable(
+				NA_IOPTIONS_LIST( editor ), container,
+				!editor->private->import_mode_mandatory && !editor->private->preferences_locked );
+		na_ioptions_list_set_default(
+				NA_IOPTIONS_LIST( editor ), container,
+				import_mode );
+		g_free( import_mode );
 
 		/* fifth tab: export format
 		 */
-		export_format = na_iprefs_get_export_format( NA_IPREFS_EXPORT_PREFERRED_FORMAT, &editor->private->export_format_mandatory );
 		container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesExportFormatVBox" );
-		nact_export_format_select( container, !editor->private->export_format_mandatory, export_format );
+		export_format = na_settings_get_string( NA_IPREFS_EXPORT_PREFERRED_FORMAT, NULL, &editor->private->export_format_mandatory );
+		na_ioptions_list_set_editable(
+				NA_IOPTIONS_LIST( editor ), container,
+				!editor->private->export_format_mandatory && !editor->private->preferences_locked );
+		na_ioptions_list_set_default(
+				NA_IOPTIONS_LIST( editor ), container,
+				export_format );
+		g_free( export_format );
 
 		/* sixth tab: default schemes
 		 */
@@ -437,15 +591,16 @@ on_base_initialize_base_window( NactPreferencesEditor *editor )
 }
 
 static void
-on_base_all_widgets_showed( NactPreferencesEditor *editor )
+on_base_show_widgets( NactPreferencesEditor *editor, gpointer user_data )
 {
-	static const gchar *thisfn = "nact_preferences_editor_on_all_widgets_showed";
+	static const gchar *thisfn = "nact_preferences_editor_on_base_show_widgets";
 	GtkNotebook *notebook;
 
 	g_return_if_fail( NACT_IS_PREFERENCES_EDITOR( editor ));
 
 	if( !editor->private->dispose_has_run ){
-		g_debug( "%s: editor=%p", thisfn, ( void * ) editor );
+
+		g_debug( "%s: dialog=%p, user_data=%p", thisfn, ( void * ) editor, ( void * ) user_data );
 
 		notebook = GTK_NOTEBOOK( base_window_get_widget( BASE_WINDOW( editor ), "PreferencesNotebook" ));
 		gtk_notebook_set_current_page( notebook, st_last_tab );
@@ -529,7 +684,7 @@ order_mode_on_toggled( NactPreferencesEditor *editor, GtkToggleButton *toggle_bu
 	gboolean editable;
 	gboolean active;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( toggle_button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( toggle_button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		active = gtk_toggle_button_get_active( toggle_button );
@@ -562,7 +717,7 @@ root_menu_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor )
 {
 	gboolean editable;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->root_menu = gtk_toggle_button_get_active( button );
@@ -593,7 +748,7 @@ about_item_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor )
 {
 	gboolean editable;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->about_item = gtk_toggle_button_get_active( button );
@@ -687,14 +842,18 @@ desktop_create_model( NactPreferencesEditor *editor )
 	gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( combo ), text_cell, "text", 0, NULL );
 	gtk_cell_renderer_set_visible( GTK_CELL_RENDERER( text_cell ), TRUE );
 
+	/* insert the "rely on runtime detection item"
+	 * this item must be inserted first (see desktop_setup())
+	 */
 	gtk_list_store_append( model, &row );
 	gtk_list_store_set( model, &row,
 			DESKTOP_ID_COLUMN, st_no_desktop.id,
 			DESKTOP_LABEL_COLUMN, gettext( st_no_desktop.label ),
 			-1 );
 
+	/* insert list of known desktops
+	 */
 	desktops = na_desktop_environment_get_known_list();
-
 	for( i = 0 ; desktops[i].id ; ++i ){
 		gtk_list_store_append( model, &row );
 		gtk_list_store_set( model, &row,
@@ -711,11 +870,12 @@ desktop_setup( NactPreferencesEditor *editor )
 	const NADesktopEnv *desktops;
 	guint i;
 	gint found;
+	GtkWidget *widget;
+	const gchar *desktop_id;
+	const gchar *desktop_label;
 
-	editor->private->desktop = na_settings_get_string( NA_IPREFS_DESKTOP_ENVIRONMENT, NULL, &editor->private->desktop_mandatory );
-
-	combo = base_window_get_widget( BASE_WINDOW( editor ), "DesktopComboBox" );
 	found = -1;
+	editor->private->desktop = na_settings_get_string( NA_IPREFS_DESKTOP_ENVIRONMENT, NULL, &editor->private->desktop_mandatory );
 
 	if( editor->private->desktop && strlen( editor->private->desktop )){
 		desktops = na_desktop_environment_get_known_list();
@@ -726,10 +886,27 @@ desktop_setup( NactPreferencesEditor *editor )
 		}
 	}
 
+	/* if the user has not selected a "preferred" desktop,
+	 * we suppose he relies on runtime detection
+	 */
+	if( found == -1 ){
+		found = 0;
+	}
+
+	combo = base_window_get_widget( BASE_WINDOW( editor ), "DesktopComboBox" );
 	gtk_combo_box_set_active( GTK_COMBO_BOX( combo ), found );
 
-	base_window_signal_connect( BASE_WINDOW( editor ),
-			G_OBJECT( combo ), "changed", G_CALLBACK( desktop_on_changed ));
+	base_window_signal_connect(
+			BASE_WINDOW( editor ),
+			G_OBJECT( combo ), "changed",
+			G_CALLBACK( desktop_on_changed ));
+
+	/* set the currently detected desktop
+	 */
+	widget = base_window_get_widget( BASE_WINDOW( editor ), "DesktopLabel" );
+	desktop_id = na_desktop_environment_detect_running_desktop();
+	desktop_label = na_desktop_environment_get_label( desktop_id );
+	gtk_label_set_text( GTK_LABEL( widget ), desktop_label );
 }
 
 static void
@@ -774,7 +951,7 @@ relabel_menu_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor
 {
 	gboolean editable;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->relabel_menu = gtk_toggle_button_get_active( button );
@@ -805,7 +982,7 @@ relabel_action_on_toggled( GtkToggleButton *button, NactPreferencesEditor *edito
 {
 	gboolean editable;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->relabel_action = gtk_toggle_button_get_active( button );
@@ -836,7 +1013,7 @@ relabel_profile_on_toggled( GtkToggleButton *button, NactPreferencesEditor *edit
 {
 	gboolean editable;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->relabel_profile = gtk_toggle_button_get_active( button );
@@ -868,7 +1045,7 @@ esc_quit_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor )
 	gboolean editable;
 	GtkWidget *confirm_button;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->esc_quit = gtk_toggle_button_get_active( button );
@@ -902,7 +1079,7 @@ esc_confirm_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor )
 {
 	gboolean editable;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->esc_confirm = gtk_toggle_button_get_active( button );
@@ -949,7 +1126,7 @@ auto_save_on_toggled( GtkToggleButton *button, NactPreferencesEditor *editor )
 	GtkWidget *widget;
 	gboolean sensitive;
 
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NACT_PROP_TOGGLE_EDITABLE ));
+	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( button ), NA_TOGGLE_DATA_EDITABLE ));
 
 	if( editable ){
 		editor->private->auto_save = gtk_toggle_button_get_active( button );
@@ -977,100 +1154,6 @@ auto_save_period_on_change_value( GtkSpinButton *spinbutton, NactPreferencesEdit
 	editor->private->auto_save_period = gtk_spin_button_get_value_as_int( spinbutton );
 }
 
-/*
- * preferred import mode
- */
-static void
-import_mode_setup( NactPreferencesEditor *editor )
-{
-	gboolean editable;
-	GtkWidget *ask_button, *override_button, *renumber_button, *noimport_button;
-	GtkWidget *active_button;
-	GCallback active_handler;
-
-	editor->private->import_mode = na_iprefs_get_import_mode(
-			NA_IPREFS_IMPORT_PREFERRED_MODE, &editor->private->import_mode_mandatory );
-	editable = !editor->private->preferences_locked && !editor->private->import_mode_mandatory;
-
-	ask_button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsAskButton" );
-	base_window_signal_connect( BASE_WINDOW( editor ), G_OBJECT( ask_button ), "toggled", G_CALLBACK( import_mode_on_ask_toggled ));
-
-	override_button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsOverrideButton" );
-	base_window_signal_connect( BASE_WINDOW( editor ), G_OBJECT( override_button ), "toggled", G_CALLBACK( import_mode_on_override_toggled ));
-
-	renumber_button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsRenumberButton" );
-	base_window_signal_connect( BASE_WINDOW( editor ), G_OBJECT( renumber_button ), "toggled", G_CALLBACK( import_mode_on_renumber_toggled ));
-
-	noimport_button = base_window_get_widget( BASE_WINDOW( editor ), "PrefsNoImportButton" );
-	base_window_signal_connect( BASE_WINDOW( editor ), G_OBJECT( noimport_button ), "toggled", G_CALLBACK( import_mode_on_noimport_toggled ));
-
-	switch( editor->private->import_mode ){
-		case IMPORTER_MODE_ASK:
-			active_button = ask_button;
-			active_handler = G_CALLBACK( import_mode_on_ask_toggled );
-			break;
-		case IMPORTER_MODE_OVERRIDE:
-			active_button = override_button;
-			active_handler = G_CALLBACK( import_mode_on_override_toggled );
-			break;
-		case IMPORTER_MODE_RENUMBER:
-			active_button = renumber_button;
-			active_handler = G_CALLBACK( import_mode_on_renumber_toggled );
-			break;
-		case IMPORTER_MODE_NO_IMPORT:
-		default:
-			active_button = noimport_button;
-			active_handler = G_CALLBACK( import_mode_on_noimport_toggled );
-			break;
-	}
-
-	base_gtk_utils_radio_set_initial_state(
-			GTK_RADIO_BUTTON( active_button ),
-			active_handler, editor, editable, !editor->private->preferences_locked );
-}
-
-static void
-import_mode_on_ask_toggled( GtkToggleButton *toggle_button, NactPreferencesEditor *editor )
-{
-	import_mode_on_toggled( editor, toggle_button, G_CALLBACK( import_mode_on_ask_toggled ), IMPORTER_MODE_ASK );
-}
-
-static void
-import_mode_on_override_toggled( GtkToggleButton *toggle_button, NactPreferencesEditor *editor )
-{
-	import_mode_on_toggled( editor, toggle_button, G_CALLBACK( import_mode_on_override_toggled ), IMPORTER_MODE_OVERRIDE );
-}
-
-static void
-import_mode_on_renumber_toggled( GtkToggleButton *toggle_button, NactPreferencesEditor *editor )
-{
-	import_mode_on_toggled( editor, toggle_button, G_CALLBACK( import_mode_on_renumber_toggled ), IMPORTER_MODE_RENUMBER );
-}
-
-static void
-import_mode_on_noimport_toggled( GtkToggleButton *toggle_button, NactPreferencesEditor *editor )
-{
-	import_mode_on_toggled( editor, toggle_button, G_CALLBACK( import_mode_on_noimport_toggled ), IMPORTER_MODE_NO_IMPORT );
-}
-
-static void
-import_mode_on_toggled( NactPreferencesEditor *editor, GtkToggleButton *toggle_button, GCallback cb, guint import_mode )
-{
-	gboolean editable;
-	gboolean active;
-
-	editable = ( gboolean ) GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( toggle_button ), NACT_PROP_TOGGLE_EDITABLE ));
-
-	if( editable ){
-		active = gtk_toggle_button_get_active( toggle_button );
-		if( active ){
-			editor->private->import_mode = import_mode;
-		}
-	} else {
-		base_gtk_utils_radio_reset_initial_state( GTK_RADIO_BUTTON( toggle_button ), cb );
-	}
-}
-
 static void
 on_cancel_clicked( GtkButton *button, NactPreferencesEditor *editor )
 {
@@ -1090,7 +1173,9 @@ on_dialog_ok( BaseDialog *dialog )
 {
 	NactPreferencesEditor *editor;
 	GtkWidget *container;
-	NAExportFormat *export_format;
+	NAIOption *option;
+	gchar *import_mode;
+	gchar *export_format;
 
 	g_return_if_fail( NACT_IS_PREFERENCES_EDITOR( dialog ));
 
@@ -1146,15 +1231,24 @@ on_dialog_ok( BaseDialog *dialog )
 		/* fourth tab: import mode
 		 */
 		if( !editor->private->import_mode_mandatory ){
-			na_iprefs_set_import_mode( NA_IPREFS_IMPORT_PREFERRED_MODE, editor->private->import_mode );
+			container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesImportModeVBox" );
+			option = na_ioptions_list_get_selected( NA_IOPTIONS_LIST( editor ), container );
+			g_return_if_fail( NA_IS_IMPORT_MODE( option ));
+			import_mode = na_ioption_get_id( option );
+			na_settings_set_string( NA_IPREFS_IMPORT_PREFERRED_MODE, import_mode );
+			g_free( import_mode );
 		}
 
 		/* fifth tab: export format
 		 */
 		if( !editor->private->export_format_mandatory ){
 			container = base_window_get_widget( BASE_WINDOW( editor ), "PreferencesExportFormatVBox" );
-			export_format = nact_export_format_get_selected( container );
-			na_iprefs_set_export_format( NA_IPREFS_EXPORT_PREFERRED_FORMAT, na_export_format_get_quark( export_format ));
+			option = na_ioptions_list_get_selected( NA_IOPTIONS_LIST( editor ), container );
+			g_debug( "nact_preferences_editor_on_dialog_ok: option=%p", ( void * ) option );
+			g_return_if_fail( NA_IS_EXPORT_FORMAT( option ));
+			export_format = na_ioption_get_id( option );
+			na_settings_set_string( NA_IPREFS_EXPORT_PREFERRED_FORMAT, export_format );
+			g_free( export_format );
 		}
 
 		/* sixth tab: list of default schemes
