@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -37,8 +37,6 @@
 #include <api/na-core-utils.h>
 #include <api/na-object-api.h>
 
-#include <core/na-iprefs.h>
-
 #include "base-keysyms.h"
 #include "nact-application.h"
 #include "base-gtk-utils.h"
@@ -430,26 +428,10 @@ nact_schemes_list_show_all( BaseWindow *window )
 	data = get_schemes_list_data( listview );
 
 	button = get_add_button( window );
-#if GTK_CHECK_VERSION( 2, 18, 0 )
 	gtk_widget_set_visible( GTK_WIDGET( button ), data->mode == SCHEMES_LIST_FOR_PREFERENCES );
-#else
-	if( data->mode == SCHEMES_LIST_FOR_PREFERENCES ){
-		gtk_widget_show( GTK_WIDGET( button ));
-	} else {
-		gtk_widget_hide( GTK_WIDGET( button ));
-	}
-#endif
 
 	button = get_remove_button( window );
-#if GTK_CHECK_VERSION( 2, 18, 0 )
 	gtk_widget_set_visible( GTK_WIDGET( button ), data->mode == SCHEMES_LIST_FOR_PREFERENCES );
-#else
-	if( data->mode == SCHEMES_LIST_FOR_PREFERENCES ){
-		gtk_widget_show( GTK_WIDGET( button ));
-	} else {
-		gtk_widget_hide( GTK_WIDGET( button ));
-	}
-#endif
 
 	init_view_select_first_row( data );
 }
