@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009, 2010, 2011 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010, 2011, 2012 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -39,8 +39,9 @@
 #include <core/na-gtk-utils.h>
 
 #include "base-keysyms.h"
-#include "base-marshal.h"
 #include "nact-application.h"
+#include "nact-main-window.h"
+#include "nact-marshal.h"
 #include "nact-tree-view.h"
 #include "nact-tree-model.h"
 #include "nact-tree-ieditable.h"
@@ -54,36 +55,19 @@ struct _NactTreeViewClassPrivate {
 /* private instance data
  */
 struct _NactTreeViewPrivate {
-	gboolean      dispose_has_run;
+	gboolean       dispose_has_run;
 
 	/* properties set at instanciation time
 	 */
-	BaseWindow   *window;
-	GtkContainer *parent;
-	gchar        *widget_name;
-	guint         mode;
+	BaseWindow    *window;
+	GtkContainer  *parent;
+	gchar         *widget_name;
+	guint          mode;
+	gboolean       notify_allowed;
 
 	/* runtime data
-	 *
-	 * Rationale:
-	 *
-	 * In Gtk2, we used to nact_tree_view_new() from
-	 * nact_main_window:on_instance_constructed().
-	 * This let us connect to on-base-initialize-gtk signal before it was
-	 * emitted, and so initialize the treeview on the handler.
-	 *
-	 * With Gtk3, we have to wait until the Gtk hierarchy of NactMainWindow
-	 * be actually built in order to be able to access to the embedded
-	 * GtkTreeView. As a consequence, nact_tree_view_new() has to be delayed
-	 * until nact_main_window_on_initialize_gtk().
-	 *
-	 * And so we no more can rely on the on-base-initialize-gtk signal to
-	 * initialize the view. So force a call to the handler from
-	 * on_base_initialize_view() handler.
 	 */
-	GtkTreeView  *tree_view;
-	gboolean      notify_allowed;
-	gboolean      gtk_initialized;
+	GtkTreeView   *tree_view;
 };
 
 /* instance properties
@@ -103,7 +87,6 @@ enum {
 /* signals
  */
 enum {
-	CONTEXT_MENU,
 	COUNT_CHANGED,
 	FOCUS_IN,
 	FOCUS_OUT,
@@ -133,7 +116,7 @@ static GObjectClass *st_parent_class           = NULL;
 
 static GType      register_type( void );
 static void       class_init( NactTreeViewClass *klass );
-static void       tree_ieditable_iface_init( NactTreeIEditableInterface *iface );
+static void       tree_ieditable_iface_init( NactTreeIEditableInterface *iface, void *user_data );
 static void       instance_init( GTypeInstance *instance, gpointer klass );
 static void       instance_get_property( GObject *object, guint property_id, GValue *value, GParamSpec *spec );
 static void       instance_set_property( GObject *object, guint property_id, const GValue *value, GParamSpec *spec );
@@ -141,15 +124,15 @@ static void       instance_constructed( GObject *object );
 static void       instance_dispose( GObject *application );
 static void       instance_finalize( GObject *application );
 
-static void       on_base_initialize_gtk( BaseWindow *window, GtkWindow *toplevel, gpointer user_data );
+static void       initialize_gtk( NactTreeView *view );
 static void       on_base_initialize_view( BaseWindow *window, gpointer user_data );
 static void       on_base_all_widgets_showed( BaseWindow *window, gpointer user_data );
 static gboolean   on_button_press_event( GtkWidget *widget, GdkEventButton *event, BaseWindow *window );
 static gboolean   on_focus_in( GtkWidget *widget, GdkEventFocus *event, BaseWindow *window );
 static gboolean   on_focus_out( GtkWidget *widget, GdkEventFocus *event, BaseWindow *window );
 static gboolean   on_key_pressed_event( GtkWidget *widget, GdkEventKey *event, BaseWindow *window );
+static gboolean   on_popup_menu( GtkWidget *widget, BaseWindow *window );
 static void       on_selection_changed( GtkTreeSelection *selection, BaseWindow *window );
-static void       on_selection_changed_cleanup_handler( BaseWindow *window, GList *selected_items );
 static void       on_tree_view_realized( GtkWidget *treeview, BaseWindow *window );
 static void       clear_selection( NactTreeView *view );
 static void       display_label( GtkTreeViewColumn *column, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, NactTreeView *view );
@@ -159,7 +142,7 @@ static GtkWidget *get_tree_view( NactTreeView *items_view );
 static void       iter_on_selection( NactTreeView *view, FnIterOnSelection fn_iter, gpointer user_data );
 static void       navigate_to_child( NactTreeView *view );
 static void       navigate_to_parent( NactTreeView *view );
-static void       open_popup( BaseWindow *window, GdkEventButton *event );
+static void       do_open_popup( BaseWindow *window, GdkEventButton *event );
 static void       select_row_at_path_by_string( NactTreeView *view, const gchar *path );
 static void       toggle_collapse( NactTreeView *view );
 static gboolean   toggle_collapse_iter( NactTreeView *view, GtkTreeModel *model, GtkTreeIter *iter, NAObject *object, gpointer user_data );
@@ -170,8 +153,15 @@ nact_tree_view_get_type( void )
 {
 	static GType type = 0;
 
+	static const GInterfaceInfo tree_ieditable_iface_info = {
+		( GInterfaceInitFunc ) tree_ieditable_iface_init,
+		NULL,
+		NULL
+	};
+
 	if( !type ){
 		type = register_type();
+		g_type_add_interface_static( type, NACT_TREE_IEDITABLE_TYPE, &tree_ieditable_iface_info );
 	}
 
 	return( type );
@@ -195,18 +185,10 @@ register_type( void )
 		( GInstanceInitFunc ) instance_init
 	};
 
-	static const GInterfaceInfo tree_ieditable_iface_info = {
-		( GInterfaceInitFunc ) tree_ieditable_iface_init,
-		NULL,
-		NULL
-	};
-
 	g_debug( "%s", thisfn );
 
 	type = g_type_register_static( G_TYPE_OBJECT, "NactTreeView", &info, 0 );
 
-	g_type_add_interface_static( type, NACT_TREE_IEDITABLE_TYPE, &tree_ieditable_iface_info );
-
 	return( type );
 }
 
@@ -277,30 +259,6 @@ class_init( NactTreeViewClass *klass )
 					FALSE,
 					G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE ));
 
-	/**
-	 * NactTreeView::tree-signal-open-popup
-	 *
-	 * This signal is emitted on the BaseWindow parent when the user right
-	 * clicks on the tree view.
-	 *
-	 * Signal args:
-	 * - the GdkEvent.
-	 *
-	 * Handler prototype:
-	 * void ( *handler )( BaseWindow *window, GdkEvent *event, gpointer user_data );
-	 */
-	st_signals[ CONTEXT_MENU ] = g_signal_new(
-			TREE_SIGNAL_CONTEXT_MENU,
-			G_TYPE_OBJECT,
-			G_SIGNAL_RUN_LAST,
-			0,
-			NULL,
-			NULL,
-			g_cclosure_marshal_VOID__POINTER,
-			G_TYPE_NONE,
-			1,
-			G_TYPE_POINTER );
-
 	/**
 	 * NactTreeView::tree-signal-count-changed:
 	 *
@@ -437,39 +395,15 @@ class_init( NactTreeViewClass *klass )
 			1,
 			G_TYPE_BOOLEAN );
 
-	/**
-	 * NactTreeView::tree-signal-selection-changed:
-	 *
-	 * This signal is emitted on the BaseWindow parent each time the selection
-	 * has changed in the treeview.
-	 *
-	 * Signal args:
-	 * - a #GList of currently selected #NAObjectItems.
-	 *
-	 * Handler prototype:
-	 * void ( *handler )( BaseWindow *window, GList *selected, gpointer user_data );
-	 */
-	st_signals[ SELECTION_CHANGED ] = g_signal_new_class_handler(
-			TREE_SIGNAL_SELECTION_CHANGED,
-			G_TYPE_OBJECT,
-			G_SIGNAL_RUN_CLEANUP,
-			G_CALLBACK( on_selection_changed_cleanup_handler ),
-			NULL,
-			NULL,
-			g_cclosure_marshal_VOID__POINTER,
-			G_TYPE_NONE,
-			1,
-			G_TYPE_POINTER );
-
 	klass->private = g_new0( NactTreeViewClassPrivate, 1 );
 }
 
 static void
-tree_ieditable_iface_init( NactTreeIEditableInterface *iface )
+tree_ieditable_iface_init( NactTreeIEditableInterface *iface, void *user_data )
 {
 	static const gchar *thisfn = "nact_main_window_tree_ieditable_iface_init";
 
-	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+	g_debug( "%s: iface=%p, user_data=%p", thisfn, ( void * ) iface, ( void * ) user_data );
 }
 
 static void
@@ -488,7 +422,6 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	self->private = g_new0( NactTreeViewPrivate, 1 );
 
 	self->private->dispose_has_run = FALSE;
-	self->private->gtk_initialized = FALSE;
 }
 
 static void
@@ -572,26 +505,36 @@ static void
 instance_constructed( GObject *object )
 {
 	static const gchar *thisfn = "nact_tree_view_instance_constructed";
-	NactTreeView *self;
+	NactTreeViewPrivate *priv;
 
 	g_return_if_fail( NACT_IS_TREE_VIEW( object ));
-	self = NACT_TREE_VIEW( object );
 
-	if( !self->private->dispose_has_run ){
-		g_debug( "%s: object=%p", thisfn, ( void * ) object );
+	priv = NACT_TREE_VIEW( object )->private;
 
-		base_window_signal_connect( self->private->window,
-				G_OBJECT( self->private->window ), BASE_SIGNAL_INITIALIZE_WINDOW, G_CALLBACK( on_base_initialize_view ));
-
-		base_window_signal_connect( self->private->window,
-				G_OBJECT( self->private->window ), BASE_SIGNAL_ALL_WIDGETS_SHOWED, G_CALLBACK( on_base_all_widgets_showed ));
-
-		g_object_set_data( G_OBJECT( self->private->window ), WINDOW_DATA_TREE_VIEW, self );
+	if( !priv->dispose_has_run ){
 
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->constructed ){
 			G_OBJECT_CLASS( st_parent_class )->constructed( object );
 		}
+
+		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
+		base_window_signal_connect(
+				priv->window,
+				G_OBJECT( priv->window ),
+				BASE_SIGNAL_INITIALIZE_WINDOW,
+				G_CALLBACK( on_base_initialize_view ));
+
+		base_window_signal_connect(
+				priv->window,
+				G_OBJECT( priv->window ),
+				BASE_SIGNAL_SHOW_WIDGETS,
+				G_CALLBACK( on_base_all_widgets_showed ));
+
+		g_object_set_data( G_OBJECT( priv->window ), WINDOW_DATA_TREE_VIEW, object );
+
+		initialize_gtk( NACT_TREE_VIEW( object ));
 	}
 }
 
@@ -600,8 +543,6 @@ instance_dispose( GObject *object )
 {
 	static const gchar *thisfn = "nact_tree_view_instance_dispose";
 	NactTreeView *self;
-	NactTreeModel *model;
-	GtkTreeStore *ts_model;
 
 	g_return_if_fail( NACT_IS_TREE_VIEW( object ));
 
@@ -612,11 +553,6 @@ instance_dispose( GObject *object )
 
 		self->private->dispose_has_run = TRUE;
 
-		model = NACT_TREE_MODEL( gtk_tree_view_get_model( self->private->tree_view ));
-		ts_model = GTK_TREE_STORE( gtk_tree_model_filter_get_model( GTK_TREE_MODEL_FILTER( model )));
-		gtk_tree_store_clear( ts_model );
-		g_debug( "%s: tree store cleared", thisfn );
-
 		if( self->private->mode == TREE_MODE_EDITION ){
 			nact_tree_ieditable_terminate( NACT_TREE_IEDITABLE( self ));
 		}
@@ -660,88 +596,87 @@ instance_finalize( GObject *instance )
  * Returns: a newly allocated NactTreeView object, which will be owned
  * by the caller. It is useless to unref it as it will automatically
  * auto-destroys itself at @window finalization.
+ *
+ * This function is called from NactMainWindow::on_base_initialize_gtk()
+ * signal handler, thus only once during the application life.
  */
 NactTreeView *
 nact_tree_view_new( BaseWindow *window, GtkContainer *parent, const gchar *treeview_name, NactTreeMode mode )
 {
 	NactTreeView *view;
 
-	view = g_object_new( NACT_TREE_VIEW_TYPE,
+	view = g_object_new( NACT_TYPE_TREE_VIEW,
 			TREE_PROP_WINDOW,      window,
 			TREE_PROP_PARENT,      parent,
 			TREE_PROP_WIDGET_NAME, treeview_name,
 			TREE_PROP_MODE,        mode,
 			NULL );
 
-#ifdef NA_MAINTAINER_MODE
-	na_gtk_utils_dump_children( parent );
-#endif
-
 	return( view );
 }
 
+/*
+ * called from instance_constructed()
+ *   from nact_tree_view_new()
+ *     from NactMainWindow::on_base_initialize_gtk() signal handler
+ */
 static void
-on_base_initialize_gtk( BaseWindow *window, GtkWindow *toplevel, gpointer user_data )
+initialize_gtk( NactTreeView *items_view )
 {
-	static const gchar *thisfn = "nact_tree_view_on_base_initialize_gtk";
-	NactTreeView *items_view;
+	static const gchar *thisfn = "nact_tree_view_initialize_gtk";
 	GtkTreeView *treeview;
+	NactTreeModel *model;
 	GtkWidget *label;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
 	GList *renderers;
 
-	items_view = NACT_TREE_VIEW( g_object_get_data( G_OBJECT( window ), WINDOW_DATA_TREE_VIEW ));
+	g_debug( "%s: items_view=%p", thisfn, ( void * ) items_view );
 
-	if( !items_view->private->dispose_has_run ){
-		g_debug( "%s: window=%p (%s), toplevel=%p (%s), user_data=%p",
-				thisfn, ( void * ) window, G_OBJECT_TYPE_NAME( window ),
-				( void * ) toplevel, G_OBJECT_TYPE_NAME( toplevel ), ( void * ) user_data );
+	treeview = GTK_TREE_VIEW( get_tree_view( items_view ));
+	model = nact_tree_model_new( items_view->private->window, treeview, items_view->private->mode );
+	gtk_tree_view_set_model( treeview, GTK_TREE_MODEL( model ));
+	g_object_unref( model );
+	g_debug( "%s: nact_tree_model_ref_count=%d", thisfn, G_OBJECT( model )->ref_count );
 
-		treeview = GTK_TREE_VIEW( get_tree_view( items_view ));
-		nact_tree_model_new( window, treeview, items_view->private->mode );
+	/* associates the ItemsView to the label */
+	label = na_gtk_utils_find_widget_by_name( items_view->private->parent, "ActionsListLabel" );
+	gtk_label_set_mnemonic_widget( GTK_LABEL( label ), GTK_WIDGET( treeview ));
 
-		/* associates the ItemsView to the label */
-		label = na_gtk_utils_search_for_child_widget( items_view->private->parent, "ActionsListLabel" );
-		gtk_label_set_mnemonic_widget( GTK_LABEL( label ), GTK_WIDGET( treeview ));
+	/* create visible columns on the tree view
+	 */
+	column = gtk_tree_view_column_new_with_attributes(
+			"icon",
+			gtk_cell_renderer_pixbuf_new(),
+			"pixbuf", TREE_COLUMN_ICON,
+			NULL );
+	gtk_tree_view_append_column( treeview, column );
 
-		/* create visible columns on the tree view
-		 */
-		column = gtk_tree_view_column_new_with_attributes(
-				"icon",
-				gtk_cell_renderer_pixbuf_new(),
-				"pixbuf", TREE_COLUMN_ICON,
-				NULL );
-		gtk_tree_view_append_column( treeview, column );
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes(
+			"label",
+			renderer,
+			"text", TREE_COLUMN_LABEL,
+			NULL );
+	gtk_tree_view_column_set_sort_column_id( column, TREE_COLUMN_LABEL );
+	gtk_tree_view_append_column( treeview, column );
 
-		renderer = gtk_cell_renderer_text_new();
-		column = gtk_tree_view_column_new_with_attributes(
-				"label",
-				renderer,
-				"text", TREE_COLUMN_LABEL,
-				NULL );
-		gtk_tree_view_column_set_sort_column_id( column, TREE_COLUMN_LABEL );
-		gtk_tree_view_append_column( treeview, column );
+	/* allow multiple selection
+	 */
+	selection = gtk_tree_view_get_selection( treeview );
+	gtk_tree_selection_set_mode( selection, GTK_SELECTION_MULTIPLE );
 
-		/* allow multiple selection
-		 */
-		selection = gtk_tree_view_get_selection( treeview );
-		gtk_tree_selection_set_mode( selection, GTK_SELECTION_MULTIPLE );
+	/* misc properties
+	 */
+	gtk_tree_view_set_enable_tree_lines( treeview, TRUE );
 
-		/* misc properties
-		 */
-		gtk_tree_view_set_enable_tree_lines( treeview, TRUE );
-
-		if( items_view->private->mode == TREE_MODE_EDITION ){
-			column = gtk_tree_view_get_column( treeview, TREE_COLUMN_LABEL );
-			renderers = gtk_cell_layout_get_cells( GTK_CELL_LAYOUT( column ));
-			renderer = GTK_CELL_RENDERER( renderers->data );
-			gtk_tree_view_column_set_cell_data_func(
-					column, renderer, ( GtkTreeCellDataFunc ) display_label, items_view, NULL );
-		}
-
-		items_view->private->gtk_initialized = TRUE;
+	if( items_view->private->mode == TREE_MODE_EDITION ){
+		column = gtk_tree_view_get_column( treeview, TREE_COLUMN_LABEL );
+		renderers = gtk_cell_layout_get_cells( GTK_CELL_LAYOUT( column ));
+		renderer = GTK_CELL_RENDERER( renderers->data );
+		gtk_tree_view_column_set_cell_data_func(
+				column, renderer, ( GtkTreeCellDataFunc ) display_label, items_view, NULL );
 	}
 }
 
@@ -755,26 +690,31 @@ on_base_initialize_view( BaseWindow *window, gpointer user_data )
 
 	items_view = NACT_TREE_VIEW( g_object_get_data( G_OBJECT( window ), WINDOW_DATA_TREE_VIEW ));
 
-	if( !items_view->private->gtk_initialized ){
-		on_base_initialize_gtk( window, base_window_get_gtk_toplevel( window ), NULL );
-	}
-
 	if( !items_view->private->dispose_has_run ){
+
 		g_debug( "%s: window=%p (%s), user_data=%p",
-				thisfn, ( void * ) window, G_OBJECT_TYPE_NAME( window ), ( void * ) user_data );
+				thisfn,
+				( void * ) window, G_OBJECT_TYPE_NAME( window ),
+				( void * ) user_data );
 
 		treeview = GTK_TREE_VIEW( get_tree_view( items_view ));
 		items_view->private->tree_view = treeview;
 
 		/* monitors the selection */
 		selection = gtk_tree_view_get_selection( treeview );
-		base_window_signal_connect( window,
-				G_OBJECT( selection ), "changed", G_CALLBACK( on_selection_changed ));
+		base_window_signal_connect(
+				window,
+				G_OBJECT( selection ),
+				"changed",
+				G_CALLBACK( on_selection_changed ));
 
 		/* delay all other signal connections until the widget be realized
 		 */
-		base_window_signal_connect( window,
-				G_OBJECT( treeview ), "realize", G_CALLBACK( on_tree_view_realized ));
+		base_window_signal_connect(
+				window,
+				G_OBJECT( treeview ),
+				"realize",
+				G_CALLBACK( on_tree_view_realized ));
 
 		if( items_view->private->mode == TREE_MODE_EDITION ){
 			nact_tree_ieditable_initialize( NACT_TREE_IEDITABLE( items_view ), treeview, window );
@@ -803,7 +743,7 @@ on_button_press_event( GtkWidget *widget, GdkEventButton *event, BaseWindow *win
 
 	/* single click on right button */
 	if( event->type == GDK_BUTTON_PRESS && event->button == 3 ){
-		open_popup( window, event );
+		do_open_popup( window, event );
 		stop = TRUE;
 	}
 
@@ -863,6 +803,15 @@ on_key_pressed_event( GtkWidget *widget, GdkEventKey *event, BaseWindow *window
 	return( stop );
 }
 
+/*
+ * triggered by the "popup-menu" signal, itself triggered by the keybindings
+ */
+static gboolean
+on_popup_menu( GtkWidget *widget, BaseWindow *window )
+{
+	do_open_popup( window, NULL );
+	return( TRUE );
+}
 /*
  * handles the "changed" signal emitted on the GtkTreeSelection
  */
@@ -880,45 +829,48 @@ on_selection_changed( GtkTreeSelection *selection, BaseWindow *window )
 			g_debug( "%s: selection=%p, window=%p", thisfn, ( void * ) selection, ( void * ) window );
 
 			selected_items = get_selected_items( items_view );
-			g_signal_emit_by_name( window, TREE_SIGNAL_SELECTION_CHANGED, selected_items );
+			g_signal_emit_by_name( window, MAIN_SIGNAL_SELECTION_CHANGED, selected_items );
 		}
 	}
 }
 
-/*
- * cleanup handler for our TREE_SIGNAL_SELECTION_CHANGED signal
- */
-static void
-on_selection_changed_cleanup_handler( BaseWindow *window, GList *selected_items )
-{
-	static const gchar *thisfn = "nact_tree_view_on_selection_changed_cleanup_handler";
-
-	g_debug( "%s: window=%p, selected_items=%p (count=%u)",
-			thisfn, ( void * ) window,
-			( void * ) selected_items, g_list_length( selected_items ));
-
-	na_object_free_items( selected_items );
-}
-
 static void
 on_tree_view_realized( GtkWidget *treeview, BaseWindow *window )
 {
 	g_debug( "nact_tree_view_on_tree_view_realized" );
 
 	/* expand/collapse with the keyboard */
-	base_window_signal_connect( window,
-			G_OBJECT( treeview ), "key-press-event", G_CALLBACK( on_key_pressed_event ));
+	base_window_signal_connect(
+			window,
+			G_OBJECT( treeview ),
+			"key-press-event",
+			G_CALLBACK( on_key_pressed_event ));
 
 	/* monitors whether the focus is on the view */
-	base_window_signal_connect( window,
-			G_OBJECT( treeview ), "focus-in-event", G_CALLBACK( on_focus_in ));
+	base_window_signal_connect(
+			window,
+			G_OBJECT( treeview ),
+			"focus-in-event",
+			G_CALLBACK( on_focus_in ));
 
-	base_window_signal_connect( window,
-			G_OBJECT( treeview ), "focus-out-event", G_CALLBACK( on_focus_out ));
+	base_window_signal_connect(
+			window,
+			G_OBJECT( treeview ),
+			"focus-out-event",
+			G_CALLBACK( on_focus_out ));
 
 	/* monitors mouse events */
-	base_window_signal_connect( window,
-			G_OBJECT( treeview ), "button-press-event", G_CALLBACK( on_button_press_event ));
+	base_window_signal_connect(
+			window,
+			G_OBJECT( treeview ),
+			"button-press-event",
+			G_CALLBACK( on_button_press_event ));
+
+	base_window_signal_connect(
+			window,
+			G_OBJECT( treeview ),
+			"popup-menu",
+			G_CALLBACK( on_popup_menu ));
 
 	/* force the treeview to have the focus at start
 	 * and select the first row if it exists
@@ -953,6 +905,7 @@ nact_tree_view_fill( NactTreeView *view, GList *items )
 		view->private->notify_allowed = FALSE;
 		model = NACT_TREE_MODEL( gtk_tree_view_get_model( view->private->tree_view ));
 		nact_tree_model_fill( model, items );
+		g_debug( "%s: nact_tree_model_ref_count=%d", thisfn, G_OBJECT( model )->ref_count );
 
 		view->private->notify_allowed = TRUE;
 		na_object_count_items( items, &nb_menus, &nb_actions, &nb_profiles );
@@ -1174,7 +1127,7 @@ nact_tree_view_select_row_at_path( NactTreeView *view, GtkTreePath *path )
 
 		if( !something ){
 			if( view->private->notify_allowed ){
-				g_signal_emit_by_name( view->private->window, TREE_SIGNAL_SELECTION_CHANGED, NULL );
+				g_signal_emit_by_name( view->private->window, MAIN_SIGNAL_SELECTION_CHANGED, NULL );
 			}
 		}
 	}
@@ -1198,8 +1151,6 @@ display_label( GtkTreeViewColumn *column, GtkCellRenderer *cell, GtkTreeModel *m
 {
 	NAObject *object;
 	gchar *label;
-	gboolean modified = FALSE;
-	gboolean valid = TRUE;
 
 	g_return_if_fail( view->private->mode == TREE_MODE_EDITION );
 
@@ -1213,13 +1164,11 @@ display_label( GtkTreeViewColumn *column, GtkCellRenderer *cell, GtkTreeModel *m
 		g_object_set( cell, "style-set", FALSE, NULL );
 		g_object_set( cell, "foreground-set", FALSE, NULL );
 
-		modified = na_object_is_modified( object );
-		if( modified ){
+		if( na_object_is_modified( object )){
 			g_object_set( cell, "style", PANGO_STYLE_ITALIC, "style-set", TRUE, NULL );
 		}
 
-		valid = na_object_is_valid( object );
-		if( !valid ){
+		if( !na_object_is_valid( object )){
 			g_object_set( cell, "foreground", "Red", "foreground-set", TRUE, NULL );
 		}
 
@@ -1279,9 +1228,11 @@ get_selected_items( NactTreeView *view )
 		path = ( GtkTreePath * ) it->data;
 		gtk_tree_model_get_iter( model, &iter, path );
 		gtk_tree_model_get( model, &iter, TREE_COLUMN_NAOBJECT, &object, -1 );
-		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
 		items = g_list_prepend( items, na_object_ref( object ));
 		g_object_unref( object );
+		g_debug( "%s: object=%p (%s) ref_count=%d",
+				thisfn,
+				( void * ) object, G_OBJECT_TYPE_NAME( object ), G_OBJECT( object )->ref_count );
 	}
 
 	g_list_foreach( listrows, ( GFunc ) gtk_tree_path_free, NULL );
@@ -1293,7 +1244,9 @@ get_selected_items( NactTreeView *view )
 static GtkWidget *
 get_tree_view( NactTreeView *items_view )
 {
-	return( na_gtk_utils_search_for_child_widget( items_view->private->parent, items_view->private->widget_name ));
+	return( na_gtk_utils_find_widget_by_name(
+			items_view->private->parent,
+			items_view->private->widget_name ));
 }
 
 static void
@@ -1414,19 +1367,21 @@ navigate_to_parent( NactTreeView *view )
 }
 
 static void
-open_popup( BaseWindow *window, GdkEventButton *event )
+do_open_popup( BaseWindow *window, GdkEventButton *event )
 {
 	NactTreeView *items_view;
 	GtkTreePath *path;
 
-	items_view = NACT_TREE_VIEW( g_object_get_data( G_OBJECT( window ), WINDOW_DATA_TREE_VIEW ));
+	if( event ){
+		items_view = NACT_TREE_VIEW( g_object_get_data( G_OBJECT( window ), WINDOW_DATA_TREE_VIEW ));
 
-	if( gtk_tree_view_get_path_at_pos( items_view->private->tree_view, event->x, event->y, &path, NULL, NULL, NULL )){
-		nact_tree_view_select_row_at_path( items_view, path );
-		gtk_tree_path_free( path );
+		if( gtk_tree_view_get_path_at_pos( items_view->private->tree_view, event->x, event->y, &path, NULL, NULL, NULL )){
+			nact_tree_view_select_row_at_path( items_view, path );
+			gtk_tree_path_free( path );
+		}
 	}
 
-	g_signal_emit_by_name( window, TREE_SIGNAL_CONTEXT_MENU, event );
+	g_signal_emit_by_name( window, MAIN_SIGNAL_CONTEXT_MENU, event, "/ui/TreeContext" );
 }
 
 /*
